// ********************************************************************
//
//  Routine:	alca/actaxxFill.C
//  Author:		Dmitry Golubkov
//
//  Description:	Fills TAHB, TAHD and TATR after hit selection,
//				called from somewhere else
//
//  Modifications:
//   -- none yet					06/12/00 DG
//   -- add ARTE_mark					14/12/00 DG
//   -- fill ITR hits, move output to actaxxOut		18/12/00 DG
//   -- fix precision loss of ITR resolution		18/01/01 DG
//   -- add VDS teplate					05/03/01 DG
//
// ********************************************************************

#include <iostream.h>
#include <math.h>
#include <stdio.h>
#include <stdlib.h>
#include <fstream.h>
#include <iomanip.h>
#include "arte/arte.hh"
#include "arte/RSEG_cmp.hh"
#include "arte/ArteUtil.h"
#include "rang/rgExtDef.hh"
#include "rang/zfextse.hh"
#include "rang/zfparext.hh"
#include "rang/Fit.hh"
#include "Otr/OtrI.h"

#include "Otr/OtrEvent.h"
#include "arte/ArteIOlun.h"


// ONLINE includes
#include "farm/farm.hh"
#include "table.h"
#include "esp.h"


#include "alca/actaxxFLG.hh"
#include "alca/actaxxComm.hh"


// Prototypes


int compare_HITB_indices(void const *, void const *);




// Let's do without STL for the moment.
extern TaHitStruct *taHitIndices;
extern int taHitDim;
extern int taHitNum;





int actaxxFill()
{
	ARTE_mark( "actaxxFill 0", GRR_OK );

	if ( !taHitNum )
	{
		ARTE_mark( "actaxxFill 1", GRR_OK );
		return taHitNum;
	}

	if ( !taHitIndices )
	{
		cout<<"actaxxFill: *** Error, no taHitIndices available! \n"
			<<"\ttaHitDim="<<taHitDim
			<<"\ttaHitNum="<<taHitNum
			<<endl;

		ARTE_mark( "actaxxFill 1", GRR_FISHY );
		return -1;
	}


	// Sorting:
	ARTE_mark( "qsort_in_actaxxFill 0", GRR_OK );
	qsort (taHitIndices, taHitNum, sizeof(TaHitStruct), compare_HITB_indices);
	ARTE_mark( "qsort_in_actaxxFill 1", GRR_OK );



	TAHB tahb;

	int distinctHits = 0;
	int taHitIndex(0), taHitIndexOld;

	int jTaTr, nTaTr = 0;		// N.B.: `nTaTr = 0' here is essential
	int taRsIndex(0), taRsIndexOld(0);
	int taRsIndices[taHitNum];			// My poor stack!

	for (int i=0;i<taHitNum;i++)
	{
		// Calculate corresponding TATR's index (RSEGs are not sorted)

		if ( i ) taRsIndexOld = taRsIndex;
		taRsIndex = taHitIndices[i].rs;
		jTaTr =-1;

		if ( taRsIndex )
		{
			for (jTaTr = 0; ; jTaTr++)
			{
				if ( jTaTr >= taHitNum )
				{
					cout<<"actaxxFill: *** Error, something odd, "
						<<"jTaTr="<<jTaTr
						<<", taHitNum="<<taHitNum<<endl;

					ARTE_mark( "actaxxFill 1", GRR_FISHY );
					return -1;	// This should never happen
				}

				if ( jTaTr == nTaTr )
				{
					taRsIndices[nTaTr] = taRsIndex;
					nTaTr++;
					break;
				}
				if ( taRsIndices[jTaTr] == taRsIndex ) break;
			}
		}


		// Skip equal TA hit entries

		if ( i )
		{
			taHitIndexOld	= taHitIndex;
			taHitIndex	= taHitIndices[i].hb;

			if (	taHitIndexOld == taHitIndex
				&& taRsIndexOld == taRsIndex )
			{
				int tahbOld = ArteTable<TAHB>::size();
				if ( !tahbOld ) continue;

				ArtePointer<TAHB>(tahbOld)->flg |= taHitIndices[i].who;

				continue;
			}
		}
		else
			taHitIndex = taHitIndices[i].hb;



		distinctHits++;

		ArtePointer<HITB> hitb(taHitIndex);
		tahb.flg = taHitIndices[i].who;

		// OTR or ITR and skip everything else
		if ( hitb->gede->cmp == 2 )
		{
			tahb.channel	= hitb->chid;
			tahb.dt		= hitb->y;
			tahb.flg	|= TAFLG::otr;

			tahb.gede	= ArtePointer<GEDE>(0);
			if ( !(taCommOTR.flg & TACommOTR::OtrI_gd) )
				tahb.gede	= hitb->gede;
		}
		else if ( hitb->gede->cmp == 3 )
		{
			// Inner Tracker : Torsten Zeuner 18/12/2000

			tahb.flg    |= TAFLG::itr;
			tahb.dt      = hitb->y;			// channel id 
			tahb.channel = int (hitb->r*1000000.);	// hit resolution
			tahb.gede    = hitb->gede;
		}
		else if ( hitb->gede->cmp == 1 )
		{
			// Vertex Detector

			tahb.flg    |= TAFLG::vds;
			ArtePointer<GEDE> gede = hitb->gede;

			tahb.dt		= (hitb->x - gede->xfw[0])/gede->pit;		// c.o.g.
			tahb.channel	= int (hitb->q*1.e3);	// claster charge

			// compressed side id (superlayer nameing scheme)
			/* sideid = ((superlayer<<4) | (quadrant<<2) | view) */

			int sl		= strtol(gede->gesl->name+2, NULL, 10);
			int sec		= gede->sec;
			int lay		= gede->lay;

			if ( sl < 1 || sl > 8 || sec < 1 || sec > 4 || lay < 1 || lay > 4 )
			{
				static int count = 0;
				if ( (count++) < 20 )
					printf(" *** actaxxFill: VDS plane %d has wrong SL, quadrant, veiw range!\n\t\t[failed to create TAHB]\n", int (gede.index()));

				continue;
			}

			int sideid	= ((sl-1)<<4) | ((sec-1)<<2) | (lay-1);
			tahb.gede	= ArtePointer<GEDE>(sideid);	// sideid, not Arte gede index (!)
		}
		else
			continue;

		tahb.flg  |= TAFLG::tatr&(jTaTr+1);



		// TRACKONLY flagging
		if ( jTaTr !=-1 )
			for (ArteRelationIterator<HITB> ahit = ArtePointer<RSEG>(taRsIndex)->hitb.begin();
							ahit != ArtePointer<RSEG>(taRsIndex)->hitb.end(); ++ahit )
				if ( taHitIndex == (int) (*ahit).index() )
				{
					tahb.flg |= TAFLG::fit;
					break;
				}


		if ( !ArteTable<TAHB>::push_back(tahb).index() )
		{
			ARTE_mark( "actaxxFill 1", GRR_FISHY );
			return -1;
		}
	}



	ArtePointer<EVHD> evhd(1);

	// tahd.time is used to remember EventUnixTime
	// from EVT_DATA.event_time for later use in database access.
	// must be checked against evhd->date,time which are filled in farm

	TAHD tahd;

	tahd.evt	= evhd->evt;
	tahd.run	= evhd->run;
	tahd.exp	= evhd->exp;
	tahd.time	= OtrEventInfoTime();
	tahd.dbmajor	= evhd->dbmajor;
	tahd.dbminor	= evhd->dbminor;
	tahd.vgeo	= ArtePointer<GEHD>(1)->geom;

	if ( !ArteTable<TAHD>::push_back(tahd).index() )
	{
		ARTE_mark( "actaxxFill 1", GRR_FISHY );
		return -1;
	}




	TATR tatr;

	for ( int i = 0; i < nTaTr; i++ )
	{
		taRsIndex = taRsIndices[i];
		if ( !taRsIndex ) continue;

		ArtePointer<RSEG> rseg( taRsIndex );

		// Here is the field to show who has selected this track
		tatr.who	= 0;

		// Not shure that it's correct
		for ( ArteTableIterator<TAHB> hb = ArteTable<TAHB>::begin();
					hb != ArteTable<TAHB>::end(); ++hb )
			if ( int(hb->flg & TAFLG::tatr) == i+1 )
				tatr.who |= hb->flg;


		tatr.chi	= rseg->chi;
		tatr.cmp	= rseg->cmp;
		tatr.fit	= rseg->fit;
		tatr.p		= rseg->pe;

		// maybe we'd better remove them at all?
		tatr.tahb1	= rseg->nhit;
		tatr.tahbl	= rseg->npar;

		tatr.x		= rseg->xe;
		tatr.y		= rseg->ye;
		tatr.tx		= rseg->txe;
		tatr.ty		= rseg->tye;
		tatr.z		= rseg->ze;

		if ( !ArteTable<TATR>::push_back(tatr).index() )
		{
			ARTE_mark( "actaxxFill 1", GRR_FISHY );
			return -1;
		}
	}




	memset(taHitIndices, 0, taHitNum*sizeof(TaHitStruct));
	taHitNum	= 0;




	ARTE_mark( "actaxxFill 1", GRR_OK );
	return distinctHits;
}





int compare_HITB_indices(void const *a, void const *b)
{
	// If an error occurs here, we shall know that by qsort_in_actaxxFill

	// Sort due to increasing HITB indices
	int c = ( ((TaHitStruct *)a)->hb - ((TaHitStruct *)b)->hb );

	// Sort due to increasing RSEGs
	if ( !c ) c = int( ((TaHitStruct *)a)->rs - ((TaHitStruct *)b)->rs );

	// Sort anyhow to avoid 0 return value
	return c ? c : int( (TaHitStruct *)a - (TaHitStruct *)b );
}
