// *********************************************************************
//
//  Routine:	alca/actaxxSelectOTR.C
//  Author:		Dirk Kruecker
//
//  Description:	OTR-specific TAXX selection function,
//				called from actaxxevnt.C
//
//  Modifications:
//   -- none yet					06/12/00 DG
//
// *********************************************************************

#include "arte/arte.hh"
#include "arte/RSEG_cmp.hh"

#include "alca/actaxxComm.hh"
#include "alca/actaxxFLG.hh"

void actaxxSelectOTRIO(); // Test selction based on otrio



void actaxxSelectOTR()
{
	actaxxSelectOTRIO();

	return;
}




/**********************************************************************/


#ifdef THE_TATEST_MODE

extern "C" {
	void hbook1_(const int &,const char *,const int &,
			const float &, const float &, const float &, const int);
	void hf1_(const int &, const float &, const float &);
}

#endif]





// Includes for Wouter's otrio selection
#include <set>
#include <algorithm>
#include <rang/Track.hh>
#include <rang/Node.hh>
#include <rang/State.hh>





/* DG: Borrowed from Wouter's otrio package */
bool
SelectTrack(ArtePointer<RSEG> rseg)
{
  /*const*/ int   minnmchits  = taCommOTR.nMC;	// 0;
  /*const*/ int   minnpchits  = taCommOTR.nPC;	//10;
  /*const*/ int   minntchits  = taCommOTR.nTC;	// 5;
  /*const*/ float minmomentum = taCommOTR.pcut;	//2.;
  bool rc=false ;


  if( rseg->fit==0 &&
      (rseg->cmp & Rsegc::patt) &&
      fabs(rseg->pf) * minmomentum < 1. ) {
    int nmchits(0),npchits(0),ntchits(0) ;
    for(ArteRelationIterator<RHIT> rhitIt = rseg->rhit.begin() ;
	rhitIt != rseg->rhit.end() ; rhitIt++)
      if((*rhitIt)->hitb->gede->cmp==2) {
	if((*rhitIt)->hitb->z<680) nmchits++ ;
	else                    npchits++ ;
      }
    if(nmchits>=minnmchits && npchits>=minnpchits) {

      for(ArteRelationIterator<RSEG> crsegIt = rseg->rend.begin() ;
	  crsegIt != rseg->rend.end() ; ++crsegIt) {
	ArtePointer<RSEG> tcseg = *crsegIt ;
	if( tcseg->fit==0 && (tcseg->cmp & Rsegc::trit ) ) 
	  for(ArteRelationIterator<RHIT> rhitIt = (*crsegIt)->rhit.begin() ;
	      rhitIt != (*crsegIt)->rhit.end() ; rhitIt++) 
	    if((*rhitIt)->hitb->gede->cmp==2) ntchits++;
      }
	
      if(ntchits>=minntchits) rc=true ;
    }
  }
  return rc ;
}
      




/* DG: borrowed from Wouter's otrio package */
void
OTrIOTrack(ArtePointer<RSEG> rseg)
{
  float fZ, fX, fY, fTx, fTy, fQp;

  /*const*/ float xwin = taCommOTR.xcut;	//1 ; //pit
		float x1win = taCommOTR.xexc;	// exclusion radius
  /*const*/ float zwin = taCommOTR.zcut;	//20 ;
  // IT IS IMPORTANT TO PRESERVE THE ORDER OF THE HITBS. 
  // WE CAN ACHIVE THIS BY USING A SET
			// not neceessary, TAXX takes care of that (DG)

  // collect all reconstructed hits
  vector<ArtePointer<RHIT> > rhits ;
  for(ArteRelationIterator<RHIT> rhitIt = rseg->rhit.begin() ;
      rhitIt != rseg->rhit.end() ; rhitIt++)
    if((*rhitIt)->hitb->gede->cmp==2)
      rhits.push_back(*rhitIt) ;

  // also those of related tc segments
  ArtePointer<RSEG> eseg=rseg ;
  for(ArteRelationIterator<RSEG> crsegIt = rseg->rend.begin() ;
      crsegIt != rseg->rend.end() ; ++crsegIt) {
    ArtePointer<RSEG> tcseg = *crsegIt ;
    if( tcseg->fit==0 && (tcseg->cmp & Rsegc::trit ) ) {
      eseg=tcseg ;
      for(ArteRelationIterator<RHIT> rhitIt = (*crsegIt)->rhit.begin() ;
	  rhitIt != (*crsegIt)->rhit.end() ; rhitIt++) 
	if((*rhitIt)->hitb->gede->cmp==2)
	  rhits.push_back(*rhitIt) ;
    }
  }
  
  // find RHIT at beginning of patt seg
  ArtePointer<RHIT> rhit ;
  for(vector<ArtePointer<RHIT> >::iterator rhitIt = rhits.begin() ;
      rhitIt != rhits.end() ; rhitIt++)
    if((*rhitIt)->hitb->z > 680 && 
       (!rhit.index() || rhit->hitb->z > (*rhitIt)->hitb->z))
      rhit = *rhitIt ;

 
  // copy coordinates
  fZ = rhit->hitb->z ;
  fX = rhit->x ;
  fY = rhit->y ;
  fTx = (eseg->xe - rhit->x)/(eseg->ze - fZ) ;
  fTy = (eseg->ye - rhit->y)/(eseg->ze - fZ) ;
  fQp = rhit->p ;

  
  set<ArtePointer<HITB> > hitbs ;
  for(vector<ArtePointer<RHIT> >::iterator rhitIt = rhits.begin() ;
      rhitIt != rhits.end() ; rhitIt++)
  {
    float x = (*rhitIt)->x;
    float y = (*rhitIt)->y;
    float rotx = (*rhitIt)->hitb->gede->rot[0]*x - (*rhitIt)->hitb->gede->rot[1]*y;
#ifdef THE_TATEST_MODE
    hf1_(610, (*rhitIt)->hitb->x - rotx, 1.);
#endif
    hitbs.insert((*rhitIt)->hitb) ;
  }
 
  float zmin(1500),zmax(0) ;
  for(vector<ArtePointer<RHIT> >::iterator rhitIt = rhits.begin() ;
      rhitIt != rhits.end() ; rhitIt++) {
    if(zmax<(*rhitIt)->hitb->z) zmax = (*rhitIt)->hitb->z ;
    if(zmin>(*rhitIt)->hitb->z) zmin = (*rhitIt)->hitb->z ;
  }
  
  zmax += zwin ;
  zmin -= zwin ;  

  //cout << "hits in track: " << hitbs.size() << endl ;
  if(xwin>0) {
    if ( taCommOTR.userefit > 0 )
    {

    vector<ArtePointer<RSEG> > rsegs ;
    rsegs.push_back(rseg) ;
    if(eseg.index() != rseg.index() ) rsegs.push_back(eseg) ;
    rangerFitMode fitmode = (rseg->cmp & Rsegc::magt) ? ModeP : ModeL ;
    rangerTrack* track = new rangerTrack(rsegs,fitmode) ;
    track->fit() ;
    
    // now add other hits, using the walls which have been found by the trackfit
    int prevplane=0;
    ArtePointer<GEDE> gede ;
    const rangerNode* firstnode=0;
    const rangerNode* lastnode=0;
    for(rangerTrack::rangerNodeConstIterator nodeIt = track->begin() ;
	nodeIt != track->end() ; nodeIt++) {
      int plane = (*nodeIt)->plane() ;
      if(plane>0 && prevplane!=plane && (gede=ArtePointer<GEDE>(plane))->cmp==2 &&
       gede->hitl - gede->hit1>=0) {
	const rangerState& state = (*nodeIt)->getState() ;
	float rotx = gede->rot[0]*state.x() - gede->rot[1]*state.y() ; 
	float roty = gede->rot[0]*state.y() + gede->rot[1]*state.x() ; 
	if(roty > gede->y1w[0] && roty < gede->y2w[0])
	  for(ArteTable<HITB>::iterator hitbIt = gede->hit1 ;
	      hitbIt != gede->hitl+1 ; hitbIt++) 
	    if(fabs( rotx - hitbIt->x ) < xwin*gede->pit)  hitbs.insert(*hitbIt) ;
	prevplane=plane ;
      }
      if((*nodeIt)->zNode()>680) {
	if(firstnode==0 || (firstnode->zNode() > (*nodeIt)->zNode()))
	   firstnode = *nodeIt ;
	if(lastnode==0 || (lastnode->zNode() < (*nodeIt)->zNode()))
	  lastnode = *nodeIt ;
      }
    }
    const rangerState& firststate = (const_cast<rangerNode*>(firstnode))->getState() ;
    const rangerState& laststate  = (const_cast<rangerNode*>(lastnode))->getState() ;
    fZ = firststate.z() ;
    fX = firststate.x() ;
    fY = firststate.y() ;
    fTx = (laststate.x() -  firststate.x())/(laststate.z() -  firststate.z()) ;
    fTy = (laststate.y() -  firststate.y())/(laststate.z() -  firststate.z()) ;
    
    delete track ;

    }
    else
    {

    for(ArteTable<GEDE>::iterator gedeIt = ArteTable<GEDE>::begin() ;
	gedeIt != ArteTable<GEDE>::end() ; gedeIt++) 
      if(gedeIt->cmp==2 &&
	 !(gedeIt->zmax<zmin || gedeIt->zmin>zmax) &&
	 gedeIt->hitl.index()+1-gedeIt->hit1.index()>0 ) {
	float z = (gedeIt->zmax + gedeIt->zmin)/2 ;
	ArtePointer<RHIT> closestRhit = *rhits.begin() ;
	for(vector<ArtePointer<RHIT> >::iterator rhitIt = rhits.begin() ;
	    rhitIt != rhits.end() ; rhitIt++) 
	  if(fabs(z - (*rhitIt)->hitb->z)<fabs(z - closestRhit->hitb->z))
	    closestRhit = *rhitIt ;
	
	// Is this rhit close enough ? We do not want too extrapolate
	// over too large distances (especially in magnet ...)
	
	float dz = z - closestRhit->hitb->z ;
	//cout << "selected gede: " << dz << endl ;
	if(fabs(dz) < zwin) {
	  // linear extrapolate track parameters:
	  float x = closestRhit->x + closestRhit->tx * dz ;
	  float y = closestRhit->y + closestRhit->ty * dz ;
	  // rotate parameters to frame of gede
	  float rotx = gedeIt->rot[0]*x - gedeIt->rot[1]*y ;
	  float roty = gedeIt->rot[0]*y + gedeIt->rot[1]*x ;
	
	  // Is gede hit ?
	  if(rotx > gedeIt->sxmn-xwin*gedeIt->pit && rotx < gedeIt->sxmx+xwin*gedeIt->pit &&
	     roty > gedeIt->symn && roty < gedeIt->symx) {

	    int exclude=0;
	    for(ArteTable<HITB>::iterator hitbIt = gedeIt->hit1 ;
		hitbIt != gedeIt->hitl+1; hitbIt++)
	      if(fabs(hitbIt->x - rotx) > xwin*gedeIt->pit &&
				fabs(hitbIt->x - rotx) < x1win*gedeIt->pit)
		exclude = 1;
	    if (exclude) continue;

	    for(ArteTable<HITB>::iterator hitbIt = gedeIt->hit1 ;
		hitbIt != gedeIt->hitl+1; hitbIt++)
	    {
#ifdef THE_TATEST_MODE
		hf1_(620, hitbIt->x - rotx, 1.);
#endif
	      if(fabs(hitbIt->x - rotx) < xwin*gedeIt->pit)
	      {
#ifdef THE_TATEST_MODE
		hf1_(630, hitbIt->x - rotx, 1.);
#endif
		hitbs.insert(*hitbIt) ;
	      }
	    }
	  }
	}
      }
    }
  } 

  // now fill otrio hitvector
  //cout << "number of hits close to track: " << hitbs.size() << endl ;
  //	fHits.reserve(hitbs.size()) ;
  for(set<ArtePointer<HITB> >::iterator hitbIt = hitbs.begin() ;
      hitbIt!=hitbs.end() ; ++hitbIt) 
    actaxxAdd(*hitbIt, rseg, TAFLG::otrsel );
    //	fHits.push_back(OTrIOHit(*hitbIt)) ;
}


 
// Hit selection function template

void actaxxSelectOTRIO()
{
	// Some constants from rgsltre for chi2 to compare track to it's neighbours
	float chi2 = taCommOTR.chi2_neighbour;
	const float w_x = 156.25;
	const float w_y = 1.085;
	const float w_tx = 765625.;
	const float w_ty = 5317.;

	for (ArteTableIterator<RSEG> rseg = ArteTable<RSEG>::begin();
				rseg != ArteTable<RSEG>::end(); ++rseg )
	{
		if ( !SelectTrack(*rseg) ) continue;

		int neighbour = 0;
		for (ArteTableIterator<RSEG> rs = ArteTable<RSEG>::begin();
					rs != ArteTable<RSEG>::end(); ++rs )
		{
			if ( rseg.index() == rs.index() ) continue;
			if (
				(*rs)->cmp & Rsegc::bitsign
				|| !( (*rs)->cmp & Rsegc::patt )
			   )	continue;

			// Try to delete tracks passing close to each other
			float xt	= rs->xe + rs->txe * ( (*rseg)->ze - rs->ze );
			float yt	= rs->ye + rs->tye * ( (*rseg)->ze - rs->ze );
			float txt	= rs->txe;
			float tyt	= rs->tye;

			float chix	= (*rseg)->xe  - xt; 
			float chiy	= (*rseg)->ye  - yt; 
			float chitx	= (*rseg)->txe - txt; 
			float chity	= (*rseg)->tye - tyt; 

			float chit	= w_x * chix * chix + w_y * chiy * chiy
						+ w_tx * chitx * chitx + w_ty * chity * chity;

#ifdef THE_TATEST_MODE
			hf1_(600, chit, 1.);
#endif
			if ( chit <= chi2 )
			{
				neighbour = 1;
				break;
			}
		}

		if ( neighbour ) continue;
		OTrIOTrack(*rseg);
	}
}
