//                              -*- Mode: C++ -*-
//:ECCellID.C
// 
//
// Author: Mauro Villa   
// ECAL group 
// HERA-B experiment, Desy
// e-mail: villa@bo.infn.it
//
// Created 9/4/97
/* $Log: ECCellID.C,v $
/* Revision 1.5  1999/08/04 15:21:32  natasha
/* new care version from Mauro
/*
 * Revision 1.3  1999/06/01  16:07:35  villa
 * before HH
 *
 */
//

#include "care/ECCellID.hh"
#include "care/caredata.hh"


/*
CLASS
  ECCellID
  This class is used to store a Cell Identification Number.

OPERATOR
  The access to the Cell ID is given by (overloaded) function operators.

WARNING
  The three parameter constructor and function operator make use of 
  data defined by the LUTsManager. Be sure to use these members of the class
  only after the initialization of LUTs by the LUTsManager.

*/

//////////////////// Class ECCellID static data initialization  //////////

int ECCellID::firstPlane = 1;

//////////////////// Class ECCellID definition //////////////////////

//-
// Constructor with three parameters: Module number, ix, iy.
// Module number shold be in the range [1-number of modules] and
// ix and iy in the range [1,ndim] where ndim is the block division.

 
int ECCellID::operator()(ArtePointer<GCAL> modPointer, int ix, int iy) {
  int ndim = modPointer->ncl;
  return operator()(modPointer, ix, iy, ndim);
};

//-- 
//  static function to set the first module
void ECCellID::setFirstPlane(int firstPln){
  firstPlane = firstPln;
}

//-- 
//  Get out the x and the y position of a module
void ECCellID::getCoords(float & x, float & y) const {
  int ix=0, iy=0, ndim = 0;
  getModuleIndex(ix, iy, ndim);
  float rdx = ECalInfo.xMsize/ndim;
  float rdy = ECalInfo.yMsize/ndim;
  x = modulePointer()->xmin+rdx*(ix-0.5);
  y = modulePointer()->ymin+rdy*(iy-0.5);
}
