//                              -*- Mode: C++ -*-
//:ECdbmc.C
//
//
// Author: Mauro Villa
// ECAL group
// HERA-B experiment, Desy
// e-mail: villa@bo.infn.it
//

#include "care/ECdbmc.hh"
#include <algorithm>


const char * ECEffBoardEntry::comment = "Board efficiencies for MC sim";
const char * ECEffBoardEntry::fieldDesc = "I:stat F:eff1 F:eff2";
const int ECEffBoardEntry::fieldType[] = {TABLEint, TABLEfloat, TABLEfloat};
const int ECEffBoardEntry::fieldDim[] = {1,1,1};



// Database access functions
int ECEffBoardArray::openDB(const char * dbname, 
			    const char *tablename, int access){
  int rc = 0;
  x=OpenTables((char*)dbname, access);
  if(x!=0){
    tab = BookTable(x, (char*)tablename,
		    (char*)ECEffBoardEntry::fieldDesc, 
		    (char*)ECEffBoardEntry::comment);
    if( tab==0) rc = 2;
  } else {
    rc =1 ;
  }
  return rc;
}

int ECEffBoardArray::writeToDB(int maj, int min, int access){
  
  char *table=(char*) &eftable[0];
  major = maj;
  minor = min;
  int nrow = ECALBOARDS;
  TableStamp(x,maj,min);
  SkipTableField(x,1);

  for(int i=0; i<3; i++) {
    int entries = ECEffBoardEntry::fieldDim[i];
    int dim = entries*(ECEffBoardEntry::fieldType[i] & 0xF);
    char *buffer=new char[dim*nrow];
    int bufferCursor=0;
    int tableCursor=0;
    for(int j=0; j<nrow; j++) {
      memcpy(&buffer[bufferCursor],&table[tableCursor],dim);
      bufferCursor+=dim;
      tableCursor+=sizeof(ECEffBoardEntry);
    }

    PutTableField(x,ECEffBoardEntry::fieldType[i],
		  nrow*entries,buffer);
    delete buffer;

    table+=dim;
  }
  if(!StoreTable(x,tab,access)) return -1;
  major=TableMajor(x);
  minor=TableMinor(x);

  return 0;
}

int ECEffBoardArray::readFromDB(int maj, int min, int access){
  int rc = 0;
  TableStamp(x,maj,min);
  if(FetchTable(x, tab, access)){
    if( size()!=ECALBOARDS){
      eftable.erase(eftable.begin(), eftable.end());
      for(int i=0; i<ECALBOARDS; i++){
	eftable.push_back(ECEffBoardEntry());
      }
    }
    SkipTableField(x,1);
    int nrow=ECALBOARDS;
    char *table=(char*)&eftable[0];
    for(int i=0; i<3; i++) {
      int values = ECEffBoardEntry::fieldDim[i];
      int dim = values*(ECEffBoardEntry::fieldType[i] & 0xF);
      char *buffer=new char[dim*nrow];
      int bufferCursor=0;
      int tableCursor=0;
      
      GetTableField(x, ECEffBoardEntry::fieldType[i],nrow*values,buffer);
      
      for(int j=0; j<nrow; j++) {
	memcpy(&table[tableCursor],&buffer[bufferCursor],dim);
	bufferCursor+=dim;
	tableCursor+=sizeof(ECEffBoardEntry);
      }	
      delete buffer;	
      table+=dim;
    }
    major = TableMajor(x);
    minor = TableMinor(x);
    rc = nrow;
  } else {
    rc = -1;
  }
  return rc;
}

int ECEffBoardArray::closeDB(){
  int ret1=UnbookTable(x, tab);
  tab=0;
  int ret2=CloseTables(x);
  x=NULL;
  return ((ret1==1) || (ret2==1))? 0: 1;
}




