//                              -*- Mode: C++ -*-
//: ESCTCValley.C
//
//
// Author: Mauro Villa   
// ECAL group 
// HERA-B experiment, Desy
// e-mail: villa@bo.infn.it
//
// Created 18/6/97
//
// 18.02.01 the valley cut is now a fraction of the energy cut
//

#include "care/ESCTCValley.hh"
#include "care/EMCluster.hh"
#include "care/caredata.hh"
#include "care/CareUtil.hh"

/*
CLASS
  ESCTCValley
  Concrete class to evaluate the Cluster-To-Cluster distance 
  with the help of the Valley algorithm

*/


/////////////////////////////////////////////////////
ESCTCValley::ESCTCValley() : ESCTCDisStrat(), mainClus(0.0), boundCell(0.0) {}
ESCTCValley::~ESCTCValley(){}

 
//-- Behavioural operations
//   set the reference cluster to which distances must be evaluated
void ESCTCValley::refCluster(EMCluster* clus){
  //--
  // Store the energy of the reference cluster
  mainClus = clus->maxEnergy();
  // find the first merging cell and get its energy
  EMClusterMember* childMerge = clus->next();
  for(; childMerge!=0 && !childMerge->isALeaf() ;  
      childMerge = childMerge->next());
  if( childMerge!=0 ){
    boundCell = childMerge->maxEnergy(); 
  } else {
    boundCell = 0.0;
  }
}

//--
//   evaluate the position for a cluster
ChiSquare_t ESCTCValley::evaluateDis(EMCluster* clus){
  // get the peak energy of this cluster and evaluate the distance
  Energy_t secEner = clus->maxEnergy();
  //
  Energy_t thresh = EnergyThreshold(clus);
  if( thresh>0.0 )
    dist = (secEner-boundCell)/thresh;
  else
    dist = 100.;
  return dist;
}


//--
/// test if the clusters are different
bool ESCTCValley::areDifferent() const {
  return dist>ECalInfo.valley;
}
