// *****************************************************************************
//
// source:    MuMC2Digi.C
//
// type:      source code
//
// created:   14 Mar 2001
//
// author:    Jing-Ge Shiu, Lars Soezueer 
//            email: soezueer@mail.desy.de
//
// purpose:   auxiliary functions for MUON, MPIX and MPAD
//            called by getFED_IDs and getFED
//
// changes:   
//
// *****************************************************************************
#include "etb.h"

#include "arte/arte.hh"
#include "arte/ArteMessage.hh"
#include "murec/MUDbAll.h"
#include "murec/MuHitInfo.h"
#include "daqif/MuDUTool.h"

// constants definition for the compiler
#define MUrecordID  0x1002
#define SLBrecordID 0x2000
typedef struct HITPOS {
  float x, y, z;
};

typedef struct CHINFO {
  char chambername[12];         // chmaber name
  int cmp;                      // detector component id  
  ArtePointer<GESL>  gesl;      // superlayer id in HERA-B
  int dsubl;                    // detector sublayer in its superlayer 
  int sec;                      // sector number (tube(0,1)/pad(0-3) cable no, pixel quadrant no.)
  int rtype;                    // readout type                    (0 tube, 1 pad, 2 pixel)
  int superl;                   // superlayer inside this detector (1-4)
  int rsubl;                    // readout suberlayer in superl    (our conventional labelling)
  int dno;                      // detector number in subl         (chamber id no.)
  int updown;                   // top/bottom (0/1) detector       
  int single;                   // single layer number (0/1)
  int rno;                      // readout channel number in dno   (chanlle id no.)  
  float rot[9];                 // rotation parameters for this detector (chamber)  
  float xmin,xmax;
  float ymin,ymax;
  float zmin,zmax;
  HITPOS hitpos;
} ;

static MUDbAll muAllConfig;
int MCOuterLoops2(void);
//int MCInnerLoops2(void);

static MuHitInfo muhitinfo;

/*##################################################
  chambername;              // chmaber name
  cmp;                      // detector component id  
  gesl;                     // superlayer id in HERA-B
  dsubl;                    // detector sublayer in its superlayer
  sec;                      // sector number
  rtype;                    // readout type      
  superl;                   // superlayer inside this detector
  rsubl;                    // suberlayer in superl
  dno;                      // detector number in subl
  updown;                   // top/bottom (0/1) chamber
  rno;                      // readout channel number in dno
  rot[9];                   // rotation parameters for this detector (chamber)    

  myRecChannel     // Recognizing readout Channel

 #################################################*/
CHINFO myRecChannel( ArteTable<HITB>::iterator hitbiv ) {
   
  CHINFO chinfo;
  //  ArteTable<GEDE>::iterator geden = (*hitbiv)->gede; 

  memset(&chinfo, 0, sizeof(CHINFO)); 

  muhitinfo.putmuhit(ArtePointer<HITB>(hitbiv));
  if ( muhitinfo.muhitok() == OKHITBINPUT ) {
    strcpy( chinfo.chambername,muhitinfo.dname());  
    chinfo.gesl    = muhitinfo.gesl();
    chinfo.dsubl   = muhitinfo.dsubl();
    chinfo.sec     = muhitinfo.sec();
    chinfo.rtype   = muhitinfo.readtype();
    chinfo.cmp     = muhitinfo.hitbcmp();    
    chinfo.superl  = muhitinfo.superl();
    chinfo.rsubl   = muhitinfo.rsubl();
    chinfo.updown  = muhitinfo.topbottom();
    chinfo.hitpos.x= muhitinfo.x();
    chinfo.hitpos.y= muhitinfo.y();
    chinfo.hitpos.z= muhitinfo.z();
    chinfo.dno     = muhitinfo.chamberid();
    chinfo.rno     = muhitinfo.channelid();
    chinfo.single  = muhitinfo.frontback();
  } else 
    chinfo.cmp = 0;

  return chinfo;
}

void FillInt2Char4 (LWord in, char out[]) {

  out[0]=(in>>24)&0xFF;
  out[1]=(in>>16)&0xFF;
  out[2]=(in>>8)&0xFF;
  out[3]=in&0xFF;

  return;
}

void SetDMU(Word partid, Word &fedno, Word fedID[]) {

  LWord feddata;
  char  dmudata1[4],dmudata2[4];

  if (fedno>0) {
     FillInt2Char4(SLBrecordID,dmudata1);
     if (partid&0x01) ArteTable<DMUT>::push_back(DMUT())->bits=dmudata1; 
     else if (partid&0x02) ArteTable<DMUP>::push_back(DMUP())->bits=dmudata1;
     else if (partid&0x04) ArteTable<DMUX>::push_back(DMUX())->bits=dmudata1;
     else {
         arteMsg << "MuMC2Digi.C: Please give me a break. Do one at one time." ;
         arteMsg.warn();
         return;
     }    

     for (Word fn=0;fn<fedno;fn++) {       
        feddata = 0x880000 + fedID[fn];
        FillInt2Char4(LWord(feddata),dmudata1);
        feddata = MUrecordID<<16;      
        FillInt2Char4(LWord(feddata),dmudata2);

        switch (partid) {
        case 0x01 :
           ArteTable<DMUT>::push_back(DMUT())->bits=dmudata1;      
           ArteTable<DMUT>::push_back(DMUT())->bits=dmudata2;   
           break;
        case 0x02 :
           ArteTable<DMUP>::push_back(DMUP())->bits=dmudata1;      
           ArteTable<DMUP>::push_back(DMUP())->bits=dmudata2;   
           break;
        case 0x04 :
           ArteTable<DMUX>::push_back(DMUX())->bits=dmudata1;      
           ArteTable<DMUX>::push_back(DMUX())->bits=dmudata2;   
           break;
        default:
           arteMsg << "MuMC2Digi.C: Please give me a break. Do one at one time." ;
           return;
           arteMsg.warn();
        }
 
        FillInt2Char4(0,dmudata1);
        for (int di=0;di<32;di++) {
           if (partid&0x01) ArteTable<DMUT>::push_back(DMUT())->bits=dmudata1; 
           if (partid&0x02) ArteTable<DMUP>::push_back(DMUP())->bits=dmudata1;
           if (partid&0x04) ArteTable<DMUX>::push_back(DMUX())->bits=dmudata1;
        }
     }
  }
  return;
}

// initialization of MCHitb2Digi
// clear DMUT, DMUP, and DMUX, then set them up.
int MCHitb2DigiIni() {

  //  char* Dbname = "/MUON_DB";
  Word fedno, fedID[MAXMUFED];  
  static short started = 0;
  
  if (started==0) {
    muAllConfig.ini();
    fedno = muAllConfig.getFED(7,fedID);
    started++;
  }

  if ( ArteTable<DMUT>::size() != 0 ) ArteTable<DMUT>::clear();
  if ( ArteTable<DMUP>::size() != 0 ) ArteTable<DMUP>::clear();
  if ( ArteTable<DMUX>::size() != 0 ) ArteTable<DMUX>::clear();

  fedno = muAllConfig.fedID(1,fedID);
  SetDMU(0x01, fedno, fedID);
  fedno = muAllConfig.fedID(2,fedID);
  SetDMU(0x02, fedno, fedID);
  fedno = muAllConfig.fedID(4,fedID);
  SetDMU(0x04, fedno, fedID);

 return 1;
}

// main routine of digi into hitb transformation
int MCHitb2Digi()
{

// no action if no MC
  if(ArteTable<MTRA>::size() == 0) return 0;

  MCHitb2DigiIni();  

// set the initial pointers to hits
// make hits into digi transformation
  MCOuterLoops2();
  //  MCInnerLoops2();

  return 0;
}

void MCFillDMU(Word fedID, Word nbit) {

     Word nb32 = nbit/32;
     short cmpid=0;
     LWord dmupos=0;
   
     cmpid = jsMuFindCmp(fedID, dmupos);
     if (cmpid==0) {
       arteMsg << "MuMC2Digi.C: There is no such FEDid : "
               << hex << fedID << dec ;
       arteMsg.warn();
       return;
     }

     LWord *lbuf=0, word=0, nbpos;
     nbpos = dmupos+1+nb32;
     char  dmudata[4];

     if (cmpid==COMP_ID_MUON)
        lbuf=(LWord*)(ArtePointer<DMUT>(nbpos+1)->bits);
     else if (cmpid==COMP_ID_MPAD)
        lbuf=(LWord*)(ArtePointer<DMUP>(nbpos+1)->bits);
     else if (cmpid==COMP_ID_MPIX)
        lbuf=(LWord*)(ArtePointer<DMUX>(nbpos+1)->bits);

     if (muAllConfig.endian()) word=ESP_SWAP_INT(*lbuf);
     else word = *lbuf; 

     word = ESP_SWAP_INT(word); 
     word|=(0x01<<(nbit%32)); 
     word = ESP_SWAP_INT(word);

     FillInt2Char4(word,dmudata);
     if (cmpid==COMP_ID_MUON)
          ArtePointer<DMUT>(nbpos+1)->bits=dmudata; 
     else if (cmpid==COMP_ID_MPAD)
          ArtePointer<DMUP>(nbpos+1)->bits=dmudata; 
     else if (cmpid==COMP_ID_MPIX)
          ArtePointer<DMUX>(nbpos+1)->bits=dmudata; 
	  
  return;
}

int MCOuterLoops2(void)  {

  CHINFO chinfo;
  Word sl, lay, ud, sd, nx, nr, nc, crate, nbit1, wirpad1, nbit;
  int  step, astep;

  for ( ArteTableIterator<HITB> hitbiv = ArteTable<HITB>::begin(); 
        hitbiv != ArteTable<HITB>::end();
        ++hitbiv ) {
      int cmp = (*hitbiv)->gede->gesl->cmp;
      if (cmp==8) {
	
         chinfo = myRecChannel(hitbiv); 
	 //         if ( (chinfo.rtype==1)&&(chinfo.dno==99) ) {
         if ( (chinfo.rtype==1) ) {
	   /*
            cout << (*hitbiv)->x << ", " << (*hitbiv)->y <<", " <<(*hitbiv)->z  << " <--- hit outer";
            cout << ", xmin = " << (*hitbiv)->gede->xmin 
                 << ", ymin = " << (*hitbiv)->gede->ymin 
                 << ", ymax = " << (*hitbiv)->gede->ymax << endl; 
            cout << "first pad height : " << (*hitbiv)->gede->xfw[2] 
                 << ", other pas height : " << (*hitbiv)->gede->xfw[3] 
                 << ", pad number : " << (*hitbiv)->gede->nwi[2] <<  endl;
		 */
         }

         sl  = chinfo.superl - 1;
         lay = (*hitbiv)->gede->lay - 1;
         ud  = chinfo.updown;
	 sd  = chinfo.single;
         // transform our construction traditional cable id to MuDb cable real mapping. 
	 nc  = (chinfo.rtype==0)?chinfo.sec:
               ( ( ((chinfo.dno<19)&&(chinfo.updown==0))||((chinfo.dno>18)&&(chinfo.updown==1)) )?
                 ((chinfo.sec+0x02)&0x03):chinfo.sec );
	         // change 2nd bit of chinfo.sec 
	         // for -x+y or +x-y, in MuDB, cable A/B/C/D is 2/3/0/1 
         // special correction for pad chamber 99 to fit MuDB
         if ((chinfo.rtype==1)&&(chinfo.dno==99)&&(nc==2)) nc=1;
        
         nx  = ( (chinfo.dno!=99)?chinfo.dno:
               ( (sl>1)?muAllConfig.NchambP[sl-2][ud]:muAllConfig.NchambT[sl][lay][ud] ) )- 1;
         crate = (chinfo.rtype==1)?muAllConfig.CratePP[sl-2][ud][nx]:
	         ( (sl>1)?muAllConfig.CrateP[sl-2][ud][nx]:muAllConfig.CrateT[sl][lay][ud][nx] );
         nbit1 = (sl<2)?muAllConfig.Nbit1T[sl][lay][ud][nx][nc][sd]:
        	 ( (chinfo.rtype==0)?muAllConfig.Nbit1P[sl-2][ud][nx][nc][sd]
                                    :muAllConfig.Nbit1PP[sl-2][ud][nx][nc] );
         // special modification to fit MuDB mapping
	 if ( ((nc&0x01)==0)&&(sl==3)&&(chinfo.rtype==1) ) nbit1--;
         if ( (chinfo.rtype==1)&&(chinfo.dno==99) ) nbit1-=24;

         wirpad1 = (sl<2)?muAllConfig.Wire1T[sl][lay][ud][nx][nc][sd]:  
                 ( (chinfo.rtype==0)?muAllConfig.Wire1P[sl-2][ud][nx][nc][sd]
		                    :muAllConfig.Pad1[sl-2][ud][nx][nc] );
         step  = (sl<2)?muAllConfig.StepT[sl][lay][ud][nx]:muAllConfig.StepP[sl-2][ud][nx];
         astep = (step>0)?step:-step;
	 nr = chinfo.rno - ((chinfo.rno>16)?16:0);
         if (chinfo.rtype==0) nbit=nbit1+astep*((nr-1)/2);
         else nbit=nbit1+nr-1;         

	 //      if ( (chinfo.rtype==1)&&(chinfo.dno==99) ) {
         if ( (chinfo.rtype==1) ) {
	 /*
         cout << "name : " << chinfo.chambername << endl;
         cout << "x/y/z = " << chinfo.hitpos.x << "/ " 
              << chinfo.hitpos.y << "/ " <<chinfo.hitpos.z  << endl;
         cout << dec << "superlayer : " << chinfo.superl 
              << " sublayer : "    << chinfo.rsubl 
              << ", chamber : "    << chinfo.dno 
              << ", ch_type : "    << chinfo.rtype
              << ", up/down : "    << chinfo.updown
              << ", sd no.  : "    << sd            // chinfo.single
              << ", sec no. : "    << chinfo.sec
              << ", channel : "    << chinfo.rno  
              << ", nr      : "    << nr  << endl;
         cout << "crate " << hex << crate
              << ", nbit1 " << dec << nbit1 
              << ", wirpad1 " << wirpad1 << ", step " << step 
              << " ===> nbit " << nbit << endl;
	      */
         }   	 
      MCFillDMU (crate,nbit);
      }
  }
 return 1;
}; 

int MCInnerLoops2(void)  {

  CHINFO chinfo;
  //  Word sl, lay, ud, sd, nx, nc;

  for ( ArteTableIterator<HITB> hitbiv = ArteTable<HITB>::begin(); 
        hitbiv != ArteTable<HITB>::end();
        ++hitbiv ) {
      int cmp = (*hitbiv)->gede->gesl->cmp;
      if (cmp==9) {
	/*
         cout <<  (*hitbiv)->x << "," << (*hitbiv)->y <<"," <<(*hitbiv)->z  << "<--- hit inner" << endl;
         chinfo = myRecChannel(hitbiv); 
         cout << "name : " << chinfo.chambername << endl;
         cout << dec << "superlayer : " << chinfo.superl
              << " quadrant : "   << chinfo.sec << endl;
	      */
      }
  }
 return 1;
};


