// *****************************************************************************
//
// source:    kadaqif.C
//
// type:      source code
//
// created:   15 Mar 2001
//
// author:    Lars Soezueer
//            email: soezueer@mail.desy.de
//
// purpose:   treat KUIP commands for daqif
//
// changes:   
//
// *****************************************************************************
// standard includes
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>

// ONLINE includes
#include "component.h"

// Arte includes
#include "arte/ArteMessage.hh"
#include "arte/arte.hh"
#include "farm/farm.hh"
#include "daqif/kadaqif.hh"

extern "C"
{
  void kugetc_(char *string, int *len, int len1);
  void kugetr_(float *);
  char* ku_gets();
}

kadaqif* kadaqif::m_instance = 0;

kadaqif* kadaqif::Instance()
{
  if (m_instance == 0) {
    m_instance = new kadaqif();
  }
  return m_instance;
}

kadaqif::kadaqif():
 m_compmask(0x7FFF),
 m_mctruth_Ecut(1.0),
 m_daqfile("./daqif.daq")
{};

unsigned int update_compmask(unsigned int compid, int status)
{
  return ( status ?
           kadaqif::get_compmask() | compid :
           kadaqif::get_compmask() & ~compid );
}

extern "C"
void daqif_SUBD()
{
  ARTE_mark( "daqif_SUBD 0", 0);

  // read in subdetector
  int len;        
  int len1 = 4;
  char input[7];
  kugetc_( input, &len, len1 );
  input[len] = 0;
  
  char *subdetector;
  subdetector = new char[len+1];
  strncpy( subdetector, input, len );
  subdetector[len] = '\0';
  
  for( int i=0; i<len; i++ ) subdetector[i] = toupper( subdetector[i] );

  if( strncmp( subdetector,"SVD",   3 ) &&
      strncmp( subdetector,"VDS",   3 ) &&
      strncmp( subdetector,"ITR",   3 ) && 
      strncmp( subdetector,"HPT",   3 ) &&
      strncmp( subdetector,"HIPT",  4 ) &&
      strncmp( subdetector,"OTR",   3 ) && 
      strncmp( subdetector,"RICH",  4 ) &&
      strncmp( subdetector,"TRD",   3 ) && 
      strncmp( subdetector,"ECAL",  4 ) &&
      strncmp( subdetector,"MUON",  4 ) && 
      strncmp( subdetector,"MPIX",  4 ) && 
      strncmp( subdetector,"MPAD",  4 ) && 
      strncmp( subdetector,"FLT",   3 ) &&
      strncmp( subdetector,"SLT",   3 ) &&
      strncmp( subdetector,"FARM",  4 ) && 
      strncmp( subdetector,"TARG",  4 ) &&
      strncmp( subdetector,"MCTR",  4 ) && 
      strncmp( subdetector,"ALL",   3 ) ){
    arteMsg << "DAQIF/SUBD: Unknown subdetector: " << subdetector ;
    arteMsg.warn();
    delete subdetector;
    return;
  }
  if( !strncmp( subdetector,"HPT",   3 ) ||
      !strncmp( subdetector,"HIPT",  4 ) ||
      !strncmp( subdetector,"TRD",   3 ) || 
      !strncmp( subdetector,"MPIX",  4 ) || 
      !strncmp( subdetector,"FLT",   3 ) ||
      !strncmp( subdetector,"SLT",   3 ) ||
      !strncmp( subdetector,"FARM",  4 ) ){
    arteMsg << "DAQIF/SUBD: Subdetector: " << subdetector  << " not supported.";
    arteMsg.warn();
    delete subdetector;
    return;
  }

  //   read in subdetector status
  len  = 0;
  len1 = 3;
  kugetc_( input, &len, len1 );
  input[len] = 0;
  char *cstatus = new char[len+1];
  strncpy( cstatus, input, len );
  cstatus[len] = '\0';
  
  if( len>0 ) for( int i=0; i<len; i++ ) cstatus[i] = toupper( input[i] );
  
  if(strncmp(cstatus,"ON",2) && strncmp(cstatus,"OFF",3)){
    arteMsg << "DAQIF/SUBD: Unknown subdetector status: " << cstatus ;
    arteMsg.warn();
    delete subdetector;
    delete cstatus;
    return;
  }

  // set the compmask
  int status = 1;
  if( !strncmp( cstatus,"OFF", 3 ) ) status=0;

  if( !strncmp( subdetector,"SVD", 3)
   || !strncmp( subdetector,"VDS", 3)
   || !strncmp( subdetector,"ALL", 3) )
    kadaqif::set_compmask(update_compmask(COMP_M_SVD,status));
  if( !strncmp( subdetector,"ITR", 3)
   || !strncmp( subdetector,"ALL", 3) )
    kadaqif::set_compmask(update_compmask(COMP_M_ITR,status));
  if( !strncmp( subdetector,"OTR", 3)
   || !strncmp( subdetector,"ALL", 3) )
    kadaqif::set_compmask(update_compmask(COMP_M_OTR,status));
  if( !strncmp( subdetector,"RICH", 4)
   || !strncmp( subdetector,"ALL", 3) )
    kadaqif::set_compmask(update_compmask(COMP_M_RICH,status));
  if( !strncmp( subdetector,"ECAL", 4)
   || !strncmp( subdetector,"ALL", 3) )
    kadaqif::set_compmask(update_compmask(COMP_M_ECAL,status));
  if( !strncmp( subdetector,"MUON", 4)
   || !strncmp( subdetector,"ALL", 3) )
    kadaqif::set_compmask(update_compmask(COMP_M_MUON,status));
  if( !strncmp( subdetector,"MPAD", 4)
   || !strncmp( subdetector,"ALL", 3) )
    kadaqif::set_compmask(update_compmask(COMP_M_MPAD,status));
  if( !strncmp( subdetector,"TARG", 4)
   || !strncmp( subdetector,"ALL", 3) )
    kadaqif::set_compmask(update_compmask(COMP_M_TARG,status));
  if( !strncmp( subdetector,"MCTR", 4)
   || !strncmp( subdetector,"ALL", 3) )
    kadaqif::set_compmask(update_compmask(COMP_M_MCTRUTH,status));

  // clean up
  delete subdetector;
  delete cstatus;

  ARTE_mark( "daqif_SUBD 1", 0);
}

extern "C"
void daqif_MCTRUTH_ECUT()
{
  ARTE_mark( "daqif_MCTRUTH_ECUT 0", 0);
  float Ecut;

  kugetr_(&Ecut);
  kadaqif::set_mctruth_Ecut(Ecut);

  ARTE_mark( "daqif_MCTRUTH_ECUT 1", 0);
}

extern "C"
void daqif_DaqFileName()
{
  ARTE_mark( "daqif_DaqFileName 0", 0);

  // read in filename
  char *filename = new char[255];
  strncpy(filename, ku_gets(),255);
  kadaqif::set_daqfile(filename);

  ARTE_mark( "daqif_DaqFileName 1", 0);
}

extern "C" void daqifkuip_();

static class daqifIniter
{
public:
  daqifIniter() { 
    atregkuip( daqifkuip_, 1 ); 
} 
} _daqifIniter;
