#include <stdio.h>
#include <strings.h>
#include <ctype.h>
#include <stdlib.h>
#include "arte/Arte.h"
#include "auto/MEMO.h"
#include "auto/MEMA.h"
#include "auto/MEMS.h"
#include "auto/MEIO.h"
#include "dbarte/DbArteBridge.h"
#include "arte/ArteFlag.h"
#include "arte/mecCount.h"

int DbLedaCompatibility = 0;

void mmbook_(int *, int *), mmrupd_(int *), mmdrop_(int *), mmfree_(int *);
void chhtot_(char *, char *, int *);
void iounit_(char *, int *, int);


DbTableDesc *DbArteTableDesc(char *tName)

/************************************************************************
**
** PARAMETERS
**	tName	 in    table name 
**
** RETURNS
**	Arte table description to be used in MIZZI
**
** FUNCTION DEFINITION
**	Returns Arte table description to be used in MIZZI.
*************************************************************************
*/

{
int tab;
static DbTableDesc TD;
char    errMess[256];
char    lastName[90],curName[90];
int	mema;
int	nCol;
int	iCol;
int	col;
int	isFirst;
int	type;
int     i;
char	oneDesc[100];
int	lastType = -1;
int	lastIlow;
int	lastIupp;
float	lastFlow;
float	lastFupp;
float	lastAccu;
int	nEq;
int	iRpt;
int	changed;
int var;


	{ /* Lookup table index */
	int n,nlink,var;
	char mess[80]="ArteTableDesc: ";
	char curName[DB_MAX_NAME_SIZE+1];
	
	  /* Look for bank name in MEMO bank. */

	 for( n = 1, nlink = 0 ; n <= NMEMO() ; n++ ) 
	 	{
    	 	var = DB_MAX_NAME_SIZE/4;
    		chhtot_(MEMO(n).name, curName, &var );
    		if( strcmp( tName, curName ) == 0 ) 
    			{
      			nlink = n; 
      			break; 
    			} 
		}

	  if( nlink == 0 ) 
	  {
    	  static int errPtr;
    	  sprintf(mess+strlen(mess),
       	        "bank %s is not booked in MEMO bank",tName);
    	  mecErrorCount( &errPtr, mess );
    	  return NULL;
  	  }
  	  
        tab=nlink;
	}

var = DB_MAX_NAME_SIZE/4;
chhtot_(MEMO(tab).name, TD.Name, &var);
chhtot_(MEMO(tab).name, TD.Comm, &var);
TD.Entries=0;

if(!DbLedaCompatibility)
	strcpy(TD.Fmt,"I:NROWS-FIELD");
else
	TD.Fmt[0]=0;

lastName[0]=0;
isFirst = 1;

mema = MEMO(tab).mema;
nCol = MEMO(tab).ocol;

nEq = 0;
changed = 0;

for (iCol=0; iCol<nCol; iCol++)
   {
   int i;
   col = mema+iCol;
   type = MEMA(col).type;
   strncpy(curName,MEMA(col).name,85);
   for(i=0;i<80;i++)
   	{
   	if(curName[i]==' ')
   		{
   		curName[i]=0;
   		break;
   		}
   	curName[i]=toupper(curName[i]);
   	}
   	
   if (type == 1 || type == 2 || type == 3 || type == 4)
      type = 1;

   /* check possible types */
   if (type <= 0 || type >= 11)
      {
      static int errPtr;
      
      sprintf(errMess, "ArteTableDesc: Table %s: unknown type %d in MEMA\n",
         tName,type);
      mecWarningCount(&errPtr, errMess);
      return(NULL);
      }

   /* 
   ** If sequence of equal attributes is empty, save attributes 
   */ 
   if (nEq == 0)
      {
      lastIlow = MEMA(col).ilow;
      lastIupp = MEMA(col).iupp;
      lastFlow = MEMA(col).flow;
      lastFupp = MEMA(col).fupp;
      lastAccu = MEMA(col).accu;
      lastType = type;
      
      strcpy(lastName,curName);
      
      changed = 0;
      nEq = 1;
      if (iCol != nCol-1)
         continue;
      }
   /* 
   ** Otherwize check that previos and current attibutes are the same 
   */ 
   else
      {
      changed = 1;
      if(strlen(lastName)==strlen(curName)
      		&& strcmp(lastName,curName)==0)
      	{
        changed = 0;
        nEq++;
        if (iCol != nCol-1)
           continue;
        }
      }


   /* 
   ** Generate description string
   */
   for (iRpt=0; iRpt<2; iRpt++)
      {
      strcpy(oneDesc,"");
      if (!isFirst)
         strcat(oneDesc,",");
      else
         isFirst = 0;
   
      if (nEq != 1)
         sprintf(oneDesc+strlen(oneDesc),"%d",nEq);

      /* generate */
      switch(lastType)

         {
         case 5:	/* int */
            strcpy(oneDesc,"I1");
            break;
         
         case 6:	/* bit */
            strcpy(oneDesc,"B1");
            break;
         
         case 7:	/* float */
            strcpy(oneDesc,"F");
            break;
         
         case 8:	/* holl */
	 case 11:
            strcpy(oneDesc,"H");
            break;
			
	  case 9:    /* short */
			strcpy(oneDesc,"S");
            break;
	  case 10:    /* ushort */
			strcpy(oneDesc,"U");
            break;
         
         case 1:
         case 2:
         case 3:
         case 4:	/* relation */
/*            if(lastType==4)
            {
            printf("4444444444: table %20.20s, field %20.20s\n",
            	MEMO(tab).name,MEMA(col).name);
            }*/
            strcpy(oneDesc,"I1");
            break;
         }

      if (strlen(TD.Fmt)+strlen(oneDesc) > TABLE_DESC_LEN)
         {
         static int errPtr;
      
         sprintf(errMess, "ArteTableDesc: Table %s: too long description\n",
            tName);
         mecWarningCount(&errPtr, errMess);
         return(NULL);
         }
      else
         {
         char MizziType[4];
         MizziType[1]=MizziType[2]=0;
         switch(oneDesc[0])
         	{
         	case 'I': MizziType[0]='I'; break;
         	case 'F': MizziType[0]='F'; break;
         	case 'B': MizziType[0]='U'; MizziType[1]='I'; break;
         	case 'H': MizziType[0]= (DbLedaCompatibility?'B':'A'); break;
			case 'S': MizziType[0]='S'; break;
			case 'U': MizziType[0]='U'; MizziType[1]='S'; break;
         	default:  MizziType[0]='?';
         	}
			
		 TD.Type[TD.Entries]=oneDesc[0];
		 if(DbLedaCompatibility && lastType==1)
			 TD.Type[TD.Entries]='R';
		 TD.Dim[TD.Entries]=nEq==1?0:nEq;
		 TD.Entries++;
		 if(MizziType[0]=='A' || MizziType[0]=='B') nEq*=4;
		 
		 if(DbLedaCompatibility)
		 {
			if(lastType!=1)
			{
			if(nEq==1)
              sprintf(TD.Fmt+strlen(TD.Fmt),
              		" %s:%s",MizziType,lastName);
            else
              {
              int i;
              for(i=0;i<nEq;i++)
              sprintf(TD.Fmt+strlen(TD.Fmt),
              		" %s:%s-%d",MizziType,lastName,i);
              }
		    }
		  }
		  else 
              sprintf(TD.Fmt+strlen(TD.Fmt),
              		" %s:%s",MizziType,lastName);
         }
      
      /* new sequence started */
      lastIlow = MEMA(col).ilow;
      lastIupp = MEMA(col).iupp;
      lastFlow = MEMA(col).flow;
      lastFupp = MEMA(col).fupp;
      lastAccu = MEMA(col).accu;
      lastType = type;
      
      strcpy(lastName,curName);
      
      /* Save the previous too if this is the last item */
      if (!(iCol == nCol-1 && changed == 1))
         break;
      nEq = 1;
      }
   nEq = 1;
   }

   for(i=0;TD.Fmt[i];i++)
        if(TD.Fmt[i]=='_')
	      TD.Fmt[i]='-';
   
   if(DbLedaCompatibility)
   {
   var=strlen(TD.Fmt);
	for(i=0;i<var;i++)
		TD.Fmt[i]=TD.Fmt[i+1];
   }

return &TD;
}


