// ********************************************************************
//
// Routine: DQ_RECO_RTRA()
// Author:  P.Conde
//
// Description: some counting on the RTRA's.
//
// Filling of bins for RTRA histogram:
//
//   Number of events: first bin.
//   Second bin: number of RTRA's.
//   3rd bin: VDS-only tracks.
//   4th:    VDS+OTR
//   5th:    VDS+OTR+ECAL
//   6th:    VDS+OTR+RICH
//   7th:    VDS+OTR+ECAL+RICH
//   8th:    VDS+ECAL
//   9th:    VDS+ECAL+RICH
//   10th:   VDS+RICH
//   11th:   OTR-only
//   12th:   OTR+ECAL
//   13th;   OTR+ECAL+RICH
//   14th;   OTR+RICH
 
//   18rd bin: VDS+MUON
//   19th:    VDS+OTR+MUON
//   20th:    VDS+OTR+ECAL+MUON
//   21th:    VDS+OTR+RICH+MUON
//   22th:    VDS+OTR+ECAL+RICH+MUON
//   23th:    VDS+ECAL+MUON
//   24th:    VDS+ECAL+RICH+MUON
//   25th:   VDS+RICH+MUON
//   26th:   OTR-only+MUON
//   27th:   OTR+ECAL+MUON
//   28th;   OTR+ECAL+RICH+MUON
//   29th;   OTR+RICH+MUON

//   33th: Number of RTRA's with VDS segment
//   34th: Number of RTRA's with OTR/ITR segment
//   35th: Number of RTRA's with ECAL segment
//   36th: Number of RTRA's with RICH segment
//   37th: Number of RTRA's with MUON segment

//   40th: Number of RTRA's without VDS segment
//   41th: Number of RTRA's without OTR/ITR segment
//   42th: Number of RTRA's without ECAL segment
//   43th: Number of RTRA's without RICH segment
//   44th: Number of RTRA's without MUON segment
//
//   46th: Number of selected RTRA's.
//
// Subdetectors histograms: for each segment store the number of times
//                          that it's used in RTRA's.
//
// changes:
// 07 Aug 2000 (AG) remove printout
// 21 Nov 2000 (PC) Muons: defined by the seeded muon reconstruction.
// 21 Nov 2000 (PC) Count selected RTRA's.
//
// ********************************************************************

// standrad includes
#include <iostream.h>
#include <stdlib.h>
#include <math.h>
#include <algo.h>


#include "arte/arte.hh"
#include "auto/auto.hh"
#include "arte/RSEG_cmp.hh"
#include "farm/farm.hh"
#include "rhp.h" 
#include "dq/DQ.h"


// ********* Histograms to be declare somewhere:

#define HOFF             2000
#define HIST_RTRA(I)     DQ_ID(COMP_RECO,LVL_REC,HOFF+I)

#define RTRA_H_ID        HIST_RTRA(10)
#define VDS_SEG_H_ID     HIST_RTRA(11)
#define OTR_SEG_H_ID     HIST_RTRA(12)
#define ECAL_SEG_H_ID    HIST_RTRA(13)
#define RICH_SEG_H_ID    HIST_RTRA(14)
//#define MUON_SEG_H_ID    HIST_RTRA(15)   

//
//
//
int
DQ_RECO_RTRA_init()
{
  // show up
  ARTE_mark( "DQ_RECO_RTRA_init 0", 0 );
  cout << "DQ_RECO_RTRA_init" << endl;

  rhp_hbook1(RTRA_H_ID,"RTRA information",50,-0.5,49.5); 
  rhp_hbook1(VDS_SEG_H_ID,"VDS segment",15,-0.5,14.5); 
  rhp_hbook1(OTR_SEG_H_ID,"OTR segment",15,-0.5,14.5);
  rhp_hbook1(ECAL_SEG_H_ID,"ECAL segment",15,-0.5,14.5); 
  rhp_hbook1(RICH_SEG_H_ID,"RICH segment",15,-0.5,14.5);
  //rhp_hbook1(MUON_SEG_H_ID,"MUON segment",15,-0.5,14.5);

  // show up
  ARTE_mark( "DQ_RECO_RTRA_init 1", 0 );

  return 0;
}

//
//
//
int
DQ_RECO_RTRA_evnt()
{

  // show up
  ARTE_mark( "DQ_RECO_RTRA_evnt 0", 0 );

  rhp_hf1(RTRA_H_ID,0.,1.);

  // Loop over RTRA's

   
  for(ArteTable<RTRA>::iterator<RTRA> rtra = ArteTable<RTRA>::begin();
      rtra != ArteTable<RTRA>::end(); ++rtra){


    int vds=0, otr=0, ecal=0, rich=0, muon=0;
    for(ArteRelationIterator<RSEG> rseg = (*rtra)->rseg.begin();
	rseg != (*rtra)->rseg.end(); ++rseg){
	
      if( ((*rseg)->cmp & Rsegc::bitsign) != 0 ) continue;
      if((*rseg)->fit == 7) continue;
      if((*rseg)->cmp & Rsegc::vxd ) vds++;
      if((*rseg)->cmp & Rsegc::patt ) otr++;
      if((*rseg)->cmp & Rsegc::rich ) rich++;
    }
    for(ArteRelationIterator<RCCL> rccl = (*rtra)->rccl.begin();
	rccl != (*rtra)->rccl.end(); ++rccl){
      if((*rccl)->esum > 1) ecal++; // Count only clusters with more than 1GeV.
    }

    if((*rtra)->flm == 3 || (*rtra)->flm == 13) muon++;

    rhp_hf1(RTRA_H_ID,1.,1.);


    if( vds !=0 &&  otr == 0 && rich == 0 && ecal == 0 && muon == 0 )
      rhp_hf1(RTRA_H_ID,3.,1.);

    if( vds !=0 &&  otr !=0 && rich == 0 && ecal == 0 && muon ==0 )
      rhp_hf1(RTRA_H_ID,4.,1.);

    if( vds !=0 &&  otr !=0 && rich == 0 && ecal != 0 && muon ==0 )
      rhp_hf1(RTRA_H_ID,5.,1.);

    if( vds !=0 &&  otr !=0 && rich != 0 && ecal == 0 && muon ==0 )
      rhp_hf1(RTRA_H_ID,6.,1.);

    if( vds !=0 &&  otr !=0 && rich != 0 && ecal != 0 && muon ==0 ) 
      rhp_hf1(RTRA_H_ID,7.,1.);

    if( vds !=0 &&  otr ==0 && rich == 0 && ecal != 0 && muon ==0 )
      rhp_hf1(RTRA_H_ID,8.,1.);

    if( vds !=0 &&  otr ==0 && rich != 0 && ecal != 0 && muon ==0 )
      rhp_hf1(RTRA_H_ID,9.,1.);

    if( vds !=0 &&  otr ==0 && rich != 0 && ecal == 0 && muon ==0 ) 
      rhp_hf1(RTRA_H_ID,10.,1.);

    if( vds ==0 &&  otr !=0 && rich == 0 && ecal == 0 && muon ==0 ) 
      rhp_hf1(RTRA_H_ID,11.,1.);

    if( vds ==0 &&  otr !=0 && rich == 0 && ecal != 0 && muon ==0 ) 
      rhp_hf1(RTRA_H_ID,12.,1.);

    if( vds ==0 &&  otr !=0 && rich != 0 && ecal != 0 && muon ==0 )
      rhp_hf1(RTRA_H_ID,13.,1.);

    if( vds ==0 &&  otr !=0 && rich != 0 && ecal == 0 && muon ==0 )
      rhp_hf1(RTRA_H_ID,14.,1.);



    if( vds !=0 &&  otr == 0 && rich == 0 && ecal == 0 && muon != 0 ) 
      rhp_hf1(RTRA_H_ID,18.,1.);

    if( vds !=0 &&  otr !=0 && rich == 0 && ecal == 0 && muon !=0 )
      rhp_hf1(RTRA_H_ID,19.,1.);

    if( vds !=0 &&  otr !=0 && rich == 0 && ecal != 0 && muon !=0 ) 
      rhp_hf1(RTRA_H_ID,20.,1.);

    if( vds !=0 &&  otr !=0 && rich != 0 && ecal == 0 && muon !=0 ) 
      rhp_hf1(RTRA_H_ID,21.,1.);

    if( vds !=0 &&  otr !=0 && rich != 0 && ecal != 0 && muon !=0 ) 
      rhp_hf1(RTRA_H_ID,22.,1.);

    if( vds !=0 &&  otr ==0 && rich == 0 && ecal != 0 && muon !=0 ) 
      rhp_hf1(RTRA_H_ID,23.,1.);

    if( vds !=0 &&  otr ==0 && rich != 0 && ecal != 0 && muon !=0 ) 
      rhp_hf1(RTRA_H_ID,24.,1.);

    if( vds !=0 &&  otr ==0 && rich != 0 && ecal == 0 && muon !=0 ) 
      rhp_hf1(RTRA_H_ID,25.,1.);

    if( vds ==0 &&  otr !=0 && rich == 0 && ecal == 0 && muon !=0 ) 
      rhp_hf1(RTRA_H_ID,26.,1.);

    if( vds ==0 &&  otr !=0 && rich == 0 && ecal != 0 && muon !=0 ) 
      rhp_hf1(RTRA_H_ID,27.,1.);

    if( vds ==0 &&  otr !=0 && rich != 0 && ecal != 0 && muon !=0 ) 
      rhp_hf1(RTRA_H_ID,28.,1.);

    if( vds ==0 &&  otr !=0 && rich != 0 && ecal == 0 && muon !=0 )
      rhp_hf1(RTRA_H_ID,29.,1.);

    
    // Number of RTRA's that have the i_th subdetector:

    if( vds != 0) rhp_hf1(RTRA_H_ID,33.,1.);
    if( otr != 0) rhp_hf1(RTRA_H_ID,34.,1.);
    if( ecal != 0) rhp_hf1(RTRA_H_ID,35.,1.);
    if( rich != 0) rhp_hf1(RTRA_H_ID,36.,1.);
    if( muon != 0) rhp_hf1(RTRA_H_ID,37.,1.);

    // Number of RTRA's that doesn't have the i_th subdetector:

    if( vds == 0) rhp_hf1(RTRA_H_ID,40.,1.);
    if( otr == 0) rhp_hf1(RTRA_H_ID,41.,1.);
    if( ecal == 0) rhp_hf1(RTRA_H_ID,42.,1.);
    if( rich == 0) rhp_hf1(RTRA_H_ID,43.,1.);
    if( muon == 0) rhp_hf1(RTRA_H_ID,44.,1.);

    // Check if the RTRA was selected: 

    int Sel = (*rtra)->fit/10000;
    if( Sel == 1 )  rhp_hf1(RTRA_H_ID,46.,1.);


  } // End of loop in RTRA's.



  // Loop over the reconstructed segments:
 
  for(ArteTable<RSEG>::iterator<RSEG> rseg = ArteTable<RSEG>::begin();
      rseg != ArteTable<RSEG>::end(); ++rseg){

      if(((*rseg)->cmp & Rsegc::bitsign) != 0 ) continue;
      if((*rseg)->fit == 7) continue;
      if((*rseg)->cmp & Rsegc::vxd ) {
	
	int used =0;
	for(ArteRelationIterator<RTRA> rtra = (*rseg)->rtra.begin();
	  rtra != (*rseg)->rtra.end(); rtra++){
	  used++;
	}
	// fill VDS histogram:
	if(used < 14) rhp_hf1(VDS_SEG_H_ID,used,1.);
	else rhp_hf1(VDS_SEG_H_ID,14.,1.);
      }

      if((*rseg)->cmp & Rsegc::patt ) {

	int used =0;
	for(ArteRelationIterator<RTRA> rtra = (*rseg)->rtra.begin();
	  rtra != (*rseg)->rtra.end(); rtra++){
	  used++;
	}
	// fill  histogram:
	if(used < 14) rhp_hf1(OTR_SEG_H_ID,used,1.);
	else rhp_hf1(OTR_SEG_H_ID,14.,1.);
      }

      if((*rseg)->cmp & Rsegc::rich ) {

	int used =0;
	for(ArteRelationIterator<RTRA> rtra = (*rseg)->rtra.begin();
	  rtra != (*rseg)->rtra.end(); rtra++){
	  used++;
	}
	// fill  histogram:
	if(used < 14) rhp_hf1(RICH_SEG_H_ID,used,1.);
	else rhp_hf1(RICH_SEG_H_ID,14.,1.);
      }
      
  }// End of loop in segments.

  // Loop in clusters:

  for(ArteTable<RSEG>::iterator<RCCL> rccl = ArteTable<RCCL>::begin();
      rccl != ArteTable<RCCL>::end(); ++rccl){
    
    int used = 0;
    for(ArteRelationIterator<RTRA> rtra = (*rccl)->rtra.begin();
	rtra != (*rccl)->rtra.end(); rtra++){
      used++;
    }

    if(used < 14) rhp_hf1(ECAL_SEG_H_ID,used,1.);
    else rhp_hf1(ECAL_SEG_H_ID,14.,1.);

  }// End of loop in clusters.

  // show up
  ARTE_mark( "DQ_RECO_RTRA_evnt 1", 0 );

  return 0;

} // End of function DQ_FILL_RTRA_hist.

