 
// *****************************************************************************
//
// source:    DQ_marple.C
//
// type:      source code
//
// created:   23 Nov 2000
//
// author:    Dr. Andreas Gellrich
//            HERA-B Collaboration
//            Humboldt-University Berlin
//            Physics Department
//            Invalidenstr. 110
//            10115 Berlin
//            Germany
//            email: Andreas.Gellrich@desy.de
//
// called by: DQ_init(), MARPLE
//
// hints:     use histogram IDs 7000-7999
//            steering is done by '/DQ/SUBD RECO ON rec'
//            DQ_marple_init() is called automatically in DQ_init()
//            DQ_marple_evnt( args ) should be called in MARPLE
//
// changes:
// 23 Nov 2000 (AG) creation of template
// 24 Nov 2000 (AG) cured naming bug
// 25 Nov 2000 (OI) implementing real code
//
// ******************************************************************

// standard includes
#include <iostream.h>
#include <stdlib.h>
#include <stdio.h>

// ONLINE includes
#include "rhp.h"

// Arte includes
#include "arte/arte.hh"
#include "farm/farm.hh"
#include "dq/DQ.h"
#include "dq/DQ_RECO.h"

#include "marple/marple.hh"

//
// init function (called by DQ_init())
//
int
DQ_RECO_marple_init()
{
  // show up
  ARTE_mark( "DQ_marple_init 0", 0 );
  cout << "DQ_marple_init" << endl;

  // booking 
  if( matchMARPLEVdsPc )
    {
      rhp_hbook1( DQ_ID( COMP_RECO, LVL_REC, 7001 +10*MARPLEIDVDSPC ), 
		  "MARPLE: VDS-PC, dX z=455", 50, -4, 4 );
      rhp_hbook1( DQ_ID( COMP_RECO, LVL_REC, 7002  +10*MARPLEIDVDSPC ), 
		  "MARPLE: VDS-PC, dY z=45", 50, -6, 6 );
    }

  if( matchMARPLEVdsOtr )
    {
      rhp_hbook1( DQ_ID( COMP_RECO, LVL_REC, 7001 +10*MARPLEIDVDSOTR ), 
		  "MARPLE: VDS-OTR, dX z=160", 50, -3, 3 );
      rhp_hbook1( DQ_ID( COMP_RECO, LVL_REC, 7002 +10*MARPLEIDVDSOTR ), 
		  "MARPLE: VDS-OTR, dY z=160", 50, -20, 20 );
      rhp_hbook1( DQ_ID( COMP_RECO, LVL_REC, 7003  +10*MARPLEIDVDSOTR ), 
		  "MARPLE: VDS-OTR, dTX z=160", 50, -0.01, 0.01 );
      rhp_hbook1( DQ_ID( COMP_RECO, LVL_REC, 7004  +10*MARPLEIDVDSOTR ), 
		  "MARPLE: VDS-OTR, dTY z=160", 50, -0.030, 0.030 );
    }
  
  if( matchMARPLEVdsEcal )
    {
      rhp_hbook1( DQ_ID( COMP_RECO, LVL_REC, 7002  +10*MARPLEIDVDSECAL ),
		  "MARPLE: VDS-ECAL, dY,  z=ECAL", 50, -6, 6 );
    }
  
  if( matchMARPLEVdsRich )
    {
      rhp_hbook1( DQ_ID( COMP_RECO, LVL_REC, 7004 +10*MARPLEIDVDSRICH  ), 
		  "MARPLE: VDS-RICH, dTY z=RICH, no RICH ID", 50, -0.01, 0.01 );
    }
  
  if( matchMARPLEOtrRich )
    {
      rhp_hbook1( DQ_ID( COMP_RECO, LVL_REC, 7003  +10*MARPLEIDOTRRICH ), 
		  "MARPLE: OTR-RICH, dTX z=RICH", 50, -0.01, 0.01 );
      rhp_hbook1( DQ_ID( COMP_RECO, LVL_REC, 7004  +10*MARPLEIDOTRRICH ), 
		  "MARPLE: OTR-RICH, dTY z=RICH", 50, -0.03, 0.03 );
    }
  
  if( matchMARPLEEcalRich )
    {
      rhp_hbook1( DQ_ID( COMP_RECO, LVL_REC, 7004  +10*MARPLEIDVDSPC ), 
		  "MARPLE: RICH-ECAL, dTY z=ECAL", 50, -0.01, 0.01 );
    }
  
  if( matchMARPLEOtrEcal )
    {
      rhp_hbook1( DQ_ID( COMP_RECO, LVL_REC, 7001  +10*MARPLEIDVDSPC ), 
		  "MARPLE: OtR-ECAL, dX z=ECAL", 50, -10, 10 );
      rhp_hbook1( DQ_ID( COMP_RECO, LVL_REC, 7002  +10*MARPLEIDVDSPC ), 
		  "MARPLE: OtR-ECAL,INNER, dY z=ECAL", 50, -20, 20 );
    }

  // show up
  ARTE_mark( "DQ_marple_init 1", 0 );
  return 0; // return error code (0 = okay)
}

//
// event function (called in MARPLE)
// type defines match ( see marple/marple.hh for details )
// extra is used to choos histogram withit same math
// order of variables : 
//     1 -dX; 2 -dY; 3 - dTX; 4 - dTY; 5 - dPE

// When histograms are filled the following cuts are applied (if used by MARPLE)
//  dTY  - all combinations no cuts
//  dY   - all combinations passed dTY cut
//  dTX  - all combinations passed dTY, dY cuts
//  dX   - all combinations passed dTY, dY, dTX cuts
//  dPE  - all combinations passed dTY, dY, dTX, dX cuts
//  chi2 cut as well as cut on Npairs is not used usually at that time.
// if energy delta cut is used instead of usual delta cut histograms are not filled

int
DQ_RECO_marple_evnt( MARPLETYPEOFMATCH type, int var, float value )
{
  // show up
  ARTE_mark( "DQ_marple_evnt 0", 0 );

  // check flag
  if( DQ_flag( COMP_RECO, LVL_M_REC ) ){
    ARTE_mark( "DQ_marple_evnt 1", 0 );
    return 0;
  }
  
  //
  // filling 
  //

  int histId = 7000 + 10*int(type) + var;
  rhp_hf1( DQ_ID( COMP_RECO, LVL_REC, histId), value, 1 );
  

  // show up
  ARTE_mark( "DQ_marple_evnt 1", 0 );
  return 0; // return error code (0 = okay)
}
