
// ****************************************************************************
//
// source:    usevnt.C
//
// type:      source code
//
// author:    Andreas Gellrich
//            HERA-B Collaboration
//            Humboldt-University Berlin
//            Physics Department
//            Invalidenstr. 110
//            10115 Berlin
//            Germany
//            email: Andreas.Gellrich@desy.de
//
// purpose:   usevnt calling the event reconstruction
//
// called by: ARTE
//
// changes:
// 19 Jan 2001 (AG) added comments
//
// ****************************************************************************

// standard includes
#include <iostream.h> 

// ARTE includes
#include "arte/ArteFlag.h"
#include "farm/farm.hh"

// prototypes
extern "C" void rcevnt_();
extern "C" int  didraw_();

// externals
extern int CYCLE; // cycling for online event display

//
// program code
// 
extern "C" void
usevnt_()
{
  ARTE_mark( "usevnt BEGIN", 0 );

  //-------------------------------------------------------------------
  // print event info
  //-------------------------------------------------------------------
  cout << endl;
  cout << "###########################################################" << endl;
  cout << "usevnt_: "
       << "nevt="  << arteflag_.nevents  << " " 
       << "run="   << faDDAQrun_number() << " "
       << "exp="   << faDDAQexp_number() << " "
       << "time="  << faDDAQevent_time() << " "
       << "key="   << faDDAQdbKey()      << " "
       << "rev="   << faDDAQdbVersion()  << " "
       << "slp="   << faDDAQslp_node()   << " "
       << "farm="  << faDDAQfarm_node()  << " "
       << "tcode=" << faDDAQFCS_tcode()  << " "
       << "BX="    << faDDAQphysBX()     << " "
       << "ia="    << faDTARia()         << endl;

  //-------------------------------------------------------------------
  // invoke event reconstruction
  //-------------------------------------------------------------------
  ARTE_mark( "rcevnt", 0 );
  rcevnt_();

  //-------------------------------------------------------------------
  // print table info
  //-------------------------------------------------------------------
  faPrintNrow();
  //faPrintEVRC();
  //faPrintEVTI();
  
  //-------------------------------------------------------------------  
  // event display (automatic cycling over events)
  //-------------------------------------------------------------------  
  if( CYCLE ){
    ARTE_mark( "didraw", 0 );
    didraw_();    
  }
  
  // return
  ARTE_mark( "usevnt END", 0 );
  return;
}
