#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include "arte/arte.hh"
#include "arte/ArteUtil.h"
#include "arte/ArteMessage.hh"
#include "gean/zgeom.h"

vector<hgdet>  *zgdet::ptdet[33];


/*************************************************************************/
void zgdet::ini(int n){ 
     ptdet[ind_pt] = new vector<hgdet>;
     ptdet[ind_pt]->reserve(n); 
     if(ptdet[ind_pt]==NULL) {
        arteMsg << "no hgdet[n] allocated";
        arteMsg.error();      
     }
}
/*************************************************************************/
void zgdet::fin() {
     delete ptdet[ind_pt];
}
/*************************************************************************/
void zgdet::increase() {
     hgdet HGDET;
     ptdet[ind_pt]->push_back(HGDET);
}
/*************************************************************************/
void zgdet::Set_default() {
     vector<hgdet>::iterator det = ptdet[ind_pt]->end()-1;
     memset(det,0,sizeof(hgdet));
     det->sides = 1; 
}
/*************************************************************************/ 
void zgdet::Copy_previous() {
     if (ptdet[ind_pt]->size() >1 ) {
     vector<hgdet>::iterator det = ptdet[ind_pt]->end()-1;
     vector<hgdet>::iterator old = ptdet[ind_pt]->end()-2;
     memcpy(det,old,sizeof(hgdet)); 
     }
     else { 
        arteMsg <<"zgdet::Copy previous: previous does not exist";
        arteMsg.error();
     }
}
/*************************************************************************/
void zgdet::Set_vol_det_Rel() {
     vector<hgvol>::iterator vol = zgvol::ptvol[ind_pt]->end()-1;
     vector<hgdet>::iterator det = ptdet[ind_pt]->end()-1;
     vol->index_hgdet = det - ptdet[ind_pt]->begin() + 1;
     det->index_hgvol =  zgvol::ptvol[ind_pt]->size();
}
/*************************************************************************/
void zgdet::Read_sens_dim(char *card) {
     vector<hgvol>::iterator vol = zgvol::ptvol[ind_pt]->end()-1;
     vector<hgdet>::iterator det = ptdet[ind_pt]->end()-1;
     if(vol->npar == 3) {
       int nitem=sscanf(card,"%f %f",&det->sensdx,&det->sensdy); 
       if (nitem!=2) {
          arteMsg <<"check data on sens_dimension cards";
          arteMsg.error();
       } 
     }   
}
/*************************************************************************/
void zgdet::Read_sens_offset(char* card){ 
     vector<hgvol>::iterator vol = zgvol::ptvol[ind_pt]->end()-1;
     vector<hgdet>::iterator det = ptdet[ind_pt]->end()-1;
     if(vol->npar == 3) {
       int nitem=sscanf(card,"%f%f",&det->sens_offset_x,&det->sens_offset_y); 
       if (nitem!=2) {
          arteMsg <<"check data on sens_offset cards";
          arteMsg.error();
       } 
     }   
}
/*************************************************************************/
void zgdet::Read_sides(char *card) {
     vector<hgdet>::iterator det = ptdet[ind_pt]->end()-1;
     int nitem=sscanf(card,"%d",&det->sides);
     if (nitem!=1) {
        arteMsg <<"check data on side cards";        arteMsg.error();
     } 
}
/*************************************************************************/
void zgdet::Read_x_wire(char *card) {
     vector<hgvol>::iterator vol = zgvol::ptvol[ind_pt]->end()-1;
     vector<hgdet>::iterator det = ptdet[ind_pt]->end()-1;
     float xw;
     for (int i=0;i<det->sides;i++) {
         int nitem=sscanf(card,"%f",&xw);
         det->x_wire[i] = xw + vol->rel_px;
         if (nitem!=1) {
            arteMsg <<"check data on x_wire cards";
            arteMsg.error();
         } 
         card+=itemlength(card,nitem);
     }
}
/*************************************************************************/
void zgdet::Read_z_wire(char *card) {
     vector<hgvol>::iterator vol = zgvol::ptvol[ind_pt]->end()-1;
     vector<hgdet>::iterator det = ptdet[ind_pt]->end()-1;
     float zw;
     for (int i=0;i<det->sides;i++) {
         int nitem=sscanf(card,"%f",&zw);
         det->z_wire[i] = zw + vol->rel_pz;
         if (nitem!=1) {
            arteMsg <<"check data on z_wire cards";      arteMsg.error();
         } 
         card+=itemlength(card,nitem);
     }
}
/*************************************************************************/
void zgdet::Read_n_wire(char *card) {
     vector<hgdet>::iterator det = ptdet[ind_pt]->end()-1;
         for (int i=0;i<det->sides;i++) {
            int nitem=sscanf(card,"%d",&det->n_wire[i]);
            if (nitem!=1) {
                arteMsg <<"check data on n_wire cards";
                arteMsg.error();} 
            card+=itemlength(card,nitem);}}
/*************************************************************************/
void  zgdet::Read_y1_wire(char *card) {
     vector<hgdet>::iterator det = ptdet[ind_pt]->end()-1;
         for (int i=0;i<det->sides;i++) {
            int nitem=sscanf(card,"%f",&det->y1_wire[i]);
            if (nitem!=1) {
                arteMsg <<"check data on y1_wire cards";
                arteMsg.error();} 
            card+=itemlength(card,nitem);}}
/*************************************************************************/
void  zgdet::Read_y2_wire(char *card) {
     vector<hgdet>::iterator det = ptdet[ind_pt]->end()-1;
         for (int i=0;i<det->sides;i++) {
            int nitem=sscanf(card,"%f",&det->y2_wire[i]);
            if (nitem!=1) {
                arteMsg <<"check data on y2_wire cards";
                arteMsg.error();} 
            card+=itemlength(card,nitem);}}
/*************************************************************************/
void  zgdet::Read_wire_orientation(char *card){  
     vector<hgdet>::iterator det = ptdet[ind_pt]->end()-1;
         for (int i=0;i<det->sides;i++) {
              int nitem=sscanf(card,"%f",&det->wire_orient[i]);
              if (nitem!=1) {
                arteMsg <<"check data on wire_orient cards";
                arteMsg.error();}
              card+=itemlength(card,nitem); }}
/*************************************************************************/
void  zgdet::Read_polarity(char *card)    {
     vector<hgdet>::iterator det = ptdet[ind_pt]->end()-1;
         for (int i=0;i<det->sides;i++) {
              int nitem=sscanf(card,"%f",&det->polarity[i]);
              if (nitem!=1) {
                 arteMsg <<"check data on polarity card";
                 arteMsg.error();} 
              card+=itemlength(card,nitem);}}
/*************************************************************************/
void  zgdet::Read_pitch(char *card) {
     vector<hgdet>::iterator det = ptdet[ind_pt]->end()-1;
        for (int i=0;i<det->sides;i++) {
             int nitem=sscanf(card,"%f",&det->pitch[i]);
             if (nitem!=1) {
                 arteMsg <<"check data on pitch cards";
                 arteMsg.error();}         
             card+=itemlength(card,nitem);}}
/*************************************************************************/
void  zgdet::Read_effectivity(char *card) {
     vector<hgdet>::iterator det = ptdet[ind_pt]->end()-1;
        int nitem=sscanf(card,"%f",&det->effectivity);
        if (nitem!=1) {
            arteMsg <<"check data on effectivity cards";
            arteMsg.error();}  }
/*************************************************************************/
void  zgdet::Read_accuracy(char *card) {
     vector<hgdet>::iterator det = ptdet[ind_pt]->end()-1;
        int nitem=sscanf(card,"%f",&det->accuracy);
        if (nitem!=1) {
            arteMsg <<"check data on accuracy cards";
            arteMsg.error();}  }
/*************************************************************************/
void  zgdet::Read_double_hit_res(char *card) {
     vector<hgdet>::iterator det = ptdet[ind_pt]->end()-1;
        int nitem=sscanf(card,"%f",&det->double_hit_res);
        if (nitem!=1) {
            arteMsg <<"check data on double_hit_res cards";
            arteMsg.error();}   }
/*************************************************************************/
void zgdet::Get_sens_dim(int &nr, float &sens_dx, float &sens_dy) {
     vector<hgvol>::iterator vol = zgvol::ptvol[ind_pt]->begin() + nr -1;
     vector<hgdet>::iterator det = ptdet[ind_pt]->begin() + vol->index_hgdet-1;
     sens_dx = det->sensdx;
     sens_dy = det->sensdy;
}
/*************************************************************************/
void zgdet::Get_sens_offset(int &nr,float &sens_of_x,float &sens_of_y) {
     vector<hgvol>::iterator vol = zgvol::ptvol[ind_pt]->begin() + nr -1;
     vector<hgdet>::iterator det = ptdet[ind_pt]->begin() + vol->index_hgdet-1;
     sens_of_x = det->sens_offset_x;
     sens_of_y = det->sens_offset_y;
}
/*************************************************************************/
int  zgdet::Get_sides(int &nr) {
     vector<hgvol>::iterator vol = zgvol::ptvol[ind_pt]->begin() + nr -1;
     vector<hgdet>::iterator det = ptdet[ind_pt]->begin() + vol->index_hgdet-1;
     return det->sides;
}
/*************************************************************************/
void zgdet::Get_x_wire(int &nr,float x_wire[],int n_el ) {
     vector<hgvol>::iterator vol = zgvol::ptvol[ind_pt]->begin() + nr -1;
     vector<hgdet>::iterator det = ptdet[ind_pt]->begin() + vol->index_hgdet-1;
     for (int i=0;i<n_el;i++) { 
         x_wire[i] = det->x_wire[i];
     }
} 
/*************************************************************************/
void zgdet::Get_z_wire(int &nr,float z_wire[],int n_el ) {
     vector<hgvol>::iterator vol = zgvol::ptvol[ind_pt]->begin() + nr -1;
     vector<hgdet>::iterator det = ptdet[ind_pt]->begin() + vol->index_hgdet-1;
     for (int i=0;i<n_el;i++) { 
         z_wire[i] = det->z_wire[i];
     }
} 
/*************************************************************************/
void zgdet::Get_n_wire(int &nr,int   n_wire[],int n_el ) {
     vector<hgvol>::iterator vol = zgvol::ptvol[ind_pt]->begin() + nr -1;
     vector<hgdet>::iterator det = ptdet[ind_pt]->begin() + vol->index_hgdet-1 ;
     for (int i=0;i<n_el;i++) { 
         n_wire[i] = det->n_wire[i];
     }
} 
/*************************************************************************/
void zgdet::Get_y1_wire(int &nr,float y1_wire[],int n_el ) {
     vector<hgvol>::iterator vol = zgvol::ptvol[ind_pt]->begin() + nr -1;
     vector<hgdet>::iterator det = ptdet[ind_pt]->begin() + vol->index_hgdet-1;
     for (int i=0;i<n_el;i++) { 
         y1_wire[i] = det->y1_wire[i];
     }
} 
/*************************************************************************/
void zgdet::Get_y2_wire(int &nr,float y2_wire[],int n_el ) {
     vector<hgvol>::iterator vol = zgvol::ptvol[ind_pt]->begin() + nr -1;
     vector<hgdet>::iterator det = ptdet[ind_pt]->begin() + vol->index_hgdet-1;
     for (int i=0;i<n_el;i++) { 
         y2_wire[i] = det->y2_wire[i];
     }
} 
/*************************************************************************/
void zgdet::Get_wire_orientation(int &nr,float wire_orient[],int n_el){  
     vector<hgvol>::iterator vol = zgvol::ptvol[ind_pt]->begin() + nr -1;
     vector<hgdet>::iterator det = ptdet[ind_pt]->begin() + vol->index_hgdet-1;
     for (int i=0;i<n_el;i++) { 
         wire_orient[i] = det->wire_orient[i];
     }
} 
/*************************************************************************/
void zgdet::Get_polarity(int &nr,float polarity[],int n_el)    {
     vector<hgvol>::iterator vol = zgvol::ptvol[ind_pt]->begin() + nr -1;
     vector<hgdet>::iterator det = ptdet[ind_pt]->begin() + vol->index_hgdet-1;
     for (int i=0;i<n_el;i++) { 
         polarity[i] = det->polarity[i];
     }
} 
/*************************************************************************/
void zgdet::Get_pitch(int &nr, float pitch[],int n_el) {
     vector<hgvol>::iterator vol = zgvol::ptvol[ind_pt]->begin() + nr -1;
     vector<hgdet>::iterator det = ptdet[ind_pt]->begin() + vol->index_hgdet-1; 
     for (int i=0;i<n_el;i++) { 
         pitch[i] = det->pitch[i];
     }
} 
/*************************************************************************/
float zgdet::Get_effectivity(int &nr) {
     vector<hgvol>::iterator vol = zgvol::ptvol[ind_pt]->begin() + nr -1;
     vector<hgdet>::iterator det = ptdet[ind_pt]->begin() + vol->index_hgdet-1;
     return det->effectivity;
}
/*************************************************************************/
float zgdet::Get_accuracy(int &nr) {
     vector<hgvol>::iterator vol = zgvol::ptvol[ind_pt]->begin() + nr -1;
     vector<hgdet>::iterator det = ptdet[ind_pt]->begin() + vol->index_hgdet-1;
     return det->accuracy;
}
/*************************************************************************/
float zgdet::Get_double_hit_res(int &nr) {
     vector<hgvol>::iterator vol = zgvol::ptvol[ind_pt]->begin() + nr -1;
     vector<hgdet>::iterator det = ptdet[ind_pt]->begin() + vol->index_hgdet-1;
     return det->double_hit_res;
}
/*************************************************************************/
void  zgdet::FillSensParameter(int &nr, const ArtePointer<NEDE> nd) {
     vector<hgvol>::iterator vol = zgvol::ptvol[ind_pt]->begin() + nr -1;
     vector<hgdet>::iterator det = ptdet[ind_pt]->begin() + vol->index_hgdet-1;

     nd->nsl = det->sides; 

     float sxmn=-1.e+9, sxmx=1.e+9, symn=-1.e+9,  symx= 1.e+9;
     if(det->sensdx>0.) { 
        sxmn = .5*(nd->xmin+nd->xmax - det->sensdx) + det->sens_offset_x; 
        sxmx = .5*(nd->xmin+nd->xmax + det->sensdx) + det->sens_offset_x;
     }
     if(det->sensdy>0.) { 
        symn = .5*(nd->ymin+nd->ymax - det->sensdy) + det->sens_offset_y; 
        symx = .5*(nd->ymin+nd->ymax + det->sensdy) + det->sens_offset_y;
     }

     for(int m=0;m<nd->nsl;m++) {
        if(!det->n_wire[m]) continue; 
        float cosx =cos(AR_DEGRAD*det->wire_orient[m]);
        float sinx =sin(AR_DEGRAD*det->wire_orient[m]);
        if(!m) {
           nd->cos  = cosx;     nd->sin  = sinx;
        } 
        float x_center = 
                      .5*((nd->xmin+nd->xmax)*cosx - (nd->ymin+nd->ymax)*sinx);
        float y_center=
                      .5*((nd->xmin+nd->xmax)*sinx + (nd->ymin+nd->ymax)*cosx);
        nd->xfw[m] = det->x_wire[m] + x_center;

        if(det->sensdx <= 0. || sxmn <= nd->xmin) 
                                              sxmn=nd->xfw[m]-.5*det->pitch[m];
        if(det->sensdx <= 0. || sxmx >= nd->xmax) 
	                 sxmx = nd->xfw[m] +(det->n_wire[m]-0.5)*det->pitch[m];
        if(fabs(det->y2_wire[m]-det->y1_wire[m]) > 1.e-5) { 
	   nd->y1w[m] = det->y1_wire[m] + y_center;
           nd->y2w[m] = det->y2_wire[m] + y_center;
        }
        else { 
           if(det->sensdy <=0.) {
              nd->y1w[m]=nd->ymin; nd->y2w[m] = nd->ymax;
           }
           else             {
              nd->y1w[m]=symn;      nd->y2w[m] = symx;
           }
        }
        if(det->sensdy <= 0.) {
           symn = nd->y1w[m];      symx = nd->y2w[m];
        }
        nd->zfw[m] = 0.5*(nd->zmin+nd->zmax) + det->z_wire[m]; 
	nd->nwi[m]=  det->n_wire[m];
     }          
     nd->sxmn = sxmn; nd->sxmx = sxmx; nd->symn = symn; nd->symx = symx;

     nd->pit = det->pitch[0];
     nd->acc = det->accuracy;
     nd->eff = det->effectivity; 
     nd->dod = det->double_hit_res;
}
