*CMZ :  1.00/01 10/08/95  00.53.51  by  S.B.
*-- Author :    Alexander Snizhko   09/03/95
************************************************************************
*
* L1MUMSC
* written by Alexander Snizhko, 09/03/95 22.28.58
*
* This routines action: Makes allowance for multiple scattering in iron
*
*
* Arguments  kind         description
*          (in,update,out)
*----------------------------------------------------------------------*
* var1        in           this variable will only be read
*
* iAlgo       in              algorythm number
* xold        in              x coordinate of the last point found
* yold        in              y coordinate of the last point found
* zold        in              z - coordinate (all the same)
* z1old       in              z coordinate of the last but one point found
* z           in              z position for prediction
* xMin        in,out          left point of prediction
* xMax        in,out          right point of the prediction
* y0          in,out          low point of prediction
* sy          in,out          height of prediction
*
************************************************************************
c
      SUBROUTINE L1MUMSC(iAlgo,xold,yold,zold,z1old,z,xMin,xMax,y0,sy)
c
      IMPLICIT NONE
#include "l1si/l1param.fi"
      INTEGER iAlgo
      REAL xold,yold,zold,z1old,z,xMin,xMax,y0,sy,
     $     sigmax,sigmay,alpha,pcut,r
      REAL sigma1,sigma2,sigma3
      REAL xshield
      INTEGER isl,ipred
      REAL XDeltaMU, YDeltaMU

      REAL A(4,3),B(4,3)
      REAL X0
      REAL zShieldMU3
      REAL zShieldMU1
      REAL xRadLenCalo
      REAL X0Calo
      REAL XnoMultScat
      DATA A / 760., 880., 980., 1070.,        ! for MU3 -> MU1
     &         840., 960., 1050.,1150.,        ! for MU1 -> TC2
     &         570., 720., 840., 930./         ! for TC2 -> TC1

      DATA B / -5520.,-6300.,-7040.,-7730.,    ! for MU3 -> MU1
     &         -6050.,-6950.,-7550.,-8270.,    ! for MU1 -> TC2
     &         -3700.,-4800.,-5580.,-6340./    ! for TC2 -> TC1

      DATA X0 / 1.76 /  ! X0 for iron
      DATA zShieldMU3 /188./ ! width of iron between MU3 and MU1
      DATA zShieldMU1 /120./ ! width of iron between MU1 and ECAL
      DATA xRadLenCalo /20./ ! number or X0 in Calo
      DATA X0Calo /1.6 /     ! X0 for calorimeter
      DATA XnoMultScat / 120. /
c
c
c Superlayer  4  -->  TC2
c             6  -->  MU1
c             7  -->  MU3
c

      if( iAlgo.ne.1.and.iAlgo.ne.2.and.iAlgo.ne.3 )THEN
         WRITE (L1LUNprn,123) ' L1MUMSC ==> wrong algorythm ',
     &    iAlgo
 123     FORMAT(2x,A60,i5)
         RETURN
      endif

c++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
c
c Calculation or low momentum cut
c -------------------------------
c
      r=sqrt(xold*xold+yold*yold) ! radius of last point
c
c     Algorythm is used as a pointer in multiple scattering parameter
c     array
c
      isl = iAlgo
c
c Number of prediction function we will use
c ( influences efficiency )  :
c
      ipred = L1MUMSCpar(isl)
c
c The very calculation of low momentum cut
c in the next superlayer
c
      pcut=A(ipred,isl)/r+B(ipred,isl)/r/r
c
c Cut on pcut=pcut(r) at small momenta ( <=> large R )
c
c      if(pcut.lt.3.and.iAlgo.eq.1) pcut = 3.
c      if(pcut.lt.5.and.(iAlgo.eq.2.or.iAlgo.eq.3)) pcut = 5.
c
      IF( pcut .LT. T1MUTHRpar(isl) )pcut=T1MUTHRpar(isl)
c
c++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
c
c
c Prediction part
c ---------------
c
      IF(iAlgo.eq.1) THEN ! prediction of sigma MU3 -> MU1

         xShield=zShieldMU3*sqrt(1+((xold-xMin)/(zold-z))**2+((yold-y0)
     $        /(zold-z))**2)
         alpha=0.0136/pcut*sqrt(xshield/X0)*(1.+0.038*alog(xshield/X0))
         sigmax = alpha*(zold-z)/1.7
         sigmay = sigmax

      ELSE IF(iAlgo.eq.2) THEN ! prediction of sigma MU1 -> TC2
c
c The are 3 parts:sigma for iron(MU3-MU1),iron(MU1-TC2) and calorimeter
c

c
c MU3 - MU1 part
c
         xShield=zShieldMU3*sqrt(1+((xold-xMin)/(zold-z))**2+((yold-y0)
     $        /(zold-z))**2)
         alpha=0.0136/pcut*sqrt(xshield/X0)*(1.+0.038*alog(xshield/X0))
         sigma1=alpha*(zold-z)/1.7
c
c MU1 - TC2 shielding part
c
         xShield=zShieldMU1*sqrt(1+((xold-xMin)/(zold-z))**2+((yold-y0)
     $        /(zold-z))**2)
         alpha=0.0136/pcut*sqrt(xshield/X0)*(1.+0.038*alog(xshield/X0))
         sigma2=alpha*zShieldMU1/1.7
c
c Calorimeter part
c
         alpha=0.0136/pcut*sqrt(xRadLenCalo)
     &          *(1.+0.038*alog(xRadLenCalo))
         sigma3=alpha*((xRadLenCalo*X0Calo)/1.7+XnoMultScat)
c
c
c
         sigmax=sqrt(sigma1**2+sigma2**2+sigma3**3)
         sigmay=sqrt(sigma2**2+sigma3**2)

      ELSE IF(iAlgo.eq.3) THEN ! prediction of sigma TC2 -> TC1
c
c The are 2 parts:sigma for iron(MU1-TC2) and calorimeter
c

c
c MU1 - TC2 shielding part
c
         xShield=zShieldMU1*sqrt(1+((xold-xMin)/(zold-z))**2+((yold-y0)
     $        /(zold-z))**2)
         alpha=0.0136/pcut*sqrt(xshield/X0)*(1.+0.038*alog(xshield/X0))
         sigma1=alpha*zShieldMU1/1.7
c
c Calorimeter part
c
         alpha=0.0136/pcut*sqrt(xRadLenCalo)
     &          *(1.+0.038*alog(xRadLenCalo))
         sigma2=alpha*((xRadLenCalo*X0Calo)/1.7+XnoMultScat)
c
c
c
         sigmax=sqrt(sigma1**2+sigma2**2)
         sigmay=0
         sigmax=sigmax*(zold-z)/(z1old-zold)

      ENDIF
c++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

      XDeltaMU = T1MUXMSCpar(isl) * sigmax
      YDeltaMU = T1MUYMSCpar(isl) * sigmay

      xMin = xMin - XDeltaMU
      xMax = xMax + XDeltaMU
      y0   = y0   - YDeltaMU
      sy   = sy   + YDeltaMU * 2
c
c***  printing out  ***
c
c             WRITE(L1LUNprn,888) iAlgo
c 888         FORMAT(3x,'L1MUMSC:   Alg.=',i2,
c     +		 '    multiple scattering window ')
c
c             WRITE(L1LUNprn,889) xMin,xMax,y0,y0+sy
c 889         FORMAT(3x,'  x from ',f8.2,
c     +  ' to',f8.2,' ;   y from ',f8.2,' to',f8.2)
c

      RETURN
c
      END



