//---------------------------------------------------------------------//
//         matching of RTRA to rich pseudo-segments                    //
// created 10.04.2001  by O.Igonkina
// see http://www-hera-b.desy.de/subgroup/software/arte/MARPLE/    OI
//---------------------------------------------------------------------//

    
#include "marple/marple.hh"
#include "marple/RcMatchBuilder.hh"
#include "arte/RSEG_cmp.hh"
#include "arte/arte.hh"

//extern  int rc99idRISE(float ip, float rad, float *dd);

#ifdef MARPLEPERFORMANCES
#include "arte/Chrono.hh"
#include "arte/ChronoManager.hh"
Chrono * chMatchRtraRich = 0;
#endif

// flag, z_plane, z_min, z_max, use_pair_cut, n_pair_left, n_pair_right, 
//use_delta_cut, delta_x, delta_y, delta_tx, delta_ty, delta_pe, chi2

RcMatchConst rcRtraRichConst = 
{
  110,     1000.,   -30.,  1500.,      1,          1,            2,
    1,       1.e+10,      1.e+10,   0.003,  0.004,   1.e+10 , 1000000.
};

RcMatchConst rcRtraRichSoftConst = 
{
  110,     1000.,   -30.,  1500.,      1,          2,            2,
    1,       1.e+10,      1.e+10,   0.01,  0.01,   1.e+10 , 1000000.
};

extern RcMatchConst rcSVDRichConst;
extern RcMatchConst rcSoftSRConst;
extern RcMatchConst rcOTRRichConst; 
extern RcMatchConst rcSoftORConst;

extern "C"
void FORTRAN_ROUTINE(matchrtrarich) ()
{
# ifdef MARPLEPERFORMANCES
  if(chMatchRtraRich == 0){
    ChronoManager CM;
    chMatchRtraRich = CM.createChrono("match Rtra Rich");
  }
  chMatchRtraRich->start();
# endif

  RcMatchBuilder<RcTraPoint, RcSegPoint>  RtraRichMatch;
  maPresentMatch = MARPLENOMATCH;
  //maPresentMatch =MARPLEIDRTRARICH;

  //------------------------------------------------------------------
  // put the defined constants in this builder:
  //------------------------------------------------------------------
  
  if( MARPLE_SOFT )
    RtraRichMatch.setConst(rcRtraRichSoftConst);
  else
    RtraRichMatch.setConst(rcRtraRichConst);

  int iErr; 

  //------------------------------------------------------------------
  //
  //   first part    : match VDS-OTR  or VDS-ECAL rtras to RICH
  //
  //------------------------------------------------------------------
  
  for( ArteTable<RSEG>::iterator richsegment = ArteTable<RSEG>::begin();
       richsegment != ArteTable<RSEG>::end(); ++richsegment )
    {
      if( ! maRichSegment(*richsegment) ) continue;
      if( (*richsegment)->rtra.begin() != (*richsegment)->rtra.end() )
	continue;

      RcSegPoint *MyptToPoint2 = new RcSegPoint (*richsegment ,0 );
      MyptToPoint2->setZ( rcRtraRichConst.z_plane);
      RtraRichMatch.addPointRight (*MyptToPoint2);
      MyptToPoint2 = NULL;
    }

  // no rich at all in the event - return
  if( RtraRichMatch.vectorRight().size() == 0 )
    {
      RtraRichMatch.deleteAll();  // cleaning
# ifdef MARPLEPERFORMANCES
      chMatchRtraRich->stop();
# endif
      maPresentMatch = MARPLENOMATCH;
      return;
    }
  //------------------------------------------------------------------
  
  for(ArteTable<RTRA>::iterator irtra = ArteTable<RTRA>::begin();
      irtra != ArteTable<RTRA>::end(); ++irtra)
    {
      if( (*irtra)->fit != 0 ) continue;
      if( ((*irtra)->cmp & Rsegc::bitsign) != 0 ) continue;
      // OTR and VDS only rtras are considered later
      if( (*irtra)->cmain <= 0 && (*irtra)->cecal <= 0 ) continue;
      if( (*irtra)->csvd <= 0 ) continue;

      RcTraPoint * MyptToPoint = new RcTraPoint( *irtra );
      MyptToPoint->prolongate( rcRtraRichConst.z_plane);
      RtraRichMatch.addPointLeft (*MyptToPoint);
      MyptToPoint = NULL;
	
    }


  //------------------------------------------------------------------
  RtraRichMatch.doMatch();
  RtraRichMatch.stat(); // print result of matching
  ///cout << " Step 1 : Nrtra " << RtraRichMatch.vectorLeft().size() << " Nrich"
  //  <<  RtraRichMatch.vectorRight().size() << " Nmatch " 
  //   << RtraRichMatch.matchList().size()<< endl;

  if( MARPLE_MAKERSEG ) RtraRichMatch.saveRSEG(iErr);

  RtraRichMatch.saveRelation(iErr);
  if(iErr)
    std::cout << "RTRA-RICH match :not all relations were saved. " << endl;

  RtraRichMatch.deleteAll();  // cleaning

  //------------------------------------------------------------------
  //
  //   second part    : match OTR only
  //
  //------------------------------------------------------------------
  if( MARPLE_SOFT )
    RtraRichMatch.setConst(rcSoftORConst);
  else
    RtraRichMatch.setConst(rcOTRRichConst);

  

  //------------------------------------------------------------------
  
  for(ArteTable<RTRA>::iterator irtra = ArteTable<RTRA>::begin();
      irtra != ArteTable<RTRA>::end(); ++irtra)
    {
      if( (*irtra)->fit != 0 ) continue;
      if( (*irtra)->csvd > 0  || (*irtra)->cmain <= 0 ) continue;
      if( ((*irtra)->cmp & Rsegc::bitsign) != 0 ) continue;

      RcTraPoint * MyptToPoint = new RcTraPoint( *irtra );
      MyptToPoint->prolongate( rcRtraRichConst.z_plane);
      RtraRichMatch.addPointLeft (*MyptToPoint);
      MyptToPoint = NULL;	
    }

  //------------------------------------------------------------------
  
  for( ArteTable<RSEG>::iterator richsegment = ArteTable<RSEG>::begin();
       richsegment != ArteTable<RSEG>::end(); ++richsegment )
    {
      if( ! maRichSegment(*richsegment) ) continue;
      if( (*richsegment)->rtra.begin() != (*richsegment)->rtra.end() )
	continue;
      RcSegPoint *MyptToPoint2 = new RcSegPoint (*richsegment ,0 );
      MyptToPoint2->setZ( rcRtraRichConst.z_plane);
      RtraRichMatch.addPointRight (*MyptToPoint2);
      MyptToPoint2 = NULL;
    }
  //------------------------------------------------------------------
  RtraRichMatch.doMatch();
  //cout << " Step 2 : Nrtra " << RtraRichMatch.vectorLeft().size() << " Nrich"
  //   <<  RtraRichMatch.vectorRight().size() << " Nmatch " 
  //   << RtraRichMatch.matchList().size()<< endl;

  RtraRichMatch.stat(); // print result of matching
  if( MARPLE_MAKERSEG ) RtraRichMatch.saveRSEG(iErr);

  RtraRichMatch.saveRelation(iErr);
  if(iErr)
    std::cout << "RTRA-RICH match :not all relations were saved. " << endl;

  RtraRichMatch.deleteAll();  // cleaning

  //------------------------------------------------------------------
  //
  //   third part    : match VDS only
  //
  //   no action at the moment
  //------------------------------------------------------------------
  if ( 0 ) 
    {
  if( MARPLE_SOFT )
    RtraRichMatch.setConst(rcSoftSRConst);
  else
    RtraRichMatch.setConst(rcSVDRichConst);

  
  for(ArteTable<RTRA>::iterator irtra = ArteTable<RTRA>::begin();
      irtra != ArteTable<RTRA>::end(); ++irtra)
    {
      if( (*irtra)->fit != 0 ) continue;
      if( (*irtra)->csvd <= 0  
	  || (*irtra)->cmain > 0 || (*irtra)->cecal > 0) continue;
      if( ((*irtra)->cmp & Rsegc::bitsign) != 0 ) continue;

      RcTraPoint * MyptToPoint = new RcTraPoint( *irtra );
      MyptToPoint->prolongate( rcRtraRichConst.z_plane);
      RtraRichMatch.addPointLeft (*MyptToPoint);
      MyptToPoint = NULL;	
    }
  //------------------------------------------------------------------
  
  for( ArteTable<RSEG>::iterator richsegment = ArteTable<RSEG>::begin();
       richsegment != ArteTable<RSEG>::end(); ++richsegment )
    {
      if( ! maRichSegment(*richsegment) ) continue;
      if( (*richsegment)->rtra.begin() != (*richsegment)->rtra.end() )
	continue;
      RcSegPoint *MyptToPoint2 = new RcSegPoint (*richsegment ,0 );
      MyptToPoint2->setZ( rcRtraRichConst.z_plane);
      RtraRichMatch.addPointRight (*MyptToPoint2);
      MyptToPoint2 = NULL;
    }

  //------------------------------------------------------------------
  RtraRichMatch.doMatch();
  //cout << " Step 3 : Nrtra " << RtraRichMatch.vectorLeft().size() << " Nrich"
  //   <<  RtraRichMatch.vectorRight().size() << " Nmatch " 
  //   << RtraRichMatch.matchList().size()<< endl;

  RtraRichMatch.stat(); // print result of matching
  if( MARPLE_MAKERSEG ) RtraRichMatch.saveRSEG(iErr);

  RtraRichMatch.saveRelation(iErr);
  if(iErr)
    std::cout << "RTRA-RICH match :not all relations were saved. " << endl;

  RtraRichMatch.deleteAll();  // cleaning
    }// switch for SVDonly-RICH match
  //------------------------------------------------------------------
  // correct RTRA according to RICH segment assigned
  //------------------------------------------------------------------
  for(ArteTable<RTRA>::iterator irtra = ArteTable<RTRA>::begin();
      irtra != ArteTable<RTRA>::end(); ++irtra )
    {
      // search if it is connected to marple RSEG with RICH cmp
      for( ArteRelationIterator<RSEG> irs = (*irtra)->rseg.begin();
	   irs != (*irtra)->rseg.end(); ++irs )
	{
	  if( (*irs)->fit != 7 ) continue;
	  if( ((*irs)->cmp & Rsegc::rich ) == 0 ) continue;
	  (*irs)->time = 888887;  // to distinquish from rseg-rich match
	  // serach original RICH segment
	  ArtePointer<RSEG> rich = 0;
	  for( ArteRelationIterator<RSEG> irich = (*irs)->rend.begin();
	       rich == 0 && irich != (*irs)->rend.end();  ++irich)
	    {
	      if(  ! maRichSegment(*irich) ) continue;
	      rich = *irich;
	    }
	  if( rich == 0 ) 
	    {
	      cout << " Something wrong with RTRA= " << *irtra
		   << " and marple RSEG " << *irs << endl;
	      continue;
	    }

	  (*irtra)->cer=rich->time;
	  (*irtra)->txr=rich->txf;
	  (*irtra)->tyr=rich->tyf;
          (*irtra)->hri=rich->nhit; 
	  (*irtra)->cmp |= rich->cmp;
	  (*irtra)->crich = rich.index();

	} // loop on RSEG
    }// loop on RTRA

 
  maPresentMatch = MARPLENOMATCH;

# ifdef MARPLEPERFORMANCES
    chMatchRtraRich->stop();
# endif
}

