
//## Boris Fominykh  30.12.1999
// member functions of class MuDbPixel

#include <stdio.h>
#include <string.h>
#include <ctype.h>
#include <stdlib.h>
#include <fstream.h>
#include <iostream.h>
#include <iomanip.h>

#include "arte/arte.hh"
#include "arte/ArteMessage.hh"

#include "murec/MuDbPixel.h"

extern "C" {
int MuNameDecode(char*,int*,int*,int*,int*,int*,int*);
}

//** Constructor */
MuDbPixel::MuDbPixel()
  :  MuDbChamber::MuDbChamber()
{
  Ivalid=-1;
  Clear();
}

//** Destructor */
MuDbPixel::~MuDbPixel()
{
}

//## Methods

void MuDbPixel::Clear()
{
  for(Word sl=0; sl<PixGeoSl; sl++) {
    for(Word qu=0; qu<PixGeoQu; qu++) {
      PixGede[sl][qu] = 0;
      char* name = "     ";
      strncpy(&ChamListX[sl][qu][0],name,6);
      for(Word nc=0; nc<PixNcon; nc++) {
        MaskX[sl][qu][nc]  = 0;
        Nbit1X[sl][qu][nc] = 0;
        CrateX[sl][qu][nc] = 0;
      }
    }
  }
  return;
}

int MuDbPixel::SetStruct()
{
  int ret;
// NESL objects
  MuNesl   lnesl;  
  MuDbNesl Nesl;  
  Nesl.setDBname(DBname);
  Nesl.setKTstatus(-2);
  Nesl.setRun(Run);
  Nesl.setMinor(Minor);
  ret=Nesl.Read();
  if(ret != 0) {
    arteMsg << "MuDbPixel: NESL table is NOT read ret= " << ret;;
    arteMsg << " DBname= " << DBname << " Run= " << Run;
    arteMsg << " Minor " << Minor << endl;
    arteMsg.warn();
    return 10+ret;
  }

// NEDE objects
  MuNede   lnede;  
  MuDbNede Nede;  
  Nede.setDBname(DBname);
  Nede.setKTstatus(-2);
  Nede.setRun(Run);
  Nede.setMinor(Minor);
  ret=Nede.Read();
  if(ret != 0) {
    arteMsg << "MuDbPixel: NEDE table is NOT read ret= " << ret;;
    arteMsg << " DBname= " << DBname << " Run= " << Run;
    arteMsg << " Minor " << Minor << endl;
    arteMsg.warn();
    return 20+ret;
  }

// loops over superlayers MU1-MU4
  for(int sl=0; sl<PixGeoSl; sl++) {
// superlayer key
    int neslkey = 10*sl+19;
// read the superlayer information from DB
    ret=Nesl.Get(&neslkey,&lnesl);
    if(ret != 0) {
      return 30+ret;
    }
    else {
      int cmp = lnesl.cmp;
      int qum = lnesl.lay;
      if(cmp == 9 && qum > 0) {
        int msl=sl+1;
        for(int qu=0; qu<PixGeoQu; qu++) {
          int nedekey = 1000*msl+100*(qu+1)+cmp;
          ret=Nede.Get(&nedekey,&lnede);
          if(ret != 0) {
            arteMsg << "MuDbPixel: no NEDE for key " << nedekey << endl;
            arteMsg.warn();
          }
          else {
            strncpy(&ChamListX[sl][qu][0],&(lnede.hwc[0]),9);
            ChamListX[sl][qu][5]='\0';
            PixGede[sl][qu] = lnede.inede;
          }
        }
      }
    }
  }
  Nesl.Release();
  Nede.Release();
  return 0;
}

int MuDbPixel::SetConnect()
{
// PixChamber object
  MuPixChamber  PixCh;  

  int ret=0;
  int NcabR=0;
// Config object
  MuDbConfig Conf;
  Conf.setDBname(DBname);
  Conf.setKTstatus(KTstatus);
  Conf.setRun(Run);
  Conf.setMinor(Minor);
  ret=Conf.Read(&NcabR);
  if(ret != 0 || NcabR <= 0) {
    arteMsg << "MuDbPixel: cable configuration table is NOT read, ret= " << ret;
    arteMsg << " NcabR= " << NcabR << " DBname= " << DBname << " Run= " << Run;
    arteMsg << " Minor " << Minor << endl;
    arteMsg.warn();
    return 10+ret;
  }

// hot channel object
  MuDbHots Hots;
  Hots.setDBname(DBname);
  Hots.setKTstatus(KTstatus);
  Hots.setRun(Run);
  Hots.setMinor(Minor);
  ret=Hots.Read();
  if(ret != 0) {
    arteMsg << "MuDbPixel: hot channel table is NOT read, ret= " << ret;
    arteMsg << " DBname= " << DBname << " Run= " << Run;
    arteMsg << " Minor " << Minor << endl;
    arteMsg.warn();
    return 20+ret;
  }

// loops over cables
  for(int ncb=0; ncb<NcabR; ncb++) {
    char name[10];
    Word crate=0;
    Word ncabl=0;
// get the next cable connection
    ret=Conf.GetCabl(&ncb,name,&crate,&ncabl);
    if(ret != 0) return 30+ret;
// cable bit position inside crate
    Word nbit = PixCh.Nbit*ncabl; 
// decode the cable name
    int type = 0;
    int msl  = 0;
    int lay  = 0;
    int ud   = 0;
    int qu   = 0;
    int nc   = 0;
    ret=MuNameDecode(&name[0],&type,&msl,&lay,&ud,&qu,&nc);
    if(type == 4) {
      int sl=msl-1;
      if(qu < 1 || qu > PixGeoQu) {
	arteMsg << "MuDbPixel: wrong quadrant number = " << qu << endl;
        arteMsg.warn();
        return 31;
      }
      qu-=1;
      if(nc < 1 || nc > PixNcon) {
	arteMsg << "MuDbPixel: wrong connector number = " << dec << nc << endl;
        arteMsg.warn();
        return 32;
      }
      nc-=1;
// get the cable hot channel mask
      Word status=Hots.Status(name);
// code the connection status into 16-bit word
      Word stcon = 0;
      Word one    = 0x8000;
      for(int i=0; i<PixCh.Nbit; i++) {
        if(PixCh.Pnx1[sl][nc][i] > 0 &&
           PixCh.Pny1[sl][nc][i] > 0) stcon = stcon | (one >> i);
      }
// swapping
      Word nbb=nbit/PixCh.Nbit;
      Word nba=nbit-PixCh.Nbit*nbb;
      if(nbb%2 == 0) nbb += 1;
      else nbb -= 1;
      nbit = PixCh.Nbit*nbb+nba;

      CrateX[sl][qu][nc]= crate;
      Nbit1X[sl][qu][nc]= nbit;
      MaskX[sl][qu][nc] = status & stcon;
    }
  }
  Hots.Release();
  Conf.Release();
  return 0;
}

int MuDbPixel::Ini()
{
  Ivalid=-1;
  Clear();
  int ret = 0;
  ret=MuDbPixel::SetStruct();
  if(ret != 0) return ret+10;
  ret=MuDbPixel::SetConnect();
  if(ret != 0) return ret+20;
  Ivalid=1;
  return 0;
}

void MuDbPixel::Print()
{
// create the log-file and pipe the programm output
  ofstream fout("MuDbPixel.out");
  if (!fout) {
    arteMsg << "MuDbPixel - can NOT open file for print" << endl;
    arteMsg.warn();
    return;
  }

// PixChamber object
  MuPixChamber  PixCh;  
  PixCh.Print();

  fout << "MuDbPixel: Ivalid= " << Ivalid << endl;
  for(Word sl=0; sl<PixGeoSl; sl++) {
    for(Word qu=0; qu<PixGeoQu; qu++) {
      fout << "pixel sl " << dec << sl << " quadrant " << qu;
      fout << " gede " << PixGede[sl][qu] << "  ";
      char name[10];
      strncpy(name,&ChamListX[sl][qu][0],9);
      fout << name << endl;
      for(Word nc=0; nc<PixNcon; nc++) {
        fout << "  conn " << dec << nc;
        fout << " mask " << hex << MaskX[sl][qu][nc];
        fout << " crate " << dec << CrateX[sl][qu][nc];
        fout << " bit1 " << dec << Nbit1X[sl][qu][nc];
        if((nc+1)%2 == 0) fout << endl;
      }
      fout << endl;
    }
  }
}
