/************************************************************************
*
* MuHitInfo.C
* 
* first author : Jing-Ge Shiu 
* purpose      : C++/SLT code for extract muon hit information
* called from  : 
* includ file  : MuHitInfo.h, jsMuPixTool.h
* start date   : May. 09, 2000
*
*
*-----------------------------------------------------------------------
*
* ver 1.00     : 0509, 2000 physjg
*                It works for tube/pad/pixel, but not well tested yet.
*                There are four input methods : putmuhit(...)
*                    1)input hitb pointer
*                    2)input x,y,z 
*                    3)input gede no, and two integers
*                    4)input gede pointer, and two integers
* ver 1.01     : 0510, 2000 physjg                             
*                change the caculation for pad rno to use the informaiton from GEDE
*                MuBasePara.h is not needed anymore              
*                modified for the middle pad chambers, there are 27 pad in column. 
*                add getmuzpos(int) to get the z position of reconstruction layer
* ver 1.02     : 0515, 2000 physjg
*                add xside (+/-1) for +/- x side information. 
* ver 1.03     : 0525, 2000 physjg
*                change getzlayer(float) to public member function
* ver 1.04     : 0617, 2000 physjg
*                pixx, pixy also for pad column and row no.
* ver 1.05     : 0624, 2000 physjg
*                include FEDid information?  ---> I think I better do it in another way 
*
*                2000, 1017  physjg  
*                first release 
*
**************************************************************************/
//include files for C/C++
#include <iostream.h>
#include <iomanip.h>
#include <math.h>

// include ARTE head files for new C++ interface    
#include "arte/arte.hh"        
#include "arte/RSEG_cmp.h"
#include "auto/auto.hh"
#include "murec/MuHitInfo.h"


MuHitInfo::MuHitInfo () {
 
  getzpos();
  set2ini();
  hitx = hity = hitz = 0;
  inputerr = NOINPUT;

  return;
}

void MuHitInfo::set2ini(void) {

  cmp = reccmp = sl = dsub = rsub = dno = rno 
      = rtype = sd = updown = secno = fedno = -1;
  gedeid = ArtePointer<GEDE>(-1);
  geslid = ArtePointer<GESL>(-1);
  hitbid = ArtePointer<HITB>(-1);
  inputerr = NOINPUT;
  
  return ;
}


void MuHitInfo::printzpos(void) {

  if (zno==0) getzpos();
   cout << "zno = " << zno << endl;
   for (int i=0;i<zno;i++) 
     cout << setw(3) << i << " at z = " << muzpos[i] << endl;

 return; 
}



void MuHitInfo::getzpos(void) {

  int cmpid, gotflag = 0;
  float zz;
 
  zno = 0;
  for (int i=0;i<MAXZPOS;i++) muzpos[i] = 0;

  for (ArteTable<GEDE>::iterator gedeiv = ArteTable<GEDE>::begin();
       gedeiv != ArteTable<GEDE>::end();
       gedeiv++ ) { 

    if ( (cmpid = (*gedeiv)->cmp) == 9  || cmpid == 8) 
       if ( (zz = (*gedeiv)->zfw[0]) > 1300.) {          // wire readout z position
          gotflag = 0;
          for (int i=0;i<zno&&gotflag<1;i++) 
             if (fabs(zz-muzpos[i])<.1) gotflag=1;
          if (gotflag==0) {
             if (zno >= MAXZPOS) { cout << "zpos overflow "<< endl; zno=0; return; } 
             muzpos[zno++] = zz;
             if ( ((*gedeiv)->gesl->nsl > 2)&&(cmpid==8) ) 
                if (zno >= MAXZPOS) { cout << "zpos overflow "<< endl; zno=0; return; } 
                else muzpos[zno++] = (*gedeiv)->zfw[1];  // pad readout z position
          }
       } else {
         cout << "Something is wrong for zpos(" << zz << ") of gede " << gedeiv << endl;
       }   

  }

  for (int i=0;i<zno-1;i++)
    for (int j=i+1;j<zno;j++)
      if (muzpos[i]>muzpos[j]) {
         zz = muzpos[i];muzpos[i]=muzpos[j];muzpos[j]=zz;
      } 
  
  return ;
}



int MuHitInfo::getdn() {

  // tube and pad
  if (rtype<2) {
     dno = 10*(chname[6]-'0') + (chname[7]-'0');
     if (sl>2) {   // Mu3 and Mu4
        if (dno>18) xs=+1; else xs=-1;
     } else {
       xs = -1;
       if (dsub==1) {
          if (dno>(14+sl)) xs=+1;
       } else 
          if (dsub==2) {             
             if ( ((rsub==3)&&(dno>17))||((rsub==4)&&(dno>15)) ) xs=+1;
          } else 
             if (dsub==3) {  // it should be 3 now
                if ( ((rsub==5)&&(dno>14))||((rsub==6)&&(dno>16)&&(dno<29)) ) xs=+1;
             } 
     }
  } else {
     // pixel chamber      
     dno = gedeid->sec;
     if ((dno==1)||(dno==4)) xs=+1; else xs=-1 ;
  }
   
  return 0;

}


int MuHitInfo::getzlayer(float z) {

  int zi=0;
 
  if (zno==0) getzpos();
  if (zno>0) { 
    for (zi=0;zi<zno;zi++) 
       if ( fabs(muzpos[zi]-z)<.01 ) break;
    if (zi==zno) zi=0; else zi++;
  }

  return zi;

}



int MuHitInfo::getrn() {

   float z0 = gedeid->zmin, z1 = ((ArteTable<GEDE>::iterator(gedeid))-1)->zmin, zdiff = (z0-z1);
   TWOINT pixhitxy,pixhitvp;

   zdiff *= (zdiff>0)?1.:-1;
   if (zdiff>2.5) sd=0;
   else sd = (z0>z1);

   switch (rtype) {
   float ymin,hityy,yl2, dypd1, dypd2;
   case 0 :  // for tube readout
      rno = 2*(int)( (gedeid->xmax - hitx)/gedeid->pit ) + sd + 1; 
      if ( updown>0 ) rno = 33 - rno;
      if ( (sl > 2)&&(updown==0) )       // Mt3, Mt4 top are different
         if (rno&0x01) rno++;
         else rno--; 
         secno = (rno>16)?1:0;
      break;

   case 1 :  // for pad readout
      yl2=0.01*float(gedeid->nwi[3]);    // y-dist between pads
      dypd1=gedeid->xfw[2];              // y-size of first pad
      dypd2=gedeid->xfw[3];              // y-size of standard pad

      hityy = hity*(updown==1?-1:1);
      ymin=(updown==1)?-1*gedeid->symx:gedeid->symn ;

      // for pad 1
      if ( hityy > ymin && hityy < (ymin + dypd1) ) rno = 2;
      else // regular cases for 3rd to end pads
         rno = (int)( (hityy - ymin - dypd1)/(dypd2+yl2) )+3;  
      // special cases
      if ( (dno>15)&&(dno<19) ) {
         rno += ( 3 - (sl==4));          
      }
      else if ( (updown==1)&&(dno==19) ) rno += ((sl==3)+10);
      if ( rno > 16 ) rno++;                               // channel 1 is not connected
      // set the sec number ABCD/0123
      // gedeid->xfw[1] is the x-size of pad
      secno = 2*( ( (hitx - (gedeid->xmin)) < gedeid->xfw[1])^(updown>0) ) + (rno > 16);
      pixx = 2*(dno==99?19:dno) - 1 + ( (updown==0)?(secno<2):(secno>1) );
      pixy = 31 + ((updown==0)?1:-1)*(rno-1-(rno>16)) - (updown==0) ;
      break;

   case 2 : // pixel readout
      pixhitxy = mupixinfo.getxy2pix(sl-1, rsub-1, hitx, hity);
      if ( (pixhitxy.i1>=0)&&(pixhitxy.i2>=0) ) {          // I might not need this checking
         pixhitvp = mupixinfo.pix_vp[sl-1][pixhitxy.i1][pixhitxy.i2];
         pixx = pixhitvp.i1+1;
         pixy = pixhitvp.i2+1;
         secno = mupixinfo.vp_readout[sl-1][pixhitvp.i1][pixhitvp.i2].i1;
         rno   = mupixinfo.vp_readout[sl-1][pixhitvp.i1][pixhitvp.i2].i2;
         chname[6] = '0' + secno/10;     
         chname[7] = '0' + secno%10;
      } 
      break;

   default :
      break;
   }
 
   return 0;

}



int MuHitInfo::getrn(Word chanidx, Word chanidy) {

   char thischname[12],lastchname[12];
   strcpy( thischname, gedeid->hwc);
   strcpy( lastchname, ((ArteTable<GEDE>::iterator(gedeid))-1)->hwc);
   sd = (strcmp(thischname, lastchname)==0);

   switch (rtype) {
   case 0 :  // for tube readout
      rno = 2*chanidy - sd;                               // count from xmin.
      if ( updown==0 ) rno = 33 - rno;
      if ( (sl > 2)&&(updown==0) )                        // Mt3.1 and Mt4.1 are different  
         if (rno&0x01) rno++;
         else rno--; 
         secno = (rno>16)?1:0;
      break;

   case 1 :  // for pad readout
      rno = (chanidy&0xFF)+1;                             // regular case for pad readout
      // special cases
      if ( (dno>15)&&(dno<19) ) rno += ( 3 - (sl==4)) ;
      else if ( (updown==1)&&(dno==19) ) rno += (sl==3?11:10);
      if ( rno > 16 ) rno++;                              // channle 0 is not connected
      // set the sec number ABCD/0123,  (chanidy&0X100) = 0 for pads with lower x
      secno = 2*( (updown==0)?(1-(chanidy>0xFF)):(chanidy>0xFF) ) + (rno > 16);
      pixx = 2*(dno==99?19:dno) - 1 + ( (updown==0)?(secno<2):(secno>1) );
      pixy = 31 + ((updown==0)?1:-1)*(rno-1-(rno>16)) - (updown==0) ;
      break;

   case 2 : // pixel readout
      pixx = chanidx;
      pixy = chanidy;
      if ( (--chanidx<mupixinfo.Nxrd[sl-1])&&(--chanidy<mupixinfo.Nyrd[sl-1]) ) {
         secno = mupixinfo.vp_readout[sl-1][chanidx][chanidy].i1;
         rno   = mupixinfo.vp_readout[sl-1][chanidx][chanidy].i2;
         // change the chamber name to include the asd id number
         chname[6] = '0' + secno/10;     
         chname[7] = '0' + secno%10;
      }
      break;

   default :
      break;
   }
 
   return 0;

}



Word MuHitInfo::putmuhit (ArtePointer<HITB> phitb) {

  ArtePointer<GEDE> pgede = phitb->gede; 
  ArtePointer<GESL> pgesl = pgede->gesl;

  if ( (pgede->cmp)==8||(pgede->cmp)==9 ) {
     cmp  = pgede->cmp;
     reccmp = 0;
     hitbid = phitb;
     gedeid = pgede;
     geslid = pgesl;
     sl   = geslid->nsl;
     dsub = gedeid->lay;
     strcpy( chname, gedeid->hwc);  
     if ( (rtype = (cmp==9?2:fabs(hitbid->y)<.001?0:1) )==0 ) 
        chname[1] = 't'; 
     rsub = (cmp==8?2*dsub - (gedeid->ymax>130):gedeid->sec) ;
     updown = 1 - (rsub&0x01);
     hitx = hitbid->x;
     hity = hitbid->y;
     hitz = hitbid->z;
     rot = gedeid->rot;
     rotangle = acos(double(rot[0]));
     getdn();
     getrn();                      
     zlayer = getzlayer(hitz);
     inputerr = OKHITBINPUT;

  } else { 
    set2ini();
    inputerr = NOTOKHITBINPUT;
  }  

  return inputerr;
}


Word MuHitInfo::putmuhit (ArtePointer<GEDE> pgede, int  &readtype, int &channelid) {

  if ( (pgede->cmp)==8||(pgede->cmp)==9 ) {
     rtype = ((cmp = pgede->cmp)==8)?readtype:2;
     reccmp = 0;
     hitbid = ArtePointer<HITB>(-1);
     gedeid = pgede;
     geslid = gedeid->gesl;
     sl   = geslid->nsl;
     dsub = gedeid->lay;
     strcpy( chname, gedeid->hwc);  
     if (rtype ==0 ) chname[1] = 't'; 
     rsub = (cmp==8?2*dsub - (gedeid->ymax>130):gedeid->sec) ;
     updown = 1 - (rsub&0x01);
     getdn();
     getrn(readtype,channelid);
     hitz = (rtype==0)?gedeid->zfw[0]:
                      ((rtype==1)?gedeid->zfw[1]:(gedeid->zmin+gedeid->zmax)/2);
     rot = gedeid->rot;
     rotangle = acos(double(rot[0]));
     zlayer = getzlayer(hitz);
     inputerr = OKGEDEINPUT;
  } else { 
    set2ini();
    inputerr = NOTOKGEDEINPUT;
  }  

  return inputerr;
}



Word MuHitInfo::putmuhit (int ipl, int  &readtype, int &channelid) {

  ArtePointer<GEDE> pgede = ArtePointer<GEDE>(ipl);
  return putmuhit(pgede,readtype,channelid);

}



Word MuHitInfo::putmuhit (float x0, float y0, float z) {

  TWOINT pixhitxy;
  int ipl=0, rtype=-1, channelid=-1;
  float x = 0.;
  float y = 0.;
  Word localinputerr = NOTOKXYZINPUT;

  if (zno==0) getzpos();

  if ( ( z > (muzpos[0] - 2.))&&( z < (muzpos[MAXZPOS-1] + 2.)) ) 
     for (ArteTable<GESL>::iterator gesliv = ArteTable<GESL>::begin();
         (gesliv != ArteTable<GESL>::end())&&localinputerr>MAXOKFLAG;
         gesliv++ ) { 
   
       if ( ( (gesliv->cmp)==8||(gesliv->cmp)==9 )  
	  &&( gesliv->z1 <= z)
	  &&( gesliv->z2 >= z) ) {
          float cosine, sine;
          for (ipl=gesliv->fpl;
               ipl < gesliv->fpl + gesliv->npl;
               ipl++ ) {
               ArtePointer<GEDE>gedeiv = ArtePointer<GEDE>(ipl);  
               cosine = gedeiv->rot[0];
               sine   = gedeiv->rot[3];
               x = x0 * cosine + y0 * sine;   
               y = y0 * cosine - x0 * sine;
              if ( (z >= (gedeiv)->zmin)&&(z <= (gedeiv)->zmax)
                 &&(x >= (gedeiv)->xmin)&&(x <= (gedeiv)->xmax)
		 &&(y >= (gedeiv)->ymin)&&(y <= (gedeiv)->ymax)
                 ) {
                localinputerr = OKXYZINPUT;  
                break;  
              }
          }
       }  
     }
  if ( localinputerr==OKXYZINPUT ) {
     if ( ArtePointer<GEDE>(ipl)->cmp==8 ) {
       if ( (ArtePointer<GEDE>(ipl)->gesl->nsl > 2)
          &&(fabs(z - ArtePointer<GEDE>(ipl)->zfw[0])>fabs(z - ArtePointer<GEDE>(ipl)->zfw[1])) 
	  ) {    // pad readout
          float alsy = ( ArtePointer<GEDE>(ipl)->symx - ArtePointer<GEDE>(ipl)->symn );
          float yl2 = 0.01*(ArtePointer<GEDE>(ipl)->nwi[3]);           
          float yl1 = ( ArtePointer<GEDE>(ipl)->ymax - ArtePointer<GEDE>(ipl)->ymin - alsy )/2; 
// updated 18.10.2000 by B.Fominykh
//          float tmpy, ypos, ydis, ylow, yhigh;      
          float tmpy, ylow, yhigh;      
                
	  (ArtePointer<GEDE>(ipl)->ymax > 130 )?tmpy=-y:tmpy=y;    // convert to top chamber

          // caculate the y range for the first pad
          ylow  = yl1 - yl2/2;
          yhigh = yl1 + ArtePointer<GEDE>(ipl)->xfw[2] + yl2/2;     
          if ( (tmpy<yhigh)&&(tmpy>ylow) ) channelid = 1;
          else  // then the 2nd to n-1 pad
          for (int yid=2;yid<ArtePointer<GEDE>(ipl)->nwi[2];yid++) {
              ylow  = yhigh;
              yhigh += ArtePointer<GEDE>(ipl)->xfw[3] + yl2;
              if ( (tmpy<yhigh)&&(tmpy>ylow) ) { 
                 channelid = yid; break;
              } 
	  } // if it is not in there, then check the last pad.
          if (channelid <= 0)
             if ( (tmpy>yhigh)&&(tmpy < (yl1+alsy+yl2/2)) ) channelid = ArtePointer<GEDE>(ipl)->nwi[2];

	  if (channelid<=0) {                
	     inputerr = NOTOKXYZINPUT;       
          } else {                           
             if ( (ArtePointer<GEDE>(ipl)->xmax - x) < (x - ArtePointer<GEDE>(ipl)->xmin) )             
                channelid |= 0x0100;
             rtype = 1;
             putmuhit(ipl, rtype, channelid);  
          } 
       } else {  // tube wire readout
          rtype = 0;
          channelid = int( (x- ArtePointer<GEDE>(ipl)->xmin)/ArtePointer<GEDE>(ipl)->pit ) + 1;
          putmuhit(ipl, rtype, channelid);
       }  
     } else {    // pixel readout 
       int slid = ArtePointer<GEDE>(ipl)->gesl->nsl;
       pixhitxy  = mupixinfo.getxy2pix(slid-1,ArtePointer<GEDE>(ipl)->sec-1, x, y);
       rtype     = mupixinfo.pix_vp[slid-1][pixhitxy.i1][pixhitxy.i2].i1+1;
       channelid = mupixinfo.pix_vp[slid-1][pixhitxy.i1][pixhitxy.i2].i2+1;
       putmuhit(ipl, rtype, channelid);
     }  
  }

  inputerr |= localinputerr;

  return inputerr;
}






