// 
//  Author:  Rainer Mankel                     Created:  29-Sep-98
//           Humboldt University Berlin        Modified:  
//  Email:   mankel@ifh.de                     HERA-B Collaboration
// 
// Modified:
// * Removed energy from model (steered by flag)   08-Nov-00 W.Hulsbergen
// #undef USEENERGY                                28/05/01 W. Hulsbergen 
// int rangerECClus::plane()
//

#include <iostream.h>
#include <math.h>
#include "arte/arte.hh"
#include "arte/ArteMessage.hh"
#include "rang/State.hh"
#include "rang/Node.hh"
#include "rang/ECClus.hh"
#include "rang/ScatThick.hh"
#include "rang/Geom.hh"
#include "rang/zfpar.hh"
#include "rang/utfeld.hh"
#include "rang/RadEl.hh"
#include "gean/ZGESL.h"

//-- Constructor
rangerECClus::rangerECClus() {
  cout << "This is the rangerECClus default constructor" << endl;
  rcl = 0;
  radThick = 0;
};

// this flag specifies whether energy of cluster is used in fit
#undef USEENERGY

rangerECClus::rangerECClus(ArtePointer<RCCL> rccl,int charge,rangerFitMode fm,
			   const float radTh) 
: rangerHitBase(rccl,charge,fm) {
#ifdef FULL
  cout << "This is the rangerECClus(rhit) constructor, hitb=" 
       << rhit->hitb << endl;
#endif
  rcl = rccl;
  radThick = radTh;
#ifdef USEENERGY
  int dim=3 ;
#else
  int dim=2 ;
#endif
  m = new HepVector(dim);
  (*m)(1) = rccl->x;
  (*m)(2) = rccl->y;
  V = new HepSymMatrix(dim);
  (*V)(1,1) = rccl->cve[0];
  (*V)(1,2) = rccl->cve[1];
  (*V)(2,2) = rccl->cve[2];
#ifdef USEENERGY
  (*m)(3) = 1/rccl->esum;
  (*V)(1,3) = 0;
  (*V)(2,3) = 0;
  (*V)(3,3) = (*m)(3) * (*m)(3) * 0.0004;     // assume flat error of 2 pc at the moment
#endif
}

rangerECClus::~rangerECClus(){
  delete m;
  m = 0;
  delete V;
  V = 0;
}

void rangerECClus::projectP(HepVector& resid,
			     HepMatrix& HH,rangerStateP* s) {

#ifdef FULL
  cout << "This is rangerECClus::projectP" << endl;
#endif
  resid(1) = (*m)(1) - s->x();
  resid(2) = (*m)(2) - s->y();

  HH(1,1) = 1;
  HH(1,2) = 0;
  HH(1,3) = 0;
  HH(1,4) = 0;
  HH(1,5) = 0;

  HH(2,1) = 0;
  HH(2,2) = 1;
  HH(2,3) = 0;
  HH(2,4) = 0;
  HH(2,5) = 0;

#ifdef USEENERGY
  resid(3) = (*m)(3) - fabs( s->qP() );
  HH(3,1) = 0;
  HH(3,2) = 0;
  HH(3,3) = 0;
  HH(3,4) = 0;
  HH(3,5) = (s->qP()>=0)? 1 : -1;
#endif
}

void rangerECClus::projectPI(HepVector& resid,
			     HepMatrix& HH,rangerStatePI* s) {

#ifdef FULL
  cout << "This is rangerECClus::projectP" << endl;
#endif
  resid(1) = (*m)(1) - s->x();
  resid(2) = (*m)(2) - s->y();

  HH(1,1) = 1;
  HH(1,2) = 0;
  HH(1,3) = 0;
  HH(1,4) = 0;
  HH(1,5) = 0;
  HH(1,6) = 0;

  HH(2,1) = 0;
  HH(2,2) = 1;
  HH(2,3) = 0;
  HH(2,4) = 0;
  HH(2,5) = 0;
  HH(2,6) = 0;

#ifdef USEENERGY
  resid(3) = (*m)(3) - fabs( s->qP() );
  HH(3,1) = 0;
  HH(3,2) = 0;
  HH(3,3) = 0;
  HH(3,4) = 0;
  HH(3,5) = (s->qP()>=0)? 1 : -1;
  HH(3,6) = 0;
#endif
}


void rangerECClus::projectL(HepVector& resid,
			     HepMatrix& HH,rangerStateL* s) {

#ifdef FULL
  cout << "This is rangerECClus::projectL" << endl;
#endif
  resid(1) = (*m)(1) - s->x();
  resid(2) = (*m)(2) - s->y();

  HH(1,1) = 1;
  HH(1,2) = 0;
  HH(1,3) = 0;
  HH(1,4) = 0;

  HH(2,1) = 0;
  HH(2,2) = 1;
  HH(2,3) = 0;
  HH(2,4) = 0;

#ifdef USEENERGY 
  resid(3) = 0;
  HH(3,1) = 0;
  HH(3,2) = 0;
  HH(3,3) = 0;
  HH(3,4) = 0;
#endif
}

void rangerECClus::projectLI(HepVector& resid,
			     HepMatrix& HH,rangerStateLI* s) {

#ifdef FULL
  cout << "This is rangerECClus::projectLI" << endl;
#endif
  resid(1) = (*m)(1) - s->x();
  resid(2) = (*m)(2) - s->y();

  HH(1,1) = 1;
  HH(1,2) = 0;
  HH(1,3) = 0;
  HH(1,4) = 0;
  HH(1,5) = 0;

  HH(2,1) = 0;
  HH(2,2) = 1;
  HH(2,3) = 0;
  HH(2,4) = 0;
  HH(2,5) = 0;

#ifdef USEENERGY
  resid(3) = 0;
  HH(3,1) = 0;
  HH(3,2) = 0;
  HH(3,3) = 0;
  HH(3,4) = 0;
  HH(3,5) = 0;
#endif
}

int rangerECClus::plane()
{
  // There is no gede/gwal related to an ecal cluster. Still we would
  // like to have a plane entry, since rangerTrack::insertWalls needs
  // this to find in-between-planes. Therefore, I invented yet another
  // `hack': Just return the first gwal related to the ECAL:
  static int thisplane=0 ;
  if( thisplane==0 ) {
    for(ArteTable<GESL>::iterator igesl = ArteTable<GESL>::begin();
	igesl != ArteTable<GESL>::end() && thisplane==0; ++igesl ) 
      if( igesl->cmp == LGESL_ECA ) {
	ArtePointer<GWAL> firstwall = ArtePointer<GWAL>(igesl->fwl) ;
	thisplane = -firstwall.index() ;
      }
  }
  return thisplane ;
}
