// 
//  Author:  Rainer Mankel                     Created:  01-Apr-99
//           Humboldt University Berlin        Modified:  
//  Email:   mankel@ifh.de                     HERA-B Collaboration
// 

// class rangerDomLay

#ifndef RANGER_DOMLAY_HH
#define RANGER_DOMLAY_HH


#include <vector.h>

class rangerSNM;
class rangerDomain;
class rangerDomWin;
class rangerLayPl;
class rangerHL;
class rangerDLTSet;
class rangerCompLaySet;
class rangerProxyLay;

class rangerDomLay {

public:
  rangerDomLay(){};
  rangerDomLay(rangerDomWin* wi);
  ~rangerDomLay(){};

  int accum(rangerDomWin* wi,int gLayer,rangerSNM* s,int method=-1,
	    const int subGLay1=1,const int subGLay2=999);
  int accumDLT(rangerDomWin* wi,int gLayer,rangerSNM* s,int method=-1,
	       rangerDLTSet* dlts=NULL) { return 0;};
  int accumComp(rangerDomWin* wi,int gLayer,rangerSNM* s,
		rangerCompLaySet& clset,int method=-1);
  int accumCmpSec(rangerDomWin* wi,int cmp,int sec,int gLayer,const int method,
		  const int subGLay1,const int subGLay2,double& zAcc);  
  void addLayPl(rangerLayPl* lp){ lpls.push_back(lp);};
  rangerDomain* dom() const;
  rangerDomWin* win() { return window; };
  const rangerDomWin* win() const { return window; };
  int meth() { return Method; };
  rangerDomLay* next(const int method,const int direction) {
    return ((direction > 0)? nxt[method] : prv[method]); };
  float z() { return zRef;};
  rangerHL* hl() { return hL; };
  const vector<rangerLayPl*>& LPs() { return lpls; };
  vector<rangerLayPl*>& setLPs() { return lpls; };
  int cntLP(int cm,int sc);
  void setHL(rangerHL* hl) { hL = hl; }; 
  void updLsn(int* inLsn);
  void updPrv(rangerDomLay* prv[]);
  void updNxt(rangerDomLay* nxt[]);
  int getLsn(int i) { return lsn[i];};
  void addProxyLay(rangerProxyLay* pl) { if (pl) ProxyLayers.push_back(pl);};
  float alpha() { return alphaStereo;};
  vector<rangerProxyLay*>& proxylay() { return ProxyLayers;};
  friend ostream& operator<<(ostream &os,const rangerDomLay &dl);
  void dump(const int level=1);

private:
  rangerDomWin* window;
  int lsn[3];   // layer sequence number within window for three methods
  rangerDomLay* nxt[3];
  rangerDomLay* prv[3];
  vector<rangerLayPl*> lpls;  // vector of layer planes
  float zRef;   // reference z value
  int Method;   // assigned method
  float alphaStereo;  // stereo angle in rad
  rangerHL* hL;
  vector<rangerProxyLay*> ProxyLayers;
};
#endif
