// 
//  Author:  Rainer Mankel                     Created:  29-Sep-98
//           Humboldt University Berlin        Modified:  
//  Email:   mankel@ifh.de                     HERA-B Collaboration
// 
// Modified:   
// - moved many routines to State.C
// - introduced accessors to covmatrix elements
//                                     14-Aug-00 Wouter Hulsbergen

// class rangerStateP
// state class for tracking with momentum

#ifndef rangerStateP_HH
#define rangerStateP_HH

#ifndef rangerState_HH
#include "rang/State.hh"
#endif
#ifndef rangerHitBase_HH
#include "rang/HitBase.hh"
#endif

class rangerStateP : public rangerState {

public:
  rangerStateP() : rangerState(5) {}
  //rangerStateP(const rangerStateP& s);  // copy constructor
  virtual ~rangerStateP() {} ;
  //   virtual rangerStateP& operator=(const rangerState& s) { 
  //     rangerState::operator=(s) ; return *this ;}
  //   rangerStateP& operator=(const rangerStateP& s) { rangerState::operator=(s) ; return *this ;}

  virtual rangerFitMode fitMode() const { return ModeP; }
  virtual rangerState* clone() const { return new rangerStateP(*this); } // produce copy

  virtual int predict(const double& zDestination,
		      const double& qPExp,rangerState*& targetState,
		      HepMatrix*& FTransport) const ;  // perform Kalman filter prediction
  virtual int predictOld(const double& zDestination,
			 const double& qPExp,rangerState*& targetState,
			 HepMatrix*& FTransport);  // perform Kalman filter prediction
  virtual void project(HepVector& resid,HepMatrix& HH,rangerHitBase* hit) {
    hit->projectP(resid,HH,this); } // project into measurement space of hit 

public:
  virtual const double qP() const { return p(5); }
  virtual const double I() const  { return 0; }

  virtual void setQp(double qp) { p(5) = qp ; }
  virtual void setI(double qp) {}

protected:

  virtual const double covXQp()  const { return cov().fast(5,1) ; }
  virtual const double covYQp()  const { return cov().fast(5,2) ; }
  virtual const double covTxQp() const { return cov().fast(5,3) ; }
  virtual const double covTyQp() const { return cov().fast(5,4) ; }
  virtual const double covQpQp() const { return cov().fast(5,5) ; }

  virtual const double covXI()  const { return 0; }
  virtual const double covYI()  const { return 0; }
  virtual const double covTxI() const { return 0; }
  virtual const double covTyI() const { return 0; }
  virtual const double covQpI() const { return 0; }
  virtual const double covII()  const { return 0; }

  virtual void setCovXQp(const double v)  { cov().fast(5,1) = v ; }
  virtual void setCovYQp(const double v)  { cov().fast(5,2) = v ; }
  virtual void setCovTxQp(const double v) { cov().fast(5,3) = v ; }
  virtual void setCovTyQp(const double v) { cov().fast(5,4) = v ; }
  virtual void setCovQpQp(const double v) { cov().fast(5,5) = v ; }

  virtual void setCovXI(const double v)  {}
  virtual void setCovYI(const double v)  {}
  virtual void setCovTxI(const double v) {}
  virtual void setCovTyI(const double v) {}
  virtual void setCovQpI(const double v) {}
  virtual void setCovII(const double v)  {}

};

#endif
