// 
//  Author:  Rainer Mankel                     Created:  29-Sep-98
//           Humboldt University Berlin        Modified:  
//  Email:   mankel@ifh.de                     HERA-B Collaboration
// 
// Modifications:
// - Slight modification in inclusion of header files
//                                      14-Aug-00 Wouter Hulsbergen
//

// class rangerVolume
// base class for detector volume

#ifndef rangerVolume_HH
#define rangerVolume_HH

#ifndef __ARTE_HH__
#include "arte/arte.hh"
#endif
#ifndef rangerGeWa_HH
#include "rang/GeWa.hh"
#endif

class rangerNode;
class rangerArrow;

class rangerVolume {

public:
  rangerVolume();                 // default constructor
  rangerVolume(rangerGeWa* gewa); // constructor from gewa
  virtual ~rangerVolume();        // virtual destructor
  inline rangerVolume* next() { return nextVolume; }; 
                                  // next volume (increasing z)
  inline rangerVolume* previous(){ return previousVolume; };
                                  // previous volume (decreasing z)
  inline rangerVolume*& getNext() { return nextVolume; }; 
                                  // next volume (increasing z)
  inline rangerVolume*& getPrevious(){ return previousVolume; };
                                  // previous volume (decreasing z)
  inline const int plNumber() const {return Gewa->plNumber();}; // mangled id
  //const int plNumber() const {return Gewa->plNumber();}; // mangled id
  inline const ArtePointer<GEDE> getGede() const {return Gewa->getGede();}; // 
  inline const ArtePointer<GWAL> getGwal() const {return Gewa->getGwal();}; // 
  inline const rangerGeWa* getGeWa() const {return Gewa;}; // return GeWa

  inline float Cos() const {return rotTot[0][0];}; // total cos of 
                                                   //projected angle
  inline float Sin() const {return rotTot[1][0];}; // total sin of 
                                                   //projected angle
  inline float ms() const {return msValue;}  // multiple scattering
  void setMs(float ms) { msValue = ms ; }

  virtual float zMean() const { return 0.5*(Gewa->zmin() + Gewa->zmax());};
                                                   // mean z for sort
  const double zRef() const { return z; }
  void rotateParam(double x,  // input parameters
		   double y,
		   double zz,
		   double tx,
		   double ty,
		   double& xP, // output parameters
		   double& yP,
		   double& zP,
		   double& txP,
		   double& tyP);
		   
  void rotateArrow(rangerArrow& a,int direction=+1);

  int passedBody(rangerNode* n);    // true if trajectory defined by n
                                    // passes body of volume

  virtual double sLength(double ,
			 double ,
			 double ,
			 double ,
			 double){ cout << "Volume::sLength: Oh no!! this is just"
				       << " the default!!" << endl;
			 return 0;};  // sLengthZ(z,x,y,tx,ty,zInt)
                                       // returns traversed length
                                       // in units of (zmax-zmin),

  float sLengNode(rangerNode* n); 

  virtual int passed(double zz,
		     double x,
		     double y,
		     double tx,
		     double ty){ return (sLength(zz,x,y,tx,ty)>0); }
                                             // true if trajectory 
                                             // passes volume
  static rangerVolume* makeVolume( rangerGeWa* gewa);

protected:
  rangerGeWa* Gewa;    // 
  float rotTot[3][3]; // total rotation matrix
  double z;           // mean z value for sort
  float msValue;      // mean theta-proj**2 * p**2
  int rotated;        // true if rot() ne 1
  int zTilted;        // true if also z axis is tilted
  int totRotated;     // true if rotTot() ne 1
  rangerVolume* nextVolume;       // pointers for linked chain
  rangerVolume* previousVolume;   //
};

#endif
