//--*-C++-*--
#ifndef RCXDATA_H
#define RCXDATA_H

// *****************************************************************************
//
// source:    
//
// type:      source code
//
// created:   26 Jul 2000
//	
// author:    Andreas Gellrich
//            HERA-B Collaboration
//            Humboldt-University Berlin
//            Physics Department
//            Invalidenstr. 110
//            10115 Berlin
//            Germany
//            E-mail: Andreas.Gellrich@desy.de
//
// changes:
// 26 Jul 2000 (AG) creation (code from Denis)
// 18 Aug 2000 (DD) copying track par from mtra fixed
// 21 Sep 2000 (DD) class MATERIAL added
// 21 Sep 2000 (DD) VERTEX::copy(ArtePointer<MVER> mver)
// 26 Sep 2000 (DD) LIKELIHOOD::flag
// 27 Sep 2000 (DD) VERTEX::fillAsSecondary
// 10 Oct 2000 (DD) USE_ARTE switches added
// 11 Oct 2000 (DD) TRACK_PARAMETER::mass added
//
// *****************************************************************************

#ifdef USE_ARTE
#include "arte/arte.hh"
#include "farm/farm.hh"
#include "rich/ricutil.h"
extern "C" {
#include "sltinfo.h"
}
#endif

#ifdef   USE_ROOT_DD
#include "TObject.h"
#include "TFile.h"
#include "TTree.h"
#endif

class datamass {
 public:

#ifdef   USE_ROOT_DD
#ifdef USE_ARTE
  inline TFile* createRootFile(char *keyword, char *title) {
    char *fname=getenv("ROOTFILE");
    if (!fname) {
      cerr<<"createRootFile: ROOTFILE not defined"<<endl;
      Exit(1);
    }
    char rname[256];
    rname[0]=0;
    sprintf(rname,"%srun%d_%s.root",fname,ArtePointer<EVHD>(1)->run,keyword);
    return (new TFile(rname,"RECREATE",title));
  }
#endif
#endif

  template<class T>  void clear(T &c) {
    memset(&c,0x0,sizeof(T));
  }

  class HERAB_EVENT {
  public:
    int run;
    int event;
    int bx;
    int flt;
    int slt;
    int vds_occup;
    int itr_occup;
    int otr_occup;
    int hpt_occup;
    int rich_occup;
    int ecal_occup;
    int muon_occup;
#ifdef USE_ARTE
    inline void copy() {
      run   = ArtePointer<EVHD>(1)->run;
      event = ArtePointer<EVHD>(1)->evt;
      bx    = ArtePointer<DIHD>(1)->bx;
      flt   = faDDAQFCS_tcode();
      slt   = 0;//SLT_TriggerCount(SLTTYPE_DIMUON)+100*SLT_TriggerCount(SLTTYPE_DIELECTRON);
      vds_occup  = ArtePointer<EVHI>(1)->hit;
      otr_occup  = ArtePointer<EVHI>(2)->hit;
      itr_occup  = ArtePointer<EVHI>(3)->hit;
      hpt_occup  = ArtePointer<EVHI>(4)->hit;
      rich_occup = ArtePointer<EVHI>(5)->hit;
      ecal_occup = ArtePointer<EVHI>(7)->hit;
      muon_occup = ArtePointer<EVHI>(8)->hit+ArtePointer<EVHI>(9)->hit;
    }
#endif
#ifdef   USE_ROOT_DD
    inline void branch(TTree *tree, char *name) {
      tree->Branch( name, this,"run/I:evt:bx:flt:slt:vds:itr:otr:hpt:rich:ecal:muon");
    }
#endif
  };

  class TRACKER_SEGMENT  {
  public:
    Float_t x,y,z;
    Float_t tx,ty;
    Float_t p;
    Float_t chi;
    Int_t   hit1, hit2, hit3;
    Int_t   cmp;
    Float_t dtx,dty;
    Float_t dx,dy;
#ifdef USE_ARTE
    inline void copy(ArtePointer<RSEG> seg) {
      x=seg->xe;
      y=seg->ye;
      z=seg->ze;
      tx=seg->txe;
      ty=seg->tye;
      p =seg->pe;
      chi=seg->chi;
      cmp=seg->cmp;
    }
#endif
#ifdef   USE_ROOT_DD
    inline void branch(TTree *tree, char *name) {
      tree->Branch( name, this,"x/F:y:z:tx:ty:p:chi:hit1/I:hit2:hit3:cmp:dtx/F:dty:dx:dy");
    }
#endif
  }; // tracker

 
  class RICH_RING  {
  public:
    Float_t tx,ty;
    Float_t rad;
    Float_t pho, sn;
    Int_t   cmp,hri;
    Float_t dtx,dty;
#ifdef USE_ARTE
    inline void copy(ArtePointer<RSEG> ring) {
      tx=ring->txf; 
      ty=ring->tyf; 
      rad=ring->time;
      pho=ring->ze; 
      sn=(ring->ye>0) ? ring->ze/sqrt(ring->ye) : 100; 
      hri=ring->nhit;
    }
    inline void copy(ArtePointer<RTRA> track) {
      tx =track->txr;
      ty =track->tyr;
      rad=track->cer;
      hri=track->hri;
    }
#endif
#ifdef   USE_ROOT_DD
    inline void branch(TTree *tree, char *name) {
      tree->Branch( name, this,"tx/F:ty:rad:pho:sn:cmp/I:hri:dtx/F:dty");
    }
#endif
  }; // RICH
  

  class LIKELIHOODS {
  public:
    float e;
    float mu;
    float pi;
    float k;
    float p;
    Int_t flag;
#ifdef USE_ARTE
    inline void copy(ArtePointer<RTRA> tr) {
      e =tr->lre;
      mu=tr->lrmu;
      pi=tr->lrpi;
      k =tr->lrk;
      p =tr->lrp;
      flag = tr->flr;
    }
    inline void copyRISE(ArtePointer<RTRA> tr) {
      if (!ArteTable<RTRI>::size()) return;
      ArtePointer<RTRI> rtri(tr.index());
      e =rtri->lte;
      mu=rtri->ltmu;
      pi=rtri->ltpi;
      k =rtri->ltk;
      p =rtri->ltp;
      flag = tr->flr;
    }
    inline void copyRITE(ArtePointer<RTRA> tr) {
      if (!ArteTable<RTRI>::size()) return;
      ArtePointer<RTRI> rtri(tr.index());
      e =rtri->lse;
      mu=rtri->lsmu;
      pi=rtri->lspi;
      k =rtri->lsk;
      p =rtri->lsp;
      flag = tr->flr;
    }
#endif
#ifdef   USE_ROOT_DD
    inline void branch(TTree *tree, char *name) {
      tree->Branch( name, this,"e/F:mu:pi:k:p:flag/I");
    }
#endif
  }; // particle ID

  class TYPE {
  public:
    int lund;
    int mother;
    int charge;
#ifdef USE_ARTE
    inline void copy(ArtePointer<MTRA> mct) {
      lund   = mct->topp->lund;
      mother = mct->btra->topp->lund;
      charge = mct->topp->chrg/3;
    }
    inline bool copy(ArtePointer<RTRA> rct) {
      if (rct->mtra.begin()==rct->mtra.end()) return false;
      copy(*rct->mtra.begin());
      return true;
    }
    inline bool copy(ArtePointer<RCCL> ect) {
      if (ect->mtra.begin()==ect->mtra.end()) return false;
      copy(*ect->mtra.begin());
      return true;
    }
    inline bool copy(ArtePointer<RSEG> rise) {
      if (rise->mtra.begin()==rise->mtra.end()) return false;
      copy(*rise->mtra.begin());
      return true;
    }
#endif
#ifdef   USE_ROOT_DD
    inline void branch(TTree *tree, char *name) {
      tree->Branch( name, this,"lund/I:mother:charge");
    }
#endif
  }; // particle type

  class ECAL_CLUSTER {
  public:
    Float_t x,y,z;
    Float_t esum;
    Int_t   wid;
    Int_t   cmp;
    Float_t dx,dy;
#ifdef USE_ARTE
    inline void copy(ArtePointer<RCCL> clust) {
      x=clust->x; 
      y=clust->y; 
      z=clust->z;
      esum=clust->esum; 
      wid=(int)clust->wid;
    }
#endif
#ifdef   USE_ROOT_DD
    inline void branch(TTree *tree, char *name) {
      tree->Branch( name, this,"x:y:z:esum:wid/I:cmp:dx/F:dy");
    }
#endif
  }; // ECAL


  class VERTEX  {
  public:
    Float_t x,y,z,tx,ty;
    Float_t mass,p,pt,ptm;
    Float_t t0c,L;
    Float_t chi;
    Float_t ex,ey, ez;
    Int_t   nt;
#ifdef USE_ARTE
    inline void copy(ArtePointer<RVER> vert) {
      x=vert->xv;
      y=vert->yv;
      z=vert->zv;
      mass=vert->mass;
      if (vert->cv[0]>0) ex=sqrt(vert->cv[0]); 
      if (vert->cv[2]>0) ey=sqrt(vert->cv[2]); 
      if (vert->cv[5]>0) ez=sqrt(vert->cv[5]); 
      nt=vert->nt;
    }
    inline void copy(ArtePointer<MTRA> t) {
      ArtePointer<MVER> mver;
      if (t->ever.begin()==t->ever.end())  mver=t->bver;
      else                                 mver=(*t->ever.begin());
      L=t->leng;
      copy(mver);
    }
    inline void copy(ArtePointer<MVER> mver) {
      x=mver->x;
      y=mver->y;
      z=mver->z;
    }
    inline void fillAsSecondary(ArtePointer<RVER> v0, float m) {
      register float dx=x-v0->xv;
      register float dy=y-v0->yv;
      register float dz=z-v0->zv;
      // -- decay length (sign from dz)
      L=sqrt(dx*dx+dy*dy+dz*dz);
      if (dz<0) L=-L;
      // -- proper time
      float beta=1/sqrt(1+m*m*p*p);
      t0c=L/beta*sqrt(1-beta*beta);
      // -- pt with respect to v0v1 line
      ptm=1./fabs(p)*sqrt(1.-1./(dx*dx/(dz*dz)+dy*dy/(dz*dz)+1));
    }
#endif
#ifdef   USE_ROOT_DD
    inline void branch(TTree *tree, char *name) {
      tree->Branch( name, this,"x/F:y:z:tx:ty:m:p:pt:ptm:t0c:L:chi:xe:ye:ze:nt/I");
    }
#endif
  }; // vertex


  class TARGET  {
  public:
    Float_t x,y,z;
    Float_t eff;
#ifdef USE_ARTE
    inline void copy(ArtePointer<GTAR> tar) {
      x   = tar->x;
      y   = tar->y;
      z   = tar->z;
      eff = tar->eff;
    }
#endif
#ifdef   USE_ROOT_DD
    inline void branch(TTree *tree, char *name) {
      tree->Branch( name, this,"x/F:y:z:eff");
    }
#endif
  }; // target


  class TRACK_PARAMETER {
  public:
    Float_t x,y,z;
    Float_t tx,ty;
    Float_t ip,pt,ptm;
    Float_t mass;
    Float_t chi;
#ifdef USE_ARTE
    inline void copy(ArtePointer<RTRA> track) {
      x=track->xf;   
      y=track->yf;   
      z=track->zf;
      tx=track->txf; 
      ty=track->tyf; 
      ip=track->pf;
      Float_t pz=1/fabs(ip*sqrt(track->txf*track->txf + track->tyf*track->tyf + 1));
      Float_t px=pz*track->txf;
      Float_t py=pz*track->tyf;
      pt=sqrt(px*px+py*py);
      ptm=0;
      mass=0;
      if (track->fit%100==2 || track->fit%100==3) mass=track->ever->mass;
      chi=track->chi;
    }
    inline void copy(ArtePointer<MTRA> track) {
      ArtePointer<MVER> ver=track->bver;
      x=ver->x;   
      y=ver->y;   
      z=ver->z;
      tx=track->px/track->pz; 
      ty=track->py/track->pz; 
      ip=1./sqrt(track->px*track->px+track->py*track->py+track->pz*track->pz);
      if (track->topp->chrg<0) ip=-ip;
      Float_t pz=1/fabs(ip*sqrt(tx*tx + ty*ty + 1));
      Float_t px=pz*tx;
      Float_t py=pz*ty;
      pt=sqrt(px*px+py*py);
      ArtePointer<MTRA> btra=track->btra;
      float dtxm=tx-btra->px/btra->pz;
      float dtym=ty-btra->py/btra->pz;
      // pz in direction of mother particle
      Float_t pzm=1/(dtxm*dtxm + dtym*dtym + 1);
      ptm=sqrt(1-pzm)/fabs(ip);
      mass=track->topp->mass;
      chi=0;
    }
#endif
#ifdef   USE_ROOT_DD
    inline void branch(TTree *tree, char *name) {
      tree->Branch( name, this,"x/F:y:z:tx:ty:p:pt:ptm:m:chi");
    }
#endif
  }; // general track point


  class MATERIAL  {
  public:
    Float_t A,Z;
    Float_t dens, radl, absl, loss;
    Int_t   flag, no;
#ifdef USE_ARTE
    inline void copy(ArtePointer<GMAT> mat) {
      A    = mat->a;
      Z    = mat->z;
      dens = mat->dens;
      radl = mat->radl;
      absl = mat->absl;
      loss = mat->loss;
      flag = mat->flag;
      no   = mat.index();
    }
#endif
#ifdef   USE_ROOT_DD
    inline void branch(TTree *tree, char *name) {
      tree->Branch( name, this,"A/F:Z:dens:radl:absl:loss:flag/I:no");
    }
#endif
  }; // material (GMAT)


};

#endif
