//--*-C++-*--
#ifndef RCXDECAY_H
#define RCXDECAY_H

// *****************************************************************************
//
// source:    
//
// type:      source code
//
// created:   26 Jul 2000
//	
// author:    Andreas Gellrich
//            HERA-B Collaboration
//            Humboldt-University Berlin
//            Physics Department
//            Invalidenstr. 110
//            10115 Berlin
//            Germany
//            E-mail: Andreas.Gellrich@desy.de
//
// changes:
// 26 Jul 2000 (AG) creation (code from Denis)
// 04 Aug 2000 (DD) wire constraint and mass calc. can be skipped
// 25 Sep 2000 (DD) min_pt2_of_tracks added
//
// *****************************************************************************

#include "arte/arte.hh"

/// Description of decay
class decay {
 public:
  decay();
  decay(decay &d);
  ~decay() {;}
  
  /// fit flag for recon. mass in RTRA bank
  int               fit;
  /// flag for booking recon. mass
  int               book_tables;
  /// recon. level
  int               rlevel;
  /// create tree in a root file
  int               root_output;
  /// flag for mass calculation
  int               calculate_mass;
  /// verbosity
  int               verb;
};


/// Description of vertexing
class vertexAttrib {
public:
  vertexAttrib();
  vertexAttrib(vertexAttrib &va);
  ~vertexAttrib() {;}

  /// rlevel: 0=no vertexing, 1=chi^2 2=kalman(T.Lohse) 3=kalman+mass
  int   rlevel;
  /// momentum flag (only used in kalman fit w/o mass constraint)
  int   momentum_flag;
  /// remove old vertices (or leave for refit) = 1 (0)
  /// Only removes kalman fit banks, RVER's are not touched
  int   remove_old_vertices;
  /// Maximum chi2 for vertexing
  float max_vertex_chi2;
  /// Mother TOPP pointer
  ArtePointer<TOPP> topp;
  /// verbosity
  int   verb;
  /// Minimum distance form primary vertex
  float min_total_mtra_leng;
  /// Require minimum MC length
  float min_mtra_leng;
  /// Require that vertex originates after a wire
  float min_dz_from_wire;
  /// Require that vertex points to a wire
  float max_drho_from_wire;
  /// Min/max mass range
  float min_mass, max_mass;
  /// Check wire after fit
  int check_wire;
  /// Miinimum tracks in primary vertex
  int min_tracks_in_v0;
  /// Exclude tracks that point to V0
  int allowed_v0_tracks;
  /// Require that tracks point to V0
  int require_v0_tracks;
  /// Change parameters of tracks used in vertex fit (mass fit only)
  int change_track_rtra;
  /// Minimum pt^2 of decay products with respect to mother direction
  float min_pt2_of_tracks;
  /// Maximum pt^2 of decay products with respect to mother direction
  float max_pt2_of_tracks;
  /// Minimum proper time s*3e10cm/s
  float min_t0c;
  /// Minimum proper time s*3e10cm/s
  float max_t0c;
};

#endif
