//--*-C++-*--
#ifndef RCXKINEMA_H
#define RCXKINEMA_H

// *****************************************************************************
//
// source:    
//
// type:      source code
//
// created:   26 Jul 2000
//	
// author:    Andreas Gellrich
//            HERA-B Collaboration
//            Humboldt-University Berlin
//            Physics Department
//            Invalidenstr. 110
//            10115 Berlin
//            Germany
//            E-mail: Andreas.Gellrich@desy.de
//
// changes:
// 26 Jul 2000 (AG) creation (code from Denis)
// 04 Apr 2001 (DD) swim from RTRA
// 21 Apr 2001 (DD) xf added
//
// *****************************************************************************

#include "arte/arte.hh"
#include "CLHEP/Vector/LorentzVector.h"


class kinema {

 public:
  /// Brehmsstrahlung recovery
  static float findBremsstrahlung(ArtePointer<RTRA> t);

  /// Four-vector creation
  static HepLorentzVector FourVector(ArtePointer<RTRA> t, float m);

  /// Four-vector creation
  static HepLorentzVector FourVector(ArtePointer<RTRA> t);

  /// Four-vector creation
  static HepLorentzVector FourVector(ArtePointer<MTRA> t);

  /// Swimming through magnet
  static int swim(const double *source,double *dest,const double zout);

  /// Swimming through magnet + RPNT creation
  static int swim(ArtePointer<RTRA> rtra, double zout);

  /// Find line in TOPP table for given lund
  static ArtePointer<TOPP> findTopp(int lund);

  /// Get Feynman X for decay track t
  static float XF(ArtePointer<RTRA> t);

  /// Get Feynman X for decay track t
  static float XF(ArtePointer<MTRA> t);

  /// Get Feynman X for 4-vector vec
  static float XF(HepLorentzVector vec);

  /// Get CM angle of daughter and mother particle
  static float angleCM(HepLorentzVector daughter, HepLorentzVector mother);

  /// Get CM angle of MC particle
  static float angleCM(ArtePointer<MTRA> mtra);
};

#endif
