
// *****************************************************************************
//
// source:    rcFillEVTI.C
//
// type:      source code
//
// created:   16 Jun 2000
//	
// author:    Dr. Andreas Gellrich
//            HERA-B Collaboration
//            Humboldt-University Berlin
//            Physics Department
//            Invalidenstr. 110
//            10115 Berlin
//            Germany
//            E-mail: Andreas.Gellrich@desy.de
//
// symbol:    rcFillEVTI
//
// called by: rcevnt_
//
// purpose:   fill time table EVTI
//
// hints:     EVTI is NOT written to output streams
//
// changes:
// 16 Jun 2000 (AG) creation
// 19 Jun 2000 (AG) fill new row when called
// 13 Nov 2000 (AG) changed OTR to RANGER
// 02 Jan 2000 (AG) change time variable names
//
// *****************************************************************************

// ARTE includes
#include "arte/arte.hh"

// externals
extern float time_rcVDS;
extern float time_rcRANGER;
extern float time_rcRICH;
extern float time_rcTRD;
extern float time_rcECAL;
extern float time_rcMUON;
extern float time_rcMatch;
extern float time_rcRefit;
extern float time_rcParticleID;
extern float time_rcVertex;

//
// program code
//
void
rcFillEVTI()
{
  // create new row
  ArtePointer<EVTI> evti = ArteTable<EVTI>::push_back( EVTI() );

  // fill in times    
  evti->vds    = time_rcVDS;
  evti->ranger = time_rcRANGER;
  evti->refit  = time_rcRefit;
  evti->rich   = time_rcRICH;
  evti->ecal   = time_rcECAL;
  evti->trd    = time_rcTRD;
  evti->muon   = time_rcMUON;
  evti->marple = time_rcMatch;
  evti->vertex = time_rcVertex;
  evti->pid    = time_rcParticleID;
}
