
// *****************************************************************************
//
// source:    rcMarkSLTtracks.C
//
// type:      source code
//
// created:   8 DEC 2000
//
// author:    Brian Petersen
//            email: aagaard@nbi.dk
//
// purpose:   Find RTRA's, which match SLT trigger tracks and mark
//            them
//
// note:      Marking is done using bit8-bit10 in component field
//
// changes:         
//  8 Dec 2000 (BP) Creation
// 11 Dec 2000 (AG) move to ARTE/dev
// 27 May 2001 (IR) Rsegc::bit8 is substituted by Rsegc::slte, Rsegc::bit9 by Rsegc::sltmu
//
// *****************************************************************************

// ARTE includes
#include "arte/arte.hh"
#include "arte/RSEG_cmp.hh"
#include "farm/farm.hh"
#include "sltinfo.h"
#include "sltarte/matchslt.hh"

// externals
extern int FADEB;

// defines
#define VDSCUT 55.0
#define OTRCUT 55.0

//
// code
//
void
rcMarkSLTtracks()
{
  ARTE_mark( "rcMarkSLTtracks 0", 0 );

  if (SLTReadDSLT()!=SLT_OK){
    ARTE_mark( "rcMarkSLTtracks 1", 0 );
    return;
  }

  SLT_STATUS stat;
  SLT_TRIGGER trig;

  // Only fill for good events
  if (SLTStatus(&stat)!=SLT_OK){
    ARTE_mark( "rcMarkSLTtracks 1", 0 );
    return;
  }
  if (!(stat.status&SLTEVENT_ACCEPT)){
    ARTE_mark( "rcMarkSLTtracks 1", 0 );
    return;
  }

  // counters
  int nMuon     = 0;
  int nElectron = 0;

  // Find the accepted trigger pairs
  for(int trignum=0;trignum<stat.ntriggers;trignum++) {

    if (SLTTrigInfo(trignum,&trig)!=SLT_OK) continue;
    if (trig.status==SLTTRIG_REJECT)  continue;

    ArtePointer<RTRA> rtrk;
    float chix,chiy;
    float rchix,rchiy;
    for(int j=0;j<trig.ntracks;j++) {

      MatchSltRTRA(trig.trackindex[j],rtrk,chix,chiy,rchix,rchiy);
      if ((chix<VDSCUT||chiy<VDSCUT)&&rchix<OTRCUT&&rchiy<OTRCUT) {
	SLT_SEEDTRACK seed;
	if( SLTSeedTrackInfo( trig.trackindex[0], &seed) != SLT_OK) continue;

	if (seed.trackID==EPLUS||seed.trackID==EMINUS){
	  nElectron++;
	  rtrk->cmp|=Rsegc::slte;
	}

	if (seed.trackID==MUPLUS||seed.trackID==MUMINUS){
	  nMuon++;
	  rtrk->cmp|=Rsegc::sltmu;
	}

	// Bit10 would be high-pt hadron, but they are not supported yet.
      }
    }
  }

  // print
  if( FADEB ){
    cout << "rcMarkSLTtracks: "
	 << "nElectron=" << nElectron << " "
	 << "nMuon="     << nMuon     << endl;
  }
  
  ARTE_mark( "rcMarkSLTtracks 0", 0 );
  return;
}
