
// ********************************************************************
//
// Routine:      rcSelTARGETtracks.C
// Author:       T. Oest
//
// Description:  VDS/RICH/TARGET match
//
// Modification:
// 03 Jan 2000 (AG) removed 99
//
// ********************************************************************

// includes
#include <iostream.h>
#include <stdlib.h>
#include <math.h>
#include <algo.h>

#include "arte/arte.hh"

#include "reco/rc99fit98.hh"

extern float MatchChi2Cut;
extern float tyRICHcorr;
extern float MatchdyMaxCut;
extern float MatchpCut;

//******************************************************************
// function
//******************************************************************
void rcSelTARGETtracks()
{
    float xECAL, yECAL, zECAL;
    float xTARGET = 0.0;
    float yTARGET = 0.0;
    float zTARGET = 0.0;

const int index[5][5] = { {0, 1,  3, 6,10},
                          {1, 2,  4, 7,11},
                          {3, 4,  5, 8,12},
                          {6, 7,  8, 9,13},
                          {10,11,12,13,14}};
const int index2[5][5] = { {0,1, 2, 3, 4},
                           {1,5, 6, 7, 8},
                           {2,6, 9,10,11},
                           {3,7,10,12,13},
                           {4,8,11,13,14}};
//-----------------------------------------------------------------
//   zTarget
//-----------------------------------------------------------------
    float diff;

    for( ArteTable<GTAR>::iterator gtar = ArteTable<GTAR>::begin();
         gtar != ArteTable<GTAR>::end();
         ++gtar ) {
      diff = (*gtar)->x * (*gtar)->x + (*gtar)->y * (*gtar)->y;
      if (diff < 1 ) {
        xTARGET  = (*gtar)->x;
        yTARGET  = (*gtar)->y;
        zTARGET  = (*gtar)->z;
      }
    };

//-----------------------------------------------------------------
//
//-----------------------------------------------------------------
    float pinv, ytarget, xtarget, tyRICH, txRICH, ycut;

      if (MatchChi2Cut > 1000.){
       ycut = MatchdyMaxCut; 
      } else {
       ycut = 2.;
      };

    for( ArteTable<RCCL>::iterator rccl = ArteTable<RCCL>::begin();
         rccl != ArteTable<RCCL>::end();
         ++rccl ) {

      if ((*rccl)->wid >= 2.){

      xECAL    = (*rccl)->x;
      yECAL    = (*rccl)->y;
      zECAL    = (*rccl)->z;


    for( ArteTable<RSEG>::iterator rRICH = ArteTable<RSEG>::begin();
         rRICH != ArteTable<RSEG>::end();
         ++rRICH ) {

      if ((*rRICH)->cmp == 67108864 &&
          (*rRICH)->ze >= 3.*sqrt(max(float(1.),(*rRICH)->ye))){


        tyRICH = (*rRICH)->tyf + tyRICHcorr;
        txRICH = (*rRICH)->txf;

        ytarget = yECAL + tyRICH * (zTARGET - zECAL);
        xtarget = xECAL + txRICH * (zTARGET - zECAL);
        pinv = - xtarget / 280.;     //  1/p


      if (fabs(ytarget) < ycut ) {


        float zmag = 450.;
        float xmag;
        xmag = xECAL + (zmag - zECAL) * (*rRICH)->txf;

        double pstart[6];
        pstart[0] = zTARGET;
        pstart[1] = xTARGET;
        pstart[2] = yTARGET;
        pstart[3] = xmag / (zmag - zTARGET);
        pstart[4] = tyRICH;
        pstart[5] = pinv;



       double z, pMeas[5], wMeas[5][5];
        for (int i=0;i<5;i++){for (int j=0;j<5;j++){wMeas[i][j]=0.0;};}


        track_98 track(pstart);

	// target spot 

        z        = zTARGET;
        pMeas[0] = xTARGET;
        pMeas[1] = yTARGET;
        pMeas[2] = pstart[3];
        pMeas[3] = pstart[4];
        pMeas[4] = pstart[5];
        wMeas[0][0] = 10.0;
        wMeas[1][1] = 10.0;

        track.add_measurement(z, &pMeas[0], &wMeas[0][0]);

	// ECAL, RICH

        float a;
 

        z        = zECAL;
        pMeas[0] = xECAL;
        pMeas[1] = yECAL;
        pMeas[2] = txRICH;
        pMeas[3] = tyRICH;
        pMeas[4] = pstart[5];

        a = (*rccl)->cve[0] * (*rccl)->cve[2] - 
            (*rccl)->cve[1] * (*rccl)->cve[1];
        if (a != 0.0){
         wMeas[0][0] =   (*rccl)->cve[2] / a;
         wMeas[1][1] =   (*rccl)->cve[0] / a;
         wMeas[1][0] = - (*rccl)->cve[1] / a;
         wMeas[0][1] = wMeas[1][0];
        } else {
         wMeas[0][0] = 1./(*rccl)->cve[0];
         wMeas[1][1] = 1./(*rccl)->cve[2];
         wMeas[1][0] = 0.0;
         wMeas[0][1] = 0.0;
        };

        a = (*rRICH)->cvf[5]  * (*rRICH)->cvf[9] - 
            (*rRICH)->cvf[8]  * (*rRICH)->cvf[8];
        if (a != 0.0){
         wMeas[2][2] =   (*rRICH)->cvf[9] / a;
         wMeas[3][3] =   (*rRICH)->cvf[5]  / a;
         wMeas[3][2] = - (*rRICH)->cvf[8]  / a;
         wMeas[2][3] = wMeas[3][2];
        } else {
         wMeas[2][2] = 1./(*rRICH)->cvf[5];
         wMeas[3][3] = 1./(*rRICH)->cvf[9];
         wMeas[3][2] = 0.0;
         wMeas[2][3] = 0.0;
        };

        track.add_measurement(z, &pMeas[0], &wMeas[0][0]);

	// fit
        double chi2 = track.fit();


        double * parameters, * covariance;;
        parameters  = track.track_parameters();
        float p     = 1./fabs(float(parameters[5]));
        covariance  = track.track_errors();


        if (chi2 < MatchChi2Cut && chi2 >0.0 &&
            p > MatchpCut){

        RTRA rtra;

        rtra.hsi = 0;
        rtra.hin = 0;
        rtra.hou = 0;
        rtra.hri = 0;
        rtra.hca = 0;
        rtra.htr = 0;
        rtra.hmu = 0;

        rtra.lre = 0.0;
        rtra.lrmu= 0.0;
        rtra.lrpi= 0.0;
        rtra.lrk = 0.0;
        rtra.lrp = 0.0;
        rtra.flr = 0;

        rtra.lte = 0.0;
        rtra.lth = 0.0;
        rtra.flt = 0;

        rtra.lee = 0.0;
        rtra.lemi= 0.0;
        rtra.leh = 0.0;
        rtra.fle = 0;

        rtra.lmmu= 0.0;
        rtra.lmh = 0.0;
        rtra.flm = 0;


        rtra.fit = 0;

	//        rtra.xf  = xtarget;
	//        rtra.yf  = ytarget;
	//        rtra.xf  = float(parameters[1]);
	//        rtra.yf  = float(parameters[2]);
	rtra.xf  = xTARGET;
	rtra.yf  = yTARGET;
        rtra.zf  = zTARGET;
        rtra.txf = float(parameters[3]);
        rtra.tyf = float(parameters[4]);
        rtra.pf  = float(parameters[5]);
        rtra.chi = float(chi2);

        rtra.lre = (*rRICH)->time;
        rtra.lee = fabs((*rccl)->esum * float(parameters[5])) ;

	rtra.cvf[index[0][0]] = 0.1;
	rtra.cvf[index[1][1]] = 0.1;

        for (int i=0; i<=4; i++){for (int j=0; j<=4; j++){
          if (i >= 2 && j >= 2){
            rtra.cvf[index[i][j]] = covariance[i*5+j];
          } else {
	   if (i != j) rtra.cvf[index[i][j]] = 0.0;
          };
        };};

      ArteTable<RTRA>::push_back( rtra );

      ArtePointer<RTRA> Prtra(ArteTable<RTRA>::size());

      createRelation( Prtra, (*rRICH), &RTRA::rseg );
      createRelation( Prtra, (*rccl), &RTRA::rccl );

        if (ArteTable<MTRA>::size() > 0 && (*rRICH)->fit != 0){
         ArtePointer<MTRA> mtra ((*rRICH)->fit);
         for (ArteRelationIterator<MTRA> mccl = rccl->mtra.begin();
              mccl != rccl->mtra.end(); mccl++){
         if (mtra == (*mccl))
          createRelation( Prtra, mtra, &RTRA::mtra );
	 };};


      };};

    };};};


      };

      
};



