
// *****************************************************************************
//
// source:    
//
// type:      source code
//
// created:   26 Jul 2000
//	
// author:    Andreas Gellrich
//            HERA-B Collaboration
//            Humboldt-University Berlin
//            Physics Department
//            Invalidenstr. 110
//            10115 Berlin
//            Germany
//            E-mail: Andreas.Gellrich@desy.de
//
// changes:
// 26 Jul 2000 (AG) creation (code from Denis)
// 07 Aug 2000 (DD) electron selection (D.S.) 
// 05 Sep 2000 (DD) bug fix, always check rich likelihoods 
// 15 Sep 2000 (DD) refit of vertex flag
// 19 Sep 2000 (DD) printout added
// 20 Sep 2000 (DD) checking for max chi2 added
// 17 Ict 2000 (DD) use_missing_p added
// 20 Oct 2000 (DD) pionRISEorRITER, pionRISEorRITER
//
// *****************************************************************************

#include "reco/rcXattribute.h"
#include "reco/rcXkinema.h"

attribute::attribute() {

  /// Momentum cut 
  minPt=0,minP=-1e5,maxP=1e5;

  /// Track quality
  maxChi=1e5;

  /// VDS quality
  minNhits=minPhits=minHSI=-1000; 
  
  /// ITR/OTR quality
  minMClayers=minPClayers=minTClayers=minHOU=minHIN=minHIO=-1000;

  /// RICH particle id
  maxElectronR2P2=maxMuonR2P2=maxPionR2P2=maxKaonR2P2=maxProtonR2P2=1e5;
  minLRE=minLRMU=minLRPI=minLRK=minLRP=-1e5;
  minLREMUPI=minLRKP=-1e5;
  /// RICH ring quality
  minRichRadius=-1e5, maxRichRadius=1e5, minRichSN=-1e5, minHRI=-1000;
  /// Exclude RICH bands
  maxLRE=maxLRMU=maxLRPI=maxLRK=maxLRP=1e5;
  /// Special ID algorithms
  pionRISEorRITER=kaonRISEorRITER=0;

  /// ECAL quality
  maxEcalEnergy=1e5, minEcalWid=-1, minEoP=-1e5, maxEoP=1e5;
  minHCA=-1000;

  /// ECAL particle id
  minLEE=minLEMI=minLEH=-1e-5; 
  maxLEE=maxLEMI=maxLEH= 1e5;

  /// MUON quality
  minHMU=-1000;

  /// Combinations
  minHCAMU=minHRIOU=minHRIIN=-1000;

  /// MC truth
  lund=mother=0;

  /// TOPP
  topp=0;
  matchTOPP=0;

  /// name
  sprintf(name,"t");

  /// fit type
  fit=0;

  /// mass range for reconstructed decays
  minMass=-1e5, maxMass=1e5, mass=-1e5;

  /// Vertex Position
  minVertexZ=-1e5, maxVertexZ=1e5;

  /// Downstream vertex (not pointing to the target)
  min_dz_from_wire=-1e5;

  /// Minimum impact parameter at primary vertex
  min_drho2_from_wire=-1e5;

  /// bremsstrahlung
  recoverBremsstrahlung=0;

  /// Use vds segment only once
  useVdsOnce=0;

  /// Use main tracker segment once
  useMainOnce=0;

  /// Clone
  clone=1;

  /// Maximum candidates
  maxCandidate=100000;

  /// Special algorithm to select electrons
  eDora=0;

  /// Special requirements to select muons
  muKen=0;

  /// Refit of vertex with mass constraint in case of a decay
  mass_refit=0;

  /// Use MC truth
  use_mc_p=use_mc_coordinates=use_mc_directions=false;

  /// Use missing momentum analysis p=m/M*P where P,M are momentum/inv. mass
  /// of all other tracks in the decay
  use_missing_p=0;
}


void attribute::print() {
  cout<<"minPt="<<minPt<<" minP="<<minP<<" maxP="<<maxP<<endl;
  cout<<"maxChi="<<maxChi<<endl;
  cout<<"minNhits="<<minNhits<<" minPhits="<<minPhits<<" minHSI="<<minHSI<<endl;
  cout<<"minMClayers="<<minMClayers<<" minPClayers="<<minPClayers<<" minTClayers="<<minTClayers
      <<" minHOU="<<minHOU<<" minHIN="<<minHIN<<" minHIO="<<minHIO<<endl;
  cout<<"maxElectronR2P2="<<maxElectronR2P2<<" maxMuonR2P2="<<maxMuonR2P2<<" maxPionR2P2="<<maxPionR2P2
      <<" maxKaonR2P2="<<maxKaonR2P2<<" maxProtonR2P2="<<maxProtonR2P2<<endl;
  cout<<"minLRE="<<minLRE<<" minLRMU="<<minLRMU<<" minLRPI="<<minLRPI<<" minLRK="<<minLRK<<" minLRP="<<minLRP<<endl;
  cout<<"minLREMUPI="<<minLREMUPI<<"minLRKP="<<minLRKP<<endl;
  cout<<"maxLRE="<<maxLRE<<" maxLRMU="<<maxLRMU<<" maxLRPI="<<maxLRPI<<" maxLRK="<<maxLRK<<" maxLRP="<<maxLRP<<endl;
  cout<<"minRichRadius="<<minRichRadius<<" maxRichRadius="<<maxRichRadius<<" minRichSN="<<minRichSN
      <<" minHRI="<<minHRI<<endl;
  cout<<"maxEcalEnergy="<<maxEcalEnergy<<" minEcalWid="<<minEcalWid
      <<" minEoP="<<minEoP<<" maxEoP="<<maxEoP<<" minHCA="<<minHCA<<endl;
  cout<<"minLEE="<<minLEE<<" minLEMI="<<minLEMI<<" minLEH="<<minLEH<<endl;
  cout<<"maxLEE="<<maxLEE<<" maxLEMI="<<maxLEMI<<" maxLEH="<<maxLEH<<endl;
  cout<<"minHMU="<<minHMU<<endl;
  cout<<"minHCAMU="<<minHCAMU<<" minHRIOU="<<minHRIOU<<" minHRIIN="<<minHRIIN<<endl;
  cout<<"pionRISEorRITER="<< pionRISEorRITER<<"   kaonRISEorRITER"<<kaonRISEorRITER<<endl;
  cout<<"lund="<<lund<<" mother="<<mother<<" topp="<<topp<<" name="<<name<<" matchTOPP="<<matchTOPP<<endl;
  cout<<"fit="<<fit<<endl;
  cout<<"minMass="<<minMass<<" maxMass="<<maxMass<<" mass="<<mass<<endl;
  cout<<"minVertexZ="<<minVertexZ<<" maxVertexZ="<<maxVertexZ
      <<" min_dz_from_wire="<< min_dz_from_wire<<" mass_refit="<<mass_refit<<endl;
  cout<<"recoverBremsstrahlung="<<recoverBremsstrahlung<<endl;
  cout<<"useVdsOnce="<<useVdsOnce<<" clone="<<clone<<" maxCandidate="<<maxCandidate<<endl;
  cout<<"eDora="<<eDora<<" muKen="<<muKen<<endl;
  cout<<"mc truth: p="<<use_mc_p<<" xyz="<<use_mc_coordinates<<" txty="<<use_mc_directions
      <<"use_missing_p"<<use_missing_p<<endl;
}


int attribute::checkAttributes(ArtePointer<RTRA> track) {


  // -- charged track only
  if (track->fit%100!=fit)                                         return RCX_WRONG_FIT;

  // -- mass and vertex range for reconstructed decays
  if (track->fit%100==2 || track->fit%100==3) {
    if (track->ever->mass>maxMass  || track->ever->mass<minMass)   return RCX_MASS_CUT;
    if (track->ever->zv<minVertexZ || track->ever->zv>maxVertexZ)  return RCX_DECAY_Z_CUT;
    if (matchTOPP && track->topp!=topp)                            return RCX_TOPP;
  }

  // -- chi2 cut
  if (track->chi>maxChi)                                           return RCX_TRACK_CHI2;


  // -- momentum 
  float p= (fabs(track->pf)>0) ? 1/track->pf : 0;
  float pt=fabs(p)*sqrt(1-1/(track->txf*track->txf+track->tyf*track->tyf+1));
  if (p<minP)                         return RCX_P_MIN;
  if (p>maxP)                         return RCX_P_MAX;
  if (pt<minPt)                       return RCX_PT_MIN;

  // -- track components
  if (track->hsi<minHSI)              return RCX_MISSING_SEGMENTS;
  if (track->hin<minHIN)              return RCX_MISSING_SEGMENTS;
  if (track->hou<minHOU)              return RCX_MISSING_SEGMENTS;
  if (track->hin+track->hou<minHIO)   return RCX_MISSING_SEGMENTS;
  if (track->hri<minHRI)              return RCX_MISSING_SEGMENTS;
  if (track->hca<minHCA)              return RCX_MISSING_SEGMENTS;
  if (track->hmu<minHMU)              return RCX_MISSING_SEGMENTS;
  if (track->hca+track->hmu<minHCAMU) return RCX_MISSING_SEGMENTS;
  if (minHRIIN>0 && track->hin<=0 &&
      track->cer<0.01)                return RCX_MISSING_SEGMENTS; // require either ring or itr
  if (minHRIOU>0 && track->hou<=0 &&
      track->cer<0.01)                return RCX_MISSING_SEGMENTS; // require either ring or otr

  // -- RICH
  if (track->cer>0.01) {
    if (track->cer<minRichRadius)                             return RCX_RICH_MIN_RAD;
    if (track->cer>maxRichRadius)                             return RCX_RICH_MAX_RAD;
  }
  if (track->lre>maxElectronR2P2  || track->lre<minLRE)       return RCX_RICH_ID;
  if (track->lrmu>maxMuonR2P2     || track->lrmu<minLRMU)     return RCX_RICH_ID;
  if (track->lrpi>maxPionR2P2     || track->lrpi<minLRPI)     return RCX_RICH_ID;
  if (track->lrk>maxKaonR2P2      || track->lrk<minLRK)       return RCX_RICH_ID;
  if (track->lrp>maxProtonR2P2    || track->lrp<minLRP)       return RCX_RICH_ID;        
  if (track->lre>maxLRE)                                      return RCX_RICH_ID; // exclude bands
  if (track->lrmu>maxLRMU)                                    return RCX_RICH_ID; // exclude bands
  if (track->lrpi>maxLRPI)                                    return RCX_RICH_ID; // exclude bands
  if (track->lrk>maxLRK)                                      return RCX_RICH_ID; // exclude bands
  if (track->lrp>maxLRP)                                      return RCX_RICH_ID; // exclude bands
  if (track->lre+track->lrmu+track->lrpi<minLREMUPI)          return RCX_RICH_ID; // pi-k separation

  if (pionRISEorRITER && track.index()<=ArteTable<RTRI>::size()) {
    bool rite=false;
    bool rise=false;
    ArtePointer<RTRI> rtri(track.index());
    if (rtri->ltpi>0)                        rise=true;
    if (rtri->lspi+rtri->lsmu+rtri->lse>0.5) rite=true;
  }
  if (kaonRISEorRITER && track.index()<=ArteTable<RTRI>::size()) {
    bool rite=false;
    bool rise=false;
    ArtePointer<RTRI> rtri(track.index());
    if (rtri->ltk>0)                         rise=true;
    if (rtri->lsk>0.1 && rtri->lspi<1e-2)    rite=true;
  }

  // -- ECAL
  float eop=0;
  if (track->hca && track->rccl.begin()!=track->rccl.end()) {
    ArtePointer<RCCL> ecal=(*track->rccl.begin());
    eop=ecal->esum*fabs(track->pf);
    if (ecal->wid<minEcalWid)                                   return RCX_ECAL_MIN_WID;
    if (ecal->esum>maxEcalEnergy)                               return RCX_ECAL_MAX_E;
    if (eop<minEoP)                                             return RCX_EOP_MIN;
    if (eop>maxEoP)                                             return RCX_EOP_MAX;
  }

  // -- electron selection cuts
  if (eDora) {
    if (track->hca<1)                                           return RCX_E_SELECT;
    if (eop<.7 || eop>1.3)                                      return RCX_E_SELECT;
    if (track->hin+track->hou<1 &&
	track->cer<(ArtePointer<RITH>(1)->rad0-.003) &&
	kinema::findBremsstrahlung(track)<1)                    return RCX_E_SELECT;
  }

  if (muKen) {
    //if (track->hmu<1)                                           return RCX_MU_SELECT;
    // ...
  }

  // -- MC
  if (track->mtra.begin()!=track->mtra.end()) {
    ArtePointer<MTRA> mtra=track->mtra.begin();
    if (lund &&  mtra->topp->lund!=lund)                        return RCX_MC_CUT;
    if (mother && mtra->btra->topp->lund!=mother)               return RCX_MC_CUT;
  }


  return RCX_OK;
}
