//--*-C++-*--
//
//  Description:
//    Fill track list from RTRA
//
//
//  Author:  Denis Dujmic
//
//  Created: Apr 04 2001
//
//  Modifications:
// 2001 May 17, DD, Gaussian smearing added: rndgauss
//
#include "arte/arte.hh"
#include "rise/RObject.h"
#include "rise/RAttribute.h"
#include "reco/rcXkinema.h"
#include "rich/PhiLambda.h"
#include "rise/RErrorMsg.h"
#include "rise/RAttribute.h"

int  trSelect( vector< ArtePointer<RTRA> > &rtra );



extern "C" void grndm_(float *dummy, int *n);
/**
   Create random number for gaussian smearing
 */
float rndgauss(float mu, float sigma) {
  static bool  cache=false;
  static float x[2];
  float  ret=0;
  int    n=2;
  if (!cache) {
    float r2=0;
    do {
      grndm_(x,&n);
      x[0]=2*x[0]-1;
      x[1]=2*x[1]-1;
      r2=x[0]*x[0]+x[1]*x[1];
    } while (r2>=1 || r2==0);
    float fac=sqrt(-2*log(r2)/r2);
    x[0] *= fac;
    x[1] *= fac;
    ret = x[0];
    cache=true;
  } else {
    ret = x[1];
    cache=false;
  }
  return ret*sigma+mu;
}


/**
   Copy RTRA table to list of tracks
*/
void RCopyRTRA() {

  // .. clear old list
  RichObject<RTrack>::list.clear();

  // ... use M. Schmelling's function to select unique VDS, main tracker segment
  vector<ArtePointer<RTRA> > Rtra;
  trSelect(Rtra);
  if (RAttribute::verb) cout<<"RCopyRTRA: trSelect size = "<<Rtra.size()<<endl;

  // ... copy list
  RTrack rtrack;
  float  zmin=850, zmax=1100;
  for (vector<ArtePointer<RTRA> >::iterator irtra=Rtra.begin();
       irtra!=Rtra.end(); ++irtra) {
    ArteRelationIterator<RPNT> irpnt=(*irtra)->rpnt.begin();
    for (; irpnt!=(*irtra)->rpnt.end(); ++irpnt) {
      if ((*irpnt)->zf>zmin && (*irpnt)->zf<zmax) break;
    }
    ArtePointer<RPNT> rpnt;
    if (irpnt!=(*irtra)->rpnt.end()) {
      // ... RPNT booked
      rpnt=(*irpnt);
      if (RAttribute::verb) cout<<"RCopyRTRA: default RPNT"<<endl;
    }
    else {
      // ... swim and book private RPNT
      kinema::swim(*irtra, 900);
      rpnt=(*(*irtra)->rpnt.begin());
      if (RAttribute::verb) cout<<"RCopyRTRA: private RPNT"<<endl;
    }

    rtrack.index = (*irtra).index();
    rtrack.point = rpnt.index();
    rtrack.x     = rpnt->xf;
    rtrack.y     = rpnt->yf;
    rtrack.z     = rpnt->zf;
    rtrack.tx    = rpnt->txf;
    rtrack.ty    = rpnt->tyf;
    rtrack.X     = RICH_LambdaPhi->phi   (rpnt->txf,rpnt->tyf);
    rtrack.Y     = RICH_LambdaPhi->lambda(rpnt->txf,rpnt->tyf);
    rtrack.ip    = rpnt->pf;
    for (int i=0; i<15; i++) rtrack.cov[i]=rpnt->cvf[i];
    if ((*irtra)->mtra.begin()!=(*irtra)->mtra.end())  
      rtrack.mctrack=(*(*irtra)->mtra.begin()).index();
    if (ArteTable<MTRA>::size() && (RAttribute::smearx>1e-5||RAttribute::smeary>1e-5)) {
      /*
	additional smearing, this is usefull only in MC
	tests to check ring quality improvement when seeds
	are bad, can be set via kumac
      */
      float stx = rndgauss(0, RAttribute::smearx);
      float sty = rndgauss(0, RAttribute::smeary);
      rtrack.X  += stx;
      rtrack.Y  += sty;
      rtrack.tx += stx;
      rtrack.ty += sty;
      rpnt->txf += stx;
      rpnt->tyf += sty;
      rtrack.cov[5] += stx*stx;
      rtrack.cov[9] += sty*sty;
    }
    RichObject<RTrack>::list.push_back(rtrack);      
  }
  if (!RichObject<RTrack>::list.size()) {
    RErrorMsg err("Event without RICH seeds",-1);
    throw err;
  }
}
