//--*-C++-*--
//
//  Description:
//    Fill ARTE tables from track list
//
//
//  Author:  Denis Dujmic
//
//  Created: Apr 04 2001
//
//  Modifications:
// 28 May 2001 (DD) Flag filled, some comments added
//
//
#include "arte/arte.hh"
#include "rise/RObject.h"
#include "rise/RRingFit.h"
#include "arte/RSEG_cmp.hh"
#include "rich/PhiLambda.h"

/**
   Fill RTRA, RSEG and book relations


   RSEG:
    rseg.xf   = phi
    rseg.yf   = lambda
    rseg.zf   = likelihood
    rseg.txf  = tx
    rseg.tyf  = ty
    rseg.pf   = ?
    rseg.cvf[15] = ?
    rseg.time = radius
    rseg.dtim = mass of best ringfit hypothesis
    rseg.chi  = chi2
    rseg.cmp  = RICH component id
    rseg.fit  = 12 for seeded rings, 13 for conversions
    rseg.xe   = Dh
    rseg.ye   = Rbs
    rseg.ze   = signal photons;
    rseg.txe  = Fs acceptance
    rseg.tye  = ?
    rseg.pe   = ?
    rseg.cve[15] = ?
    rseg.nhit = number of hits in ROI;
    rseg.npar = 3;

   RTRA:
    rtra.txr  = tx
    rtra.tyr  = ty
    rtra.flr  = the truth table, see definition in RRingFit.C
    rtra.cer  = radius
    rtra.lre ... lrp = likelihoods
    rtra.crich= 0 to differ from s/a rings
*/
void RFillARTE() {
  int verb=0;

  for (vector<RRing>::iterator  iring=RichObject<RRing>::list.begin();
       iring!=RichObject<RRing>::list.end(); ++iring) {
    if (!iring->accept) continue;

    // ... create RSEG
    RSEG rseg;
    memset(&rseg, 0x0, sizeof(RSEG));
    rseg.xf   = iring->X;
    rseg.yf   = iring->Y;
    rseg.zf   = iring->L;
    rseg.txf  = RICH_LambdaPhi->tx(iring->X, iring->Y);
    rseg.tyf  = RICH_LambdaPhi->ty(iring->X, iring->Y);
    rseg.time = iring->radius;
    rseg.dtim = iring->mass;
    rseg.chi  = iring->chi2;
    rseg.cmp  = Rsegc::rich;
    rseg.fit  = 12;
    rseg.xe   = iring->Dh;
    rseg.ye   = iring->Rbs;
    rseg.ze   = iring->S;
    rseg.txe  = iring->Fs;
    rseg.tye  = 0;
    rseg.nhit = iring->nROI;
    rseg.npar = 3;
    ArtePointer<RSEG> prseg=ArteTable<RSEG>::push_back(rseg);

    // ... RSEG-HITB
    gRingFit->setRing(*iring);
    gRingFit->selectROI();
    vector<RCorrHit> &hit_list=*gRingFit->getHitList();
    for (vector<RCorrHit>::iterator ihit=hit_list.begin();
	 ihit!=hit_list.end(); ++ihit) {
      if (ihit->assigned) 
	createRelation( prseg, ArtePointer<HITB>(ihit->ihit->index), &RSEG::hitb );
    }

    // ... RSEG-RPNT
    ArtePointer<RPNT> rpnt(iring->itrack->point);
    createRelation( prseg, rpnt, &RSEG::rpnt );

    // ... fill RTRA
    ArtePointer<RTRA> rtra(iring->itrack->index);
    rtra->txr   = iring->X;
    rtra->tyr   = iring->Y;
    rtra->cer   = iring->radius;
    rtra->hri   = (iring->S-int(iring->S)<.5) ? int(iring->S) : int(iring->S)+1;
    rtra->flr   = iring->flag;
    rtra->lre   = iring->lr[0];
    rtra->lrmu  = iring->lr[1];
    rtra->lrpi  = iring->lr[2];
    rtra->lrk   = iring->lr[3];
    rtra->lrp   = iring->lr[4];
    rtra->crich = 0;

    // ... RTRA-RSEG
    createRelation( rtra, prseg, &RTRA::rseg);

  }

}
