//
//               Minuit fitting of rings
//
//  Minimizes likelihood function provided by RingFit::likelihood().
//  Access to current ring parameters (see RRing in RObject.h) or
//  the likelihood function can be done through a global pointer gRingFit:
//         RRing &rring=*gRingFit->getRing(); // get current ring parametrs
//         float gRingFit->likelihood();      // get likelihood for the current ring
//
//
//  Created: Apr 11, 2001
//
//  Modified:
//
//
#include "rise/RObject.h"
#include "rise/RRingFit.h"
#include "rise/RErrorMsg.h"
#include "rise/RAttribute.h"
#ifdef USE_ROOT
#include "TMinuit.h"

void funcL(Int_t &npar, Double_t *gin, Double_t &f, Double_t *par, Int_t iflag) {
  RRing &rring=*gRingFit->getRing();
  rring.radius=par[0];
  rring.X=par[1];
  rring.Y=par[2];
  rring.nIt++;
  f=-(Double_t)gRingFit->likelihood();
}

void RMinuit() {

  RRing &rring=*gRingFit->getRing();
  // ... check if dof>0
  if (rring.nROI<RAttribute::minROI) {
    RErrorMsg err("Not enough hits in ROI",0);
    err.idle();
    return;
  }

  static TMinuit *minuit=0;
  int            ierflg;
  double         p0=0;
  double         n1=-1; 
  if (!minuit) {
    minuit = new TMinuit(3);
    minuit->SetFCN(funcL);
    minuit->mnexcm( "SET PRINT", &n1,1,ierflg);
    minuit->mnexcm(   "SET NOW", &p0,0,ierflg);
    minuit->mnparm(0,  "radius", rring.radiusH, .0001, 0, 0, ierflg);
    minuit->mnparm(1,       "x",       rring.X, .0001, 0, 0, ierflg);
    minuit->mnparm(2,       "y",       rring.Y, .0001, 0, 0, ierflg);
  }
  rring.nIt=0;
  minuit->mnexcm("CALL FCN", &p0 ,0,ierflg);
  minuit->mnexcm(   "MIGRAD",&p0 ,0,ierflg);
  minuit->mnexcm(   "MINOS", &p0 ,0,ierflg);
  double par[3], err[3];
  for (int i=0; i<3; i++) minuit->GetParameter(i, par[i], err[i]);
  rring.radius=par[0];
  rring.X     =par[1];
  rring.Y     =par[2];
}

#endif
