// *****************************************************************************
//
// source:    L3_Vtx.C
//
// type:      source code
//
// created:   06 Dec 1999
//
// author:    Stefan Scharein, Ullrich Schwanke
//            HERA-B Collaboration
//            DESY Zeuthen
//            Platanenallee 6
//            D-15738 Zeuthen
//            Germany
//            email: scharein@ifh.de, schwanke@ifh.de
//
// symbol:    
//
// changes:
//
// *****************************************************************************
#include "L3_Macros.hh"

#include "L3_Vtx.hh"
#include "L3_VTproto.hh"
#include "L3_VTstruct.hh"

// Covariance matrix for initialization with infinite errors.

PCovMatrix L3_Vtx::guess_PCovMatrix = {
  {
    { 10000,     0,     0 },
    {     0, 10000,     0 },
    {     0,     0, 10000 }
  }
};

Point L3_Vtx::guess_Point(0,0,0);     

void L3_Vtx::Vtx_init(Point &Vertex,PCovMatrix &Cov)
{
  #define NN "L3_Vtx::Vtx_init: "
  vtinit_( Vertex,Cov, nr, Status );
  if( Status!=0 )
    {
      P6(Vertex,endl,Cov,endl,nr,endl);
      P3(NN"vtinit_ gives Status = ",Status,endl);  
      P4(NN"not more than ",nvtmax," vertices possible.",endl);  
      nr = 0;
    }
  #undef NN
}

L3_Vtx& L3_Vtx::operator = (const L3_Vtx& a) {   
  Status = a.Status;
  nr = a.nr;
  AssociatedTracks = a.AssociatedTracks;
  return *this;
}

ostream& operator << (ostream& os, const L3_Vtx& v) {
  os << "Vertex : "<< "\t" << v.x()  << "\t" << v.y()
     << "\t" << v.z();
  os << endl;
  os << v.Cov() << endl;
  return os;
}

TCovMatrix err =          // Here err.a[4][4] stands for the error
{{                                // on the momentum measurment. We set it to
  {0,0,0,0,0   },                // some large value
  {0,0,0,0,0   },
  {0,0,0,0,0   },
  {0,0,0,0,0   },
  {0,0,0,0,1000}
}};  

int L3_Vtx::add(const L3_Track& tr,int flag) {
  #define NN "L3_Vtx::add: "
  int n = tr.nr();
  if( n<1 || n>ntrtot )
    {
      P3(NN"track with invalid tracknumber ",n,endl);
      return -1;
    }
  vtadd_(nr,n,Status);
  if( Status!=0 )
    {
      P7(NN"vtadd_ gives Status = ",Status," for track ",tr.nr()," on vertex ",nr,endl);
    }
  else 
    {
      if(flag) associate(tr);
    }
  return Status;
  #undef NN
}

void L3_Vtx::smooth(float chicut) 
{  
  #define NN "L3_Vtx::smooth: "
  do {   
    vtdis();
    float max = -42;
    int imax = -1;
    for(int i=1;i<=nTrack();i++)
      {
	float c = trkchi2(i);
	if( c > max )
	  {
	    max = c;
	    imax = i;
	  }
      }
    if( imax>0 && max>chicut )
      {
	vtkill_(nr,imax,Status);	  	
	if( Status!=0 ) 
	  P7(NN" vtkill_ returns status ",Status," for ",imax,"-th track of vertex ",nr,endl);       
	else {
	  cout << NN << " removed track " << imax << ' ' << max << ">" << chicut << endl;
	  cout << vertex() << endl;
	}
	for(vector<L3_Track>::iterator i = AssociatedTracks.begin();i!=AssociatedTracks.end();i++)
	  {
	    if( i->nr() == vtpntr_.ktr[imax-1][nr-1] )
	      {
		AssociatedTracks.erase(i);
		break;
	      }
	  }
      }
    else
      break;
  } while(1);
  #undef NN
}

int L3_Vtx::vtdis()
{
  #define NN "L3_Vtx::vtdis: "
  vtdis_(nr,Status);
  if( Status!=0 ) P5(NN" vtdis_ returns status ",Status," for vertex ",nr,endl);       
  return Status;
  #undef NN
}

void L3_Vtx::show_track(int l)
{
  cout << "track ID " << l << "  ";
  l--;
  cout << "ax=" << vttrbf_.p[l].ax;
  cout << "  ay=" << vttrbf_.p[l].ay;
  cout << "  x=" << vttrbf_.p[l].bx;
  cout << "  y=" << vttrbf_.p[l].by << endl;
}

void L3_Vtx::showChi2()
{
  #define NN "L3_Vtx::checkChi2: "
  cout << NN << endl;
  for(int i=0;i<nTrack();i++) cout << i+1 << "\t" << vtkalm_.chi2[i][nr-1] << endl;
  cout << "~"NN << endl;
  #undef NN
}

int L3_Vtx::fit(int use_momentum) 
{
  #define NN "L3_Vtx::fit: "
  vtfit_(nr,use_momentum,Status);  //zero means not to use momentum-info.  
  if( Status )
    {
      P5(NN"vtfit_ gives Status = ",Status," on vertex ",nr,endl);  
      if( L3_Parameter::Debug() ){
	cout << "Vertex " << nr << endl;
	vector<int> p = OutgoingTracks();
	cout << p.size() << " tracks: ";
	for(vector<int>::iterator i=p.begin();i!=p.end();i++) cout << *i << ' ';
	cout << endl;
	for(vector<int>::iterator i=p.begin();i!=p.end();i++) 
	  {
	    cout << "Covmatrix of track " << *i << endl;
	    int m = *i-1;
	    cout << vttrbf_.v[m] << endl;
	  }
      }
    }
  return Status;
  #undef NN
}
