*CMZ :          07/11/96  12.55.17  by  T. Oest
*-- Author :    T. Oest   27/09/96

      SUBROUTINE V0FITM(TRK1, TRK2, ZSTART, PAR, VPAR, WPAR, 
     +                 CHI2V0, NDETEC, ITERA, IRET)

************************************************************************
*
* Description: main program for the v0 fit, requires a start value
*              the region for the inhomogeneous field is set in V0FEXT (DZMAG)
*
* Called from:
*
* External references:
*
* Argument in/out description
* TRK1     in     first track
* TRK2     in     second track
* ZSTART   in     start value for Z
* PAR      out    vertex position, track parameter
*                 (z,x,y,tx1,ty1,1/p1,tx2,ty2,1/p2)
* VPAR     out    covariance matrix
* WPAR     out    (VPAR)**-1
* NDETEC   out    number of detector planes between vertex and 
*                 first measurements
* IRET     out    >=0 ok
*
************************************************************************

      IMPLICIT NONE

#include "arte/Arte.fi"
#include "auto/EV.fi"
#include "auto/CON.fi"

      INTEGER          TRK1, TRK2, NDETEC, IRET
      DOUBLE PRECISION ZSTART, PAR(9), VPAR(9,9), WPAR(9,9), CHI2V0

      LOGICAL          COVAR, INIT, FITI
      INTEGER          I, ITERA, MAXIT, IERR
      DOUBLE PRECISION ZV, DCHI2, DCHI2I, CHIOLD
      DOUBLE PRECISION DTX, DTY

      INTEGER          NZ1, NZ2, NZMAX
      PARAMETER        (NZMAX=100)
      INTEGER          NDET1(NZMAX), NDET2(NZMAX)
      DOUBLE PRECISION ZMUSC1(NZMAX), ANG1(NZMAX), ZMUSC2(NZMAX),
     +                 ANG2(NZMAX), ZLIMIT

      DOUBLE PRECISION ZVMIN, ZVMAX

#include "arte/Arte.ff"
#include "auto/EV.ff"
#include "auto/CON.ff"

      DATA MAXIT  /20/
      DATA DCHI2I /0.01/
      DATA ZVMIN / -500.0D0/, ZVMAX /1000.D0/
C-----------------------------------------------------------------
      IRET  = 0
      ITERA = 0
      ZV    = ZSTART
      COVAR = .FALSE.
C-----------------------------------------------------------------
C start vector
C-----------------------------------------------------------------
      CALL V0FITS(TRK1, TRK2, ZSTART, PAR, DTX, DTY, IERR)
c      print*,' start values found ', ierr
c      print*,' xyz     ',sngl(par(1)),sngl(par(2)),sngl(par(3))
c      print*,'tx,ty,p1 ',sngl(par(4)),sngl(par(5)),sngl(par(6))
c      print*,'tx,ty,p2 ',sngl(par(7)),sngl(par(8)),sngl(par(9))
      IF (IERR.NE.0) THEN
c       PRINT*,' ERROR IN V0FITS ',IERR
       IRET = -1
       GOTO 9000
      ENDIF
C-----------------------------------------------------------------
C search for multiple scattering planes
C-----------------------------------------------------------------
      ZLIMIT = ZSTART - 50.D0
      CALL V0MUSC(TRK1, ZLIMIT, NZMAX, NZ1, ZMUSC1, NDET1, ANG1)
      CALL V0MUSC(TRK2, ZLIMIT, NZMAX, NZ2, ZMUSC2, NDET2, ANG2)
C-----------------------------------------------------------------
C loop
C-----------------------------------------------------------------
      INIT = .TRUE.
      CALL V0FITI(INIT, COVAR, TRK1, TRK2, NZ1, ZMUSC1, ANG1,
     +            NZ2, ZMUSC2, ANG2, PAR, VPAR, WPAR,
     +            DCHI2,CHI2V0,IERR)
      IF (IERR.NE.0) THEN
c       print*,' error in v0fiti ',ierr
       IRET = -2
       GOTO 9000
      ENDIF
      INIT = .FALSE.

 100  CONTINUE

      ITERA = ITERA + 1
      CALL V0FITI(INIT, COVAR, TRK1, TRK2, NZ1, ZMUSC1, ANG1,
     +            NZ2, ZMUSC2, ANG2, PAR, VPAR, WPAR,
     +            DCHI2,CHI2V0,IERR)



      IF (IERR.EQ.0) THEN
       FITI = .TRUE.
      ELSE
       FITI = .FALSE.
       CALL V0FITE(INIT, COVAR, TRK1, TRK2, NZ1, ZMUSC1, ANG1,
     +             NZ2, ZMUSC2, ANG2, PAR, VPAR, WPAR,
     +             DCHI2,CHI2V0,IERR)
      ENDIF

      IF (IERR.NE.0) THEN
       IRET = -3
       GOTO 9000
      ENDIF

      IF (PAR(1).LT.ZVMIN .OR. PAR(1).GT.ZVMAX) THEN
       IRET = -4
       GOTO 9000
      ENDIF

c      IF (DCHI2.LT.0.0) THEN
c       PRINT*,' DCHI2 ',ITERA,SNGL(DCHI2)
c      ENDIF
c      print*,'**************************************************'
c      print*,fiti,itera, sngl(dchi2),sngl(chi2v0),sngl(par(1))

      IF (ITERA.GE.2 .AND. CHIOLD.LE.CHI2V0) THEN
       IRET = 2
      ELSE
       CHIOLD = CHI2V0
       IF (ITERA .LT. MAXIT ) THEN
        IF (DCHI2.GT.DCHI2I) GOTO 100
       ELSE
        IRET = 1
       ENDIF

      ENDIF
C-----------------------------------------------------------------
C get covariance matrix
C-----------------------------------------------------------------
      COVAR = .TRUE.

      IF (FITI) THEN
       CALL V0FITI(INIT, COVAR, TRK1, TRK2, NZ1, ZMUSC1, ANG1,
     +             NZ2, ZMUSC2, ANG2, PAR, VPAR, WPAR,
     +             DCHI2,CHI2V0,IERR)
      ELSE
       CALL V0FITE(INIT, COVAR, TRK1, TRK2, NZ1, ZMUSC1, ANG1,
     +             NZ2, ZMUSC2, ANG2, PAR, VPAR, WPAR,
     +             DCHI2,CHI2V0,IERR)
      ENDIF

      IF (IERR.NE.0) THEN
       IRET = -4
      ENDIF
C-----------------------------------------------------------------
C number of detector planes between vertex and first measurements
C-----------------------------------------------------------------
      NDETEC = 0

      DO I=1,NZ1
       IF (FRTRA_ZF(TRK1).GT.ZMUSC1(I) .AND. PAR(1).LT.ZMUSC1(I)) 
     +    NDETEC = NDETEC + NDET1(I)
      ENDDO
      DO I=1,NZ2
       IF (FRTRA_ZF(TRK2).GT.ZMUSC2(I) .AND. PAR(1).LT.ZMUSC2(I)) 
     +    NDETEC = NDETEC + NDET2(I)
      ENDDO
C-----------------------------------------------------------------
C  the end
C-----------------------------------------------------------------
 9000 END

************************************************************************
* End of V0FIT
************************************************************************





