*CMZ :          20/05/96  12.58.34  by  Unknown
*-- Author :   T.Lohse
c
c------------------------------------------------------------
c
c       subroutine:   VTMASS(iv,par,cov,chi2m,flag)
c
c       input:    iv          =  vertex index         Int
c                 par (in case of re-iteration)       R*8
c
c       output:   par(3*(ntrk+1)) = fit parameters    R*8
c                      1..3       = vertex (x,y,z)
c                      4,5        = slope_x, slope_y (1)
c                      6          = momentum (1)
c                      7,8        = slope_x, slope_y (2)
c                      9          = momentum (2)
c                      etc.
c                 cov             = cov(par)          R*8
c                 chi2m       =  chi**2 of fit        R*8
c                                ndf = #mass constr.
c                 flag        < 0 if fail             Int
c
c       action:   vertex fit for vertex iv with mass constraint
c
c------------------------------------------------------------
c
      subroutine  VTMASS ( iv, par, cov, chi2m, flag )

      implicit none

#include "vt/vtdat.fi"

      integer iv
      double precision 
     1  par(3*ntrk+3), cov(3*ntrk+3,3*ntrk+3)
      double precision 
     1  chi2m
      integer flag

      integer flag1,ifail
      integer n, iter, i, j, k, l, nn, mm, l1, l2, nn1, nn2
      integer ic, it, iic, jic

      double precision 
     1  alp(3*(ntrk+1)), Valp(3*(ntrk+1),3*(ntrk+1))
      double precision 
     1  alpc(3*(ntrk+1)), Valpc(3*(ntrk+1),3*(ntrk+1))
      double precision 
     1  dalpc(3*(ntrk+1))
      double precision 
     1  gam(ncomax,3*(ntrk+1)), Vgam(ncomax,3*(ntrk+1))
      double precision 
     1  gaVga(ncomax,ncomax), galp(ncomax)
      double precision 
     1  VGGVG(ntrk,ncomax)
      double precision 
     1  xn(ntrk), yn(ntrk), zn(ntrk), xnorm(ntrk), erg(ntrk)
      double precision 
     1  ch2old, sum, fact, gnum(ncomax), xmu(ncomax)
      double precision 
     1  SE, Sx, Sy, Sz

      flag = -1
      if (iv.gt.nvtmax)  goto 999
      if ( .not. vtset(iv) )   goto 999
      n = ntr(iv)
      if ( fit(iv) .ne. 2 ) then
        call VTFIT(iv,1,flag1)
        if (flag1.lt.0)  goto 999
      endif
      ic = nconst(iv)
      if (ic.lt.1.or.ic.gt.ncomax)   goto 999

c
c---> check kinematic boundaries
c
      do iic = 1, ic
        sum = 0.
        do  i = 1, n
          if (mask(iv,iic,i)) then
            it = ktr(iv,i)
            sum = sum + xmass(it)
          endif
        enddo
        sum = sum**2 - xmc(iv,iic)**2
        if ( sum .ge. 0. ) then
          flag = -3
          goto 999
        endif
      enddo

c
c---> initialize general state vector
c
      if (fitm(iv).eq.0) then
        alpc(1) = xvs(1,iv)
        alpc(2) = xvs(2,iv)
        alpc(3) = xvs(3,iv)
        j = 4
        do  k = 1, n
          alpc(j)   = qvs(1,iv,k)
          alpc(j+1) = qvs(2,iv,k)
          alpc(j+2) = qvs(3,iv,k)
          j = j + 3
        enddo
      else
        do  k = 1, 3*n+3
          alpc(k) = par(k)
        enddo
      endif

      iter  = 0
      ch2old = -100.

  100 continue
c
c--->  re-do filter and smoother for vertex
c
      if (iter.gt.0 .or. fitm(iv).ne.0) then
        do  i = 1, 3
          xvs(i,iv) = alpc(i)
        enddo
        do  i = 1, n
          nn=3*i
          do  j = 1, 3
            qvs(j,iv,i) = alpc(nn+j)
          enddo
        enddo
        call VTFILT(iv,1,flag1)
        if (flag1.lt.0) then
          flag = -1
          goto 999
        endif
        call VTSMO(iv)
      endif

c
c---> fill general state vector
c
      alp(1) = xvs(1,iv)
      alp(2) = xvs(2,iv)
      alp(3) = xvs(3,iv)
      j = 4
      do  k = 1, n
        alp(j)   = qvs(1,iv,k)
        alp(j+1) = qvs(2,iv,k)
        alp(j+2) = qvs(3,iv,k)
        j = j + 3
      enddo

c
c---> fill general covariance matrix
c
      do  i = 1, 3
        do  j = 1, 3
          Valp(i,j) = CS(i,j,iv)
        enddo
      enddo
      do  k = 1, n
        nn  = 3*k
        do  i = 1, 3
          do  j = 1, 3
            Valp(nn+i,nn+j) = DS(i,j,iv,k)
            Valp(nn+j,nn+i) = DS(i,j,iv,k)
            Valp(nn+i,j)    = ES(i,j,iv,k)
            Valp(j,nn+i)    = ES(i,j,iv,k)
          enddo
        enddo
      enddo

      do  k = 1, n-1
        nn  = 3*k
        do  l = k+1, n
          mm  = 3*l
          do l1 = 1, 3
            do l2 = 1, 3
              sum = 0.d0
              do  i = 1, 3
                sum = sum - ES(l1,i,iv,k)*F(l2,i,iv,l)
              enddo
              Valp(nn+l1,mm+l2) = sum
              Valp(mm+l2,nn+l1) = sum
            enddo
          enddo
        enddo
      enddo

c
c---> solve constrained fit for given vertex
c

      iter = iter + 1
      if ( iter .gt. 100*ic ) then
        flag = -2
        goto 999
      endif

      do  k = 1, n
        it = ktr(iv,k)
        fact = 1./sqrt(1.+alpc(3*k+1)**2+alpc(3*k+2)**2)
        xnorm(k) = fact
        xn(k) = alpc(3*k+1)*fact
        yn(k) = alpc(3*k+2)*fact
        zn(k) = fact
        erg(k) = sqrt( xmass(it)**2 + alpc(3*k+3)**2 )
      enddo

      do  iic = 1, ic
        SE = 0.d0
        Sx = 0.d0
        Sy = 0.d0
        Sz = 0.d0
        do  i = 1, n
          k = 3*i+3
          if ( mask(iv,iic,i)) then
            SE = SE + erg(i)
            Sx = Sx + alpc(k) * xn(i)
            Sy = Sy + alpc(k) * yn(i)
            Sz = Sz + alpc(k) * zn(i)
          endif
        enddo
        galp(iic) = SE**2 - Sx**2 - Sy**2 - Sz**2
     1                    - xmc(iv,iic)**2

        gam(iic,1) = 0.d0
        gam(iic,2) = 0.d0
        gam(iic,3) = 0.d0
        do  i = 1, n
          nn  = 3*i+3
          nn1 = nn-2
          nn2 = nn-1
          gam(iic,nn1) = 0.d0
          gam(iic,nn2) = 0.d0
          gam(iic,nn)  = 0.d0
          if (mask(iv,iic,i)) then
            gam(iic,nn)  = 2.d0*( alpc(nn)/erg(i)*SE
     1           - xn(i)*Sx - yn(i)*Sy - zn(i)*Sz )
            gam(iic,nn1) = -2.d0*alpc(nn)*zn(i)*
     1         ( (1.d0-xn(i)**2)*Sx
     1          - xn(i)*(yn(i)*Sy+zn(i)*Sz) )
            gam(iic,nn2) = -2.d0*alpc(nn)*zn(i)*
     1         ( (1.d0-yn(i)**2)*Sy
     1          - yn(i)*(xn(i)*Sx+zn(i)*Sz) )
          endif
        enddo

        do  i = 1, 3*n+3
          Vgam(iic,i) = 0.d0
          do  j = 1, 3*n+3
            Vgam(iic,i) = Vgam(iic,i) + Valp(i,j)*gam(iic,j)
          enddo
        enddo

      enddo

      do  iic = 1, ic
        do jic = 1, iic
          sum = 0.d0
          do  i = 1, 3*(n+1)
            sum = sum + gam(iic,i)*Vgam(jic,i)
          enddo
          gaVga(iic,jic) = sum
          gaVga(jic,iic) = sum
        enddo
      enddo
      call DSINV(ic,gaVga,ncomax,ifail)
      if (ifail.eq.-1) then
        flag = -1
        goto 999
      endif

      do  iic = 1, ic
        gnum(iic) = galp(iic)
        do  i = 1, 3*(n+1)
          gnum(iic) = gnum(iic) + gam(iic,i)*(alp(i)-alpc(i))
        enddo
      enddo

      do  iic = 1, ic
        xmu(iic) = 0.d0
        do  jic = 1, ic
          xmu(iic) = xmu(iic) + gaVga(iic,jic)*gnum(jic)
        enddo
      enddo

      do  i = 1, 3*(n+1)
        dalpc(i) = alpc(i)
        alpc(i)  = alp(i)
        do iic = 1, ic
          alpc(i) = alpc(i) - Vgam(iic,i)*xmu(iic)
        enddo
        dalpc(i) = alpc(i) - dalpc(i)
      enddo

      chi2m = 0.d0
      do  iic = 1, ic
        chi2m = chi2m + gnum(iic)*xmu(iic)
      enddo

      if ( abs(chi2m+chi2(iv,n)-ch2old) .gt. 0.00001 ) then
        ch2old = chi2m+chi2(iv,n)
        if (fitm(iv).eq.0) goto 100
      endif

      do  i = 1, 3*(n+1)
        do  iic = 1, ic
          sum = 0.d0
          do  jic = 1, ic
            sum = sum + Vgam(jic,i)*gaVga(jic,iic)
          enddo
          VGGVG(i,iic) = sum
        enddo
      enddo

      do  i = 1, 3*(n+1)
        do  j = 1, i
          sum = Valp(i,j)
          do  iic = 1, ic
            sum = sum - VGGVG(i,iic)*Vgam(iic,j)
          enddo
          Valpc(i,j) = sum
          Valpc(j,i) = sum
        enddo
      enddo

c---> set output parameters
      do  i = 1, 3*n+3
        par(i) = alpc(i)
        do  j = 1, 3*n+3
          cov(i,j) = Valpc(i,j)
        enddo
      enddo

      fitm(iv) = 1
      flag = 0

      return

  999 continue

      fit(iv) = 0
      call VTFIT(iv,1,flag1)

      return
      end
