*CMZ :          20/05/96  12.58.34  by  Unknown
*-- Author :   T.Lohse
c
c------------------------------------------------------------
c
c       subroutine:   VTSUMP(np,parn,covn,par,cov,flag)
c
c       input:    np           =  no momenta to be added   Int
c                 parn(3*np+3) =  vertex+momenta           R*8
c                        1,2,3 =  vertex
c                          4,5 =  slope_x, slope_y (1)
c                            6 =  momentum (1)
c                          7,8 =  slope_x, slope_y (2)
c                            9 =  momentum (2)
c                        etc...
c                 covn         = cov(parn)                 R*8
c
c       output:   par(5)       =  total momentum           R*8
c                          1,2 = slope_x, slope_y
c                          3,4 = impact_x, impact_y
c                            5 = momentum
c                 cov(5,5)   = cov(par)                    R*8
c                 flag  < 0 if fail                        Int
c
c       action:   Add correlated momenta
c
c------------------------------------------------------------
c
      subroutine  VTSUMP ( np, parn, covn, par, cov, flag )

      implicit none

#include "vt/vtdat.fi"
      integer i3
      parameter (i3=3*ntrk+3)

      integer np, flag
      double precision 
     1  parn(i3), covn(i3,i3), par(5), cov(5,5)

      double precision 
     1  ps(3)
      double precision 
     1  jac(5,i3)
      double precision 
     1  fact, sum

      integer i, j, k, l, n3

      double precision 
     1  xn(ntrk), yn(ntrk), zn(ntrk)

      flag = -1
      if (np.gt.ntrk) return

      n3 = 3*np+3
c
c---> fill unit vectors in direction of momenta
c
      do  k = 1, np
        i = 3*k
        fact = 1./sqrt(1.+parn(i+1)**2+parn(i+2)**2)
        xn(k) = parn(i+1)*fact
        yn(k) = parn(i+2)*fact
        zn(k) = fact
      enddo

c
c---> sum momenta
c
      ps(1) = 0.d0
      ps(2) = 0.d0
      ps(3) = 0.d0
      do  k = 1, np
        i = 3*k+3
        ps(1) = ps(1) + xn(k)*parn(i)
        ps(2) = ps(2) + yn(k)*parn(i)
        ps(3) = ps(3) + zn(k)*parn(i)
      enddo
      par(1) = ps(1)/ps(3)
      par(2) = ps(2)/ps(3)
      par(3) = parn(1) - parn(3)*par(1)
      par(4) = parn(2) - parn(3)*par(2)
      par(5) = sqrt(ps(1)**2+ps(2)**2+ps(3)**2)
      fact  = 1./par(5)
      ps(1) = ps(1)*fact
      ps(2) = ps(2)*fact
      ps(3) = ps(3)*fact

c
c---> get Jacobian
c
      do  i = 1, 5
        do  j = 1, 3
          jac(i,j) = 0.d0
        enddo
      enddo
      jac(3,1) = 1.d0
      jac(3,3) = -par(1)
      jac(4,2) = 1.d0
      jac(4,3) = -par(2)
      do  k = 1, np
        i = 3*k+1
        fact = parn(i+2)*zn(k)/(par(5)*ps(3))
        jac(1,i) = fact*(1.d0-xn(k)**2+par(1)*xn(k)*zn(k))
        jac(2,i) = fact*xn(k)*(par(2)*zn(k)-yn(k))
        jac(3,i) = -parn(3)*jac(1,i)
        jac(4,i) = -parn(3)*jac(2,i)
        i = i+1
        jac(1,i) = fact*yn(k)*(par(1)*zn(k)-xn(k))
        jac(2,i) = fact*(1.d0-yn(k)**2+par(2)*yn(k)*zn(k))
        jac(3,i) = -parn(3)*jac(1,i)
        jac(4,i) = -parn(3)*jac(2,i)
        i = i-1
        fact = parn(i+2)*zn(k)
        jac(5,i) = fact*(ps(1)*(1.d0-xn(k)**2)-ps(2)*xn(k)*yn(k)
     1                                        -ps(3)*xn(k)*zn(k))
        i = i+1
        jac(5,i) = fact*(ps(2)*(1.d0-yn(k)**2)-ps(1)*xn(k)*yn(k)
     1                                        -ps(3)*yn(k)*zn(k))
        i = i+1
        fact = 1.d0/(par(5)*ps(3))
        jac(1,i) = fact*(xn(k)-par(1)*zn(k))
        jac(2,i) = fact*(yn(k)-par(2)*zn(k))
        jac(3,i) = -parn(3)*jac(1,i)
        jac(4,i) = -parn(3)*jac(2,i)
        jac(5,i) = ps(1)*xn(k) + ps(2)*yn(k) + ps(3)*zn(k)
      enddo
c
c---> get covariance matrix for momentum sum
c
      do  i = 1, 5
        do  j = i, 5
          cov(i,j) = 0.d0
          do  k = 1, n3
            sum = 0.d0
            do  l = 1, n3
              sum = sum + covn(k,l)*jac(j,l)
            enddo
            cov(i,j) = cov(i,j) + jac(i,k)*sum
          enddo
          cov(j,i) = cov(i,j)
        enddo
      enddo

      flag = 0

      return
      end
