#ifndef __OTR_DECODE_H__
#define __OTR_DECODE_H__

#ifdef __cplusplus
extern "C" 
{
#endif
#include <stdio.h>
#include "component.h"
#include "grr.h"
#include "etb.h"

#define MAXASD 7300
#define MAXFED 65535
#define COMPID 0x400

#define MAXTDC 960
  
typedef struct{
  unsigned char  dtime;   
  unsigned char  tzero;
  unsigned char  tzerr;
  unsigned char  status;
  unsigned short GEDE;          /* GEDE id                        */
  unsigned char  cell;          /* SubLayer + 4*(Cell in GEDE)    */  
  unsigned short ASD;           /* ASD8 index                     */
  unsigned char  wire;          /* Wire on ASD                    */
  unsigned char  fHVConnector ; /* HV connector in HV group */ 
} OTrWire;
  
typedef struct{
  unsigned char   SuperLayer;
  unsigned char   Layer;
  unsigned char   Sector;
  unsigned char   Module;
  unsigned char   ftb_ASD;
  unsigned char   TDC_Connector;
  unsigned short  TDC_FED;
  char   hwc[16];
  OTrWire   wire[16];
} OTrASD8;

typedef struct{
  unsigned short ASD[8];   /*Pointer to ASD8s               */
  unsigned char  VME_num;
  unsigned char  Sharc_Board;
  unsigned char  Sharc_Connector;
  unsigned char  status;
  unsigned short FED;
  unsigned char  crate;
  unsigned char  slot;
} OTrTDC;
  
typedef struct {
  char fHVGroupName[17] ;
  unsigned char fHVGroupCable ;
} OTrHVGroup ;

typedef struct{
  unsigned char  NLayer;
  unsigned char  Nasd[7][13];
  unsigned char  FirstASD[7][13];
  unsigned short pos[7][13];           /*pointer to 1-st ASD in sector  */
  unsigned short *Line;                /*buffer with pointers to ASD    */
} OTrSuperLayer;

typedef struct{
  OTrASD8           *ASD8s[MAXASD];       /*ASD8 information               */
  OTrTDC            *TDCs[MAXTDC];        /*TDC information                */  
  unsigned short    FEDs[MAXFED];         /*Pointer FED->TDCs              */
  OTrSuperLayer     Slayers[32];          /*Pointer SL,Lay,Sect,ASD->FED,ch*/
                                          /*0-15 '-',16-31 '+'side         */  
} OTR_Decode;
  
  enum OTrModuleType { kOTrMT_SingleMinus=0, kOTrMT_SinglePlus=1,
		       kOTrMT_DoubleMinus=2, kOTrMT_DoublePlus=3,
		       kOTrMT_SingleSpecial, kOTrMT_DoubleSpecial} ;
  
/************************************************************************/
/* OTR_Decode *OTr_Decode_init(                                         */
/*     mode               -- 0 for ascii,1 for db data sourse           */
/*     Tables_filename    -- File with list of cable  tables,           */
/*     TDC_table_filename -- "crate+board" to FED correspondence table  */
/*     );                                                               */
/* Return:      Pointer to beginning of table or NULL if error.         */
/* Description: Fills table for fast decoding.                          */
/************************************************************************/
OTR_Decode *OTr_Decode_init(int mode,const char *Tables_filename,
                                   const char *TDC_table_filename);

/************************************************************************/
/* int OTr_decode_TDC_to_ASD(                                           */
/*         OTR_Decode *OTR -- Pointer returned by OTr_Decode_init,      */
/*         unsigned short FED     -- TDC board ID,                      */
/*         unsigned char channel  -- Channel,                           */
/*         char *SL               -- SuperLayer,                        */
/*         char *Layer            -- Layer,                             */
/*         char *Sector           -- Sector,                            */
/*         char *ASD              -- ASD,                               */
/*     );                                                               */
/* Return:      1 or 0 if error.                                        */
/* Description: Convert (FED,Channel)->(SL,Layer,Sector,ASD,Wire).      */
/************************************************************************/
int OTr_Decode_TDC_to_ASD(OTR_Decode *OTR,unsigned short FED,
			  unsigned char channel,char *SL,char *Layer,
			  char *Sector,char *ASD);

/************************************************************************/
/* int OTr_Decode_ASD_to_TDC(                                           */
/*         OTR_Decode *Table -- Pointer returned by OTr_decode_TDC_init,*/
/*         unsigned char SL     -- SuperLayer,                          */
/*         unsigned char Layer  -- Layer,                               */
/*         unsigned char Sector -- Sector namber,                       */
/*         unsigned char ASD    -- ASD in sector,                       */
/*         unsigned short *FED   -- FED                                 */
/*         unsigned char *Conn  -- Connector,                           */
/*     );                                                               */
/*                                                                      */
/* Return:      1 or 0 if error.                                        */
/* Description: Convert (SL,Layer,Sector,ASD) ot (FED,Connector).       */
/************************************************************************/
int OTr_Decode_ASD_to_TDC(OTR_Decode *OTR,unsigned char SL,
			  unsigned char Layer,unsigned char Sector,
			  unsigned char ASD,unsigned short *FED,
			  unsigned char *Conn);

/************************************************************************/
/* int OTr_decode_TDC_to_Sharc(                                         */
/*         OTR_Decode *Table -- Pointer returned by OTr_decode_TDC_init,*/
/*         unsigned short FED     -- FED                                */
/*         unsigned char Channel  -- TDC channel (0..127)               */
/*         unsigned char *VME_nam -- hb-vme"VME_nam"                    */
/*         unsigned char *Board_ID-- Sharc board ID                     */
/*         unsigned short *offset -- ((sharc_link(1..30))-1)*128+       */
/*                                   ((TDC_connector(1..8))-1)*16+      */
/*                                   (channel_on_ASD(1..16)-1           */
/* Return:      1 or 0 if error.                                        */
/************************************************************************/
int OTr_Decode_TDC_to_Sharc(OTR_Decode *OTR,unsigned short FED,
			    unsigned char Channel,unsigned char *VME_num,
			    unsigned char *Board_ID,unsigned short *offset);
 
/************************************************************************/
/*  int OTr_get_event(                                                  */
/*      OTR_Decode *OTR -- Pointer to OTR_Decode struct.                */
/*      int *Nhits      -- Number of hits in event.                     */
/*      Wire *wires[]   -- Array of pointers to Wires.                  */
/*  );                                                                  */
/*  return: 1 or 0 in case of error.                                    */
/************************************************************************/
int OTr_get_event(OTR_Decode *OTR,int *Nhits,OTrWire **wires[]);

int OTr_fill_event(OTR_Decode *OTR);

/* some internal functions */
int OTr_Decode_TDC_to_ASDindex(OTR_Decode *OTR,unsigned short FED,
			      unsigned char channel,unsigned short *Line);
int OTr_Decode_ASD_to_ASDindex(OTR_Decode *OTR,unsigned char SL,
			       unsigned char Layer,unsigned char Sector,
			       unsigned char ASD,unsigned short *Line);
int OTr_Decode_TDCLine(OTR_Decode *OTR,unsigned short FED,
		       unsigned short *Line);
int OTr_Convert_GEDE_to_ASD(unsigned short GEDE, unsigned char cell,char *SL,
			   char *Layer,char *Sector,char *ASD,char *Wire);
int OTr_Convert_ASD_to_GEDE(int dbl, int w1, int ch, int plus, int *wires, 
			    int *mono, int *wire);

int OTr_Decode_ASD_to_GEDE(int moduletype, int modchanid, int *wires, 
			   int *mono, int *wire);

#ifdef __cplusplus
}
#endif

#endif
